@@ -1,10 +1,11 @@
-;;; hact.el --- GNU Hyperbole button action handling
+;;; hact.el --- GNU Hyperbole button action handling  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    18-Sep-91 at 02:57:09
+;; Last-Mod:     30-Jan-22 at 03:07:43 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -16,7 +17,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hhist hpath set)))
+(eval-and-compile (mapc #'require '(hhist set)))
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -36,9 +37,22 @@ e.g. to inhibit actions.")
 ;;; ========================================================================
 
 (defvar symtable:category-plist nil
-  "Holds a property list of Hyperbole type category symbols ('actypes or 'ibtypes) and their associated symtables.")
+  "Property list mapping Hyperbole type category symbols to symtables.
+The type categories are either 'actypes or 'ibtypes.")
 
-(defun  symtable:operate (operation symbol-or-name symtable)
+(defsubst symtable:hash-table (symtable)
+  "Return the hash-table containing symbol names and values from SYMTABLE."
+  (plist-get symtable 'hash-table))
+
+(defsubst symtable:name (symtable)
+  "Return the name of SYMTABLE as a string."
+  (plist-get symtable 'name))
+
+(defsubst symtable:select (type-category)
+  "Inline the return of the symtable for TYPE-CATEGORY, one of 'actypes or 'ibtypes."
+  (plist-get symtable:category-plist type-category))
+
+(defun    symtable:operate (operation symbol-or-name symtable)
   "Call hash-table function OPERATION with Hyperbole SYMBOL-OR-NAME as key upon SYMTABLE.
 Trigger an error if SYMBOL-OR-NAME cannot be mapped to an existing Elisp
 symbol or if SYMTABLE is invalid."
@@ -77,10 +91,6 @@ symbol or if SYMTABLE is invalid."
        (gethash def-name  hash-table))
       (_ (error "(symtable:operate): Invalid operation request: %s" operation)))))
 
-(defsubst symtable:select (type-category)
-  "Inline the return of the symtable for TYPE-CATEGORY, one of 'actypes or 'ibtypes."
-  (plist-get symtable:category-plist type-category))
-
 (defun    symtable:create (name size)
   "Create and return a new Hyperbole type symbol table with NAME and SIZE.
 Also add it under the symbol for its NAME in `symtable:category-plist'."
@@ -89,14 +99,6 @@ Also add it under the symbol for its NAME in `symtable:category-plist'."
     (setq symtable:category-plist (plist-put symtable:category-plist (intern name) symtable))
     symtable))
 
-(defsubst symtable:hash-table (symtable)
-  "Return the hash-table containing symbol names and values from SYMTABLE."
-  (plist-get symtable 'hash-table))
-
-(defsubst symtable:name (symtable)
-  "Return the name of SYMTABLE as a string."
-  (plist-get symtable 'name))
-
 (defvar   symtable:actypes (symtable:create "actypes" 97)
   "Symbol table (hash table) of Hyperbole action type symbols.
 For each actype, there are two entries whose keys are strings: one
@@ -111,11 +113,13 @@ keys is the Elisp symbol for the type, which includes the prefix.")
 
 (defsubst symtable:actype-p (symbol-or-name)
   "Return the Elisp symbol given by SYMBOL-OR-NAME if it is a Hyperbole action type name, else nil."
-  (symtable:get symbol-or-name symtable:actypes))
+  (when (or (symbolp symbol-or-name) (stringp symbol-or-name))
+    (symtable:get symbol-or-name symtable:actypes)))
 
 (defsubst symtable:ibtype-p (symbol-or-name)
   "Return the Elisp symbol given by SYMBOL-OR-NAME if it is a Hyperbole implicit button type name, else nil."
-  (symtable:get symbol-or-name symtable:ibtypes))
+  (when (or (symbolp symbol-or-name) (stringp symbol-or-name))
+    (symtable:get symbol-or-name symtable:ibtypes)))
 
 (defun    symtable:add (symbol-or-name symtable)
   "Add Hyperbole SYMBOL-OR-NAME to existing SYMTABLE.
@@ -123,7 +127,7 @@ Return the Elisp symbol for SYMBOL-OR-NAME.
 Caller must ensure SYMBOL-OR-NAME is a symbol or string."
   (symtable:operate #'puthash symbol-or-name symtable))
 
-(defalias 'symtable:delete 'symtable:remove)
+(defalias 'symtable:delete #'symtable:remove)
 
 (defun    symtable:get (symbol-or-name symtable)
   "Return the Elisp symbol given by Hyperbole SYMBOL-OR-NAME if it is in existing SYMTABLE, else nil.
@@ -160,7 +164,11 @@ Use `eq' for comparison."
 	 (new-set (set:add elt set)))
     (and new-set (put symbol property new-set))))
 
-(defalias 'symset:delete 'symset:remove)
+(defun    symset:clear (symbol)
+  "Set SYMBOL's symset to nil."
+  (setf (symbol-plist symbol) nil))
+
+(defalias 'symset:delete #'symset:remove)
 
 (defun    symset:get (symbol property)
   "Return SYMBOL's PROPERTY set."
@@ -207,7 +215,7 @@ Return the new function symbol derived from TYPE."
 	 (action (nconc (list 'defun sym params doc) body)))
     `(progn
        ,action
-       (setplist ',sym '(definition-name ,type @,property-list))
+       (setplist ',sym '(definition-name ,type ,@property-list))
        (symset:add ',type ',type-category 'symbols)
        (run-hooks 'htype-create-hook)
        ',sym)))
@@ -241,7 +249,7 @@ Return the Hyperbole symbol for the TYPE if it existed, else nil."
 (defun    htype:names (type-category &optional sym)
   "Return a list of the current definition names for TYPE-CATEGORY in priority order.
 Definition names do not contain the category prefix.
-TYPE-CATEGORY should be 'actypes, 'ibtypes or nil for all.
+TYPE-CATEGORY should be `actypes', `ibtypes' or nil for all.
 When optional SYM is given, returns the name for that symbol only, if any."
   (let ((types (symset:get type-category 'symbols))
 	(sym-name (when sym (symbol-name sym))))
@@ -353,9 +361,11 @@ Autoloads action function if need be to get the parameter list."
     (setq action (hypb:indirect-function action)))
   (cond ((null action) nil)
 	((listp action)
-	 (if (eq (car action) 'autoload)
-	     (error "(action:params): Autoload not supported: %s" action)
-	   (car (cdr action))))
+	 (cond ((eq (car action) 'closure)
+		(nth 2 action))
+	       ((eq (car action) 'autoload)
+		(error "(action:params): Autoload not supported: %s" action))
+	       (t (car (cdr action)))))
 	((hypb:emacs-byte-code-p action)
 	 (if (fboundp 'compiled-function-arglist)
 	     (compiled-function-arglist action)
@@ -370,24 +380,6 @@ Autoloads action function if need be to get the parameter list."
 			  nil param))
 	      (action:params action))))
 
-(defun action:path-args-abs (args-list &optional default-dirs)
-  "Return any paths in ARGS-LIST made absolute.
-Uses optional DEFAULT-DIRS or `default-directory'.
-Other arguments are returned unchanged."
-  (mapcar (lambda (arg) (hpath:absolute-to arg default-dirs))
-	  args-list))
-
-(defun action:path-args-rel (args-list)
-  "Return any paths in ARGS-LIST below button source loc directory made relative.
-Other paths are simply expanded.  Non-path arguments are returned unchanged."
-  (let ((loc (hattr:get 'hbut:current 'loc)))
-    (mapcar (lambda (arg)
-	      (hpath:relative-to arg (if (stringp loc)
-					 (file-name-directory loc)
-				       (buffer-local-value 'default-directory loc))))
-	    args-list)))
-
-
 ;;; ========================================================================
 ;;; action type class, actype
 ;;; ========================================================================
@@ -398,7 +390,7 @@ The value of `hrule:action' determines what effect this has.
 Alternatively act as a no-op when testing implicit button type contexts.
 First arg may be a symbol or symbol name for either an action type or a
 function.  Runs `action-act-hook' before performing action."
-  (eval `(cons 'funcall (cons 'hrule:action ',args))))
+  `(funcall hrule:action ,@args))
 
 (defun    actype:act (actype &rest args)
   "Perform action formed from ACTYPE and rest of ARGS and return value.
@@ -406,32 +398,34 @@ If value is nil, however, t is returned instead, to ensure that implicit button
 types register the performance of the action.  ACTYPE may be a symbol or symbol
 name for either an action type or a function.  Runs `action-act-hook' before
 performing ACTION."
+  (when (null actype)
+    (error "(actype:act): No action type specified"))
   (let ((prefix-arg current-prefix-arg)
 	(action (actype:action actype)))
     (if (null action)
-	(error "(actype:act): Null action for: `%s'" actype)
+	(error "(actype:act): Null action for action type: `%s'" actype)
       ;; Next 2 lines are needed so that relative paths are expanded
       ;; properly.  But in rare cases, this can improperly expand simple
       ;; string arguments like "tags" as a pathname, when it is not
       ;; being used as a path.  So do this only if actype is a defact
       ;; and not a defun to limit any potential impact. RSW - 9/22/2017
-      (and (symbolp action) (symtable:actype-p action)
-	   (setq args (action:path-args-abs args)))
+      (and (symbolp action)
+	   (symtable:actype-p action)
+	   (setq args (hpath:absolute-arguments actype args)))
       (let ((hist-elt (hhist:element)))
 	(run-hooks 'action-act-hook)
 	(prog1 (or (if (or (symbolp action) (listp action)
 			   (hypb:emacs-byte-code-p action)
+			   (subrp action)
 			   (and (stringp action) (not (integerp action))
 				(setq action (key-binding action))))
-		       (if (special-form-p action)
-			   (eval (cons action args))
-			 (apply action args))
+		       (eval (cons action args))
 		     (eval action))
 		   t)
 	  (hhist:add hist-elt))))))
 
 ;; Return the full Elisp symbol for ACTYPE, which may be a string or symbol.
-(defalias   'actype:elisp-symbol 'symtable:actype-p)
+(defalias 'actype:elisp-symbol #'symtable:actype-p)
 
 (defun    actype:def-symbol (actype)
   "Return the abbreviated symbol for ACTYPE used in its `defact'.
@@ -444,14 +438,13 @@ is returned."
       (make-symbol (substring name (match-end 0))))))
 
 (defun    actype:eval (actype &rest args)
-  "Performs action formed from ACTYPE and rest of ARGS and returns value.
+  "Perform action formed from ACTYPE and rest of ARGS and return value.
 ACTYPE may be a string containing a Lisp expression from which ACTYPE
 and ARGS are extracted.  ACTYPE may be a symbol or symbol name for
-either an action type or a function.  Runs `action-act-hook' before
+either an action type or a function.  Run `action-act-hook' before
 performing ACTION."
   (let ((prefix-arg current-prefix-arg)
-	(action (actype:action actype))
-	(act '(apply action args)))
+	(action (actype:action actype)))
     (if (null action)
 	(error "(actype:act): Null action for: `%s'" actype)
       (let ((hist-elt (hhist:element)))
@@ -460,7 +453,7 @@ performing ACTION."
 		       (hypb:emacs-byte-code-p action)
 		       (and (stringp action) (not (integerp action))
 			    (setq action (key-binding action))))
-		   (eval act)
+		   (apply action args)
 		 (eval action))
 	  (hhist:add hist-elt))))))
 
@@ -469,13 +462,14 @@ performing ACTION."
 ACTYPE may be a Hyperbole actype or Emacs Lisp function."
   (let (actname
 	action)
-    (if (stringp actype)
-	(setq actname actype
-	      actype (intern actype))
-      (setq actname (symbol-name actype)))
+    (cond ((stringp actype)
+	   (setq actname actype
+		 actype (intern actype)))
+	  ((and actype (symbolp actype))
+	   (setq actname (symbol-name actype))))
     (setq actype (or (symtable:actype-p actname) actype)
 	  action (htype:body actype))
-    (if (fboundp actype)
+    (if (functionp actype)
 	actype
       action)))
 
@@ -491,7 +485,7 @@ If no action body and actype is a bound function symbol, return that."
     (cond ((htype:body (or (symtable:actype-p actname) actype)))
 	  ((fboundp actype) actype))))
 
-(defmacro actype:create (type params doc &rest default-action)
+(defmacro defact (type params doc &rest default-action)
   "Create an action TYPE (an unquoted symbol) with PARAMS, described by DOC.
 The type uses PARAMS to perform DEFAULT-ACTION (list of the rest of the
 arguments).  A call to this function is syntactically the same as for
@@ -501,8 +495,20 @@ Return symbol created when successful, else nil."
      (symtable:add ',type symtable:actypes)
      (htype:create ,type actypes ,doc ,params ,default-action nil)))
 
-(defalias 'defact 'actype:create)
-(put      'actype:create 'lisp-indent-function 'defun)
+(put      'defact 'lisp-indent-function 'defun)
+
+;; Support edebug-defun for interactive debugging of actypes
+(def-edebug-spec defact
+  (&define name lambda-list
+           [&optional stringp]   ; Match the doc string, if present.
+           def-body))
+
+(def-edebug-spec lambda-list
+  (([&rest arg]
+    [&optional ["&optional" arg &rest arg]]
+    &optional ["&rest" arg])))
+
+(defalias 'actype:create #'defact)
 
 (defun    actype:delete (type)
   "Delete an action TYPE (a symbol).  Return TYPE's symbol if it existed."
@@ -518,14 +524,13 @@ Return nil when no documentation."
 	 (but-type (hattr:get hbut 'categ))
 	 (sym-p (and act (symbolp act)))
 	 (end-line) (doc))
-    (cond ((and but-type (fboundp but-type)
+    (cond ((and (functionp but-type)
 		(setq doc (htype:doc but-type)))
 	   ;; Is an implicit button, so use its doc string if any.
 	   )
 	  (sym-p
 	   (setq doc (htype:doc act))))
-    (if (null doc)
-	nil
+    (when doc
       (setq doc (substitute-command-keys doc))
       (or full (setq end-line (string-match "[\n]" doc)
 		     doc (substring doc 0 end-line))))
