@@ -1,10 +1,11 @@
-;;; hactypes.el --- Default action types for GNU Hyperbole
+;;; hactypes.el --- Default action types for GNU Hyperbole  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    23-Sep-91 at 20:34:36
+;; Last-Mod:     17-Apr-22 at 12:37:06 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -17,6 +18,15 @@
 
 (eval-and-compile (mapc #'require '(hbut hpath hargs hmail)))
 (eval-when-compile (mapc #'require '(comint hsettings)))
+(require 'bookmark)
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function kotl-mode:goto-cell "kotl-mode")
+(declare-function kotl-mode:beginning-of-buffer "kotl-mode")
+(declare-function rmail:msg-to-p "hrmail")
 
 ;;; ************************************************************************
 ;;; Standard Hyperbole action types
@@ -25,8 +35,7 @@
 (defact annot-bib (key)
   "Follow internal ref KEY within an annotated bibliography, delimiters=[]."
   (interactive "sReference key (no []): ")
-  (let ((opoint (point))
-	(key-regexp (concat "^[*]*[ \t]*\\\[" (ebut:key-to-label key) "\\\]"))
+  (let ((key-regexp (concat "^[*]*[ \t]*\\\[" (ebut:key-to-label key) "\\\]"))
 	citation)
     (if (save-excursion
 	  (goto-char (point-max))
@@ -49,8 +58,9 @@ inserted, delete the completions window."
 (defact display-boolean (bool-expr)
   "Display a message showing the result value of a BOOL-EXPR.
 Return any non-nil value or t."
-  (let ((result (eval bool-expr)))
-    (message "Boolean result (%s) = %s" (if result "True" "False") (prin1-to-string result))
+  (let ((result (eval bool-expr t)))
+    (message "Boolean result (%s) = %S; Expr: %S"
+	     (if result "True" "False") result bool-expr)
     (or result t)))
 
 (defact display-variable (var)
@@ -62,7 +72,7 @@ Return any non-nil value or t."
 (defact eval-elisp (lisp-expr)
   "Evaluate a Lisp expression LISP-EXPR for its side-effects and return any non-nil value."
   (interactive "xLisp to eval: ")
-  (eval lisp-expr))
+  (eval lisp-expr t))
 
 (defact exec-kbd-macro (kbd-macro &optional repeat-count)
   "Execute KBD-MACRO REPEAT-COUNT times.
@@ -102,8 +112,7 @@ error."
 				    (and (integerp repeat) (>= repeat 0))))
 			      1))
      (list macro repeat)))
-  (if (called-interactively-p 'interactive)
-      nil
+  (unless (called-interactively-p 'interactive)
     (or (and kbd-macro (or (stringp kbd-macro)
 			   (and (symbolp kbd-macro) (fboundp kbd-macro))))
 	(hypb:error "(exec-kbd-macro): Bad macro: %s" kbd-macro))
@@ -118,9 +127,9 @@ Optional non-nil second argument INTERNAL-CMD inhibits display of the shell
 command line executed.  Optional non-nil third argument KILL-PREV means
 kill the last output to the shell buffer before executing SHELL-CMD."
   (interactive
-   (let ((default  (car defaults))
-	 (default1 (nth 1 defaults))
-	 (default2 (nth 2 defaults)))
+   (let ((default  (car hargs:defaults))
+	 (default1 (nth 1 hargs:defaults))
+	 (default2 (nth 2 hargs:defaults)))
      (list (hargs:read "Shell cmd: "
 		       (lambda (cmd) (not (string-equal cmd "")))
 		       default "Enter a shell command." 'string)
@@ -130,18 +139,19 @@ kill the last output to the shell buffer before executing SHELL-CMD."
 			     default2)))))
   (require 'comint)
   (let ((buf-name "*Hyperbole Shell*")
-	(obuf (current-buffer)))
+	(obuf (current-buffer))
+	(default-dir (expand-file-name default-directory)))
     (unwind-protect
 	(progn
-	  (if (not (hpath:remote-p default-directory))
-	      (setq shell-cmd
-		    (concat "cd " default-directory "; " shell-cmd)))
+	  (unless (hpath:remote-p default-dir)
+	    (setq shell-cmd
+		  (concat "cd " default-dir " && " shell-cmd)))
 	  (if (and (get-buffer buf-name)
 		   (get-buffer-process (get-buffer buf-name)))
 	      (hpath:display-buffer buf-name)
 	    ;; (hpath:display-buffer (current-buffer))
-	    (if (eq (minibuffer-window) (selected-window))
-		(other-window 1))
+	    (when (eq (minibuffer-window) (selected-window))
+	      (other-window 1))
 	    ;; 'shell' calls pop-to-buffer which normally displays in
 	    ;; another window
 	    (setq buf-name (buffer-name (shell buf-name))))
@@ -164,17 +174,17 @@ kill the last output to the shell buffer before executing SHELL-CMD."
 (defact exec-window-cmd (shell-cmd)
   "Asynchronously execute an external window-based SHELL-CMD string."
   (interactive
-   (let ((default  (car defaults)))
+   (let ((default  (car hargs:defaults)))
      (list (hargs:read "Shell cmd: "
 		       (lambda (cmd) (not (string-equal cmd "")))
 		       default "Enter a shell command." 'string))))
   (require 'comint)
-  (let ((buf-name "*Hyperbole Shell*")
-	(cmd (if (hpath:remote-p default-directory)
-		 (concat "(" shell-cmd ") &")
-	       (concat "(cd " default-directory "; " shell-cmd ") &")))
-	(msg (format "Executing: %s" shell-cmd))
-	(shell-buf))
+  (let* ((buf-name "*Hyperbole Shell*")
+	 (default-dir (expand-file-name default-directory))
+	 (cmd (if (hpath:remote-p default-dir)
+		  (concat "(" shell-cmd ") &")
+		(concat "(cd " default-dir " && " shell-cmd ") &")))
+	 (msg (format "Executing: %s" shell-cmd)))
     (message msg)
     (save-excursion
       (save-window-excursion
@@ -196,20 +206,6 @@ kill the last output to the shell buffer before executing SHELL-CMD."
 	(comint-send-input)))
     (message msg)))
 
-(defact function-in-buffer (name pos)
-  "Display the definition of function NAME found at POS in the current buffer."
-  (save-excursion
-    (goto-char pos)
-    (unless (looking-at (regexp-quote name))
-      (let ((fume-scanning-message nil))
-	(fume-rescan-buffer)
-	(setq pos (cdr-safe (assoc name fume-funclist))))))
-  (when pos
-    (hpath:display-buffer (current-buffer))
-    (goto-char pos)
-    ;; Move to beginning of the line for compatibility with find-tag.
-    (beginning-of-line)))
-
 (defact hyp-config (&optional out-buf)
   "Insert Hyperbole configuration information at the end of the current buffer or within optional OUT-BUF."
   (hypb:configuration out-buf))
@@ -248,14 +244,14 @@ For example:  To: hyperbole-users-join@gnu.org\n")))
     (hypb:error "(hyp-source): Non-string argument: %s" buf-str-or-file)))
 
 (defact link-to-bookmark (bookmark)
-  "Display an Emacs BOOKMARK.
+  "Display an Emacs BOOKMARK (a name).
 When creating the button, if in Bookmark Menu mode, use the bookmark
 nearest point as the default.  Otherwise, utilize the most recently used
 bookmark in the current file (bookmark-current-bookmark) as the default,
 if any."
   (interactive
    (list (bookmark-completing-read "Bookmark to link to"
-				   (if (eq major-mode 'bookmark-bmenu-mode)
+				   (if (derived-mode-p 'bookmark-bmenu-mode)
 				       (bookmark-bmenu-bookmark)
 				     bookmark-current-bookmark))))
   (bookmark-jump bookmark (hpath:display-buffer-function)))
@@ -282,41 +278,40 @@ This type of link is for use within a single editor session.  Use
 
 (defact link-to-ebut (key &optional key-file)
   "Perform action given by an explicit button, specified by KEY and optional KEY-FILE.
-KEY-FILE defaults to the current buffer's file name."
+Interactively, KEY-FILE defaults to the current buffer's file name."
   (interactive
    (let (but-lbl
          but-file)
      (while (cond ((setq but-file
-			 (read-file-name
-			  "File of button to link to: " nil nil t))
-		   (if (string-equal but-file "")
-		       (progn (beep) t)))
-		  ((not (file-readable-p but-file))
-		   (message "(link-to-ebut): You cannot read `%s'."
-			    but-file)
-		   (beep) (sit-for 3))))
+                         (read-file-name
+                          "File of button to link to: " nil nil t))
+                   (if (string-equal but-file "")
+                       (progn (beep) t)))
+                  ((not (file-readable-p but-file))
+                   (message "(link-to-ebut): You cannot read `%s'."
+                            but-file)
+                   (beep) (sit-for 3))))
      (list (progn
-	     (find-file-noselect but-file)
-	     (while (string-equal "" (setq but-lbl
-					   (hargs:read-match
-					    "Button to link to: "
-					    (ebut:alist but-file)
-					    nil nil nil 'ebut)))
-	       (beep))
-	     (ebut:label-to-key but-lbl))
-	   but-file)))
+             (find-file-noselect but-file)
+             (while (string-equal "" (setq but-lbl
+                                           (hargs:read-match
+                                            "Button to link to: "
+                                            (ebut:alist but-file)
+                                            nil nil nil 'ebut)))
+               (beep))
+             (ebut:label-to-key but-lbl))
+           but-file)))
   (let (but
-	normalized-file)
-  (if key-file
-      (unless (called-interactively-p 'interactive)
-	(setq normalized-file (hpath:normalize key-file)))
-    (setq normalized-file buffer-file-name))
+        normalized-file)
+    (if key-file
+        (setq normalized-file (hpath:normalize key-file))
+      (setq normalized-file buffer-file-name))
 
-    (if (setq but (and key-file (ebut:get key nil normalized-file)))
-	(hbut:act but)
+    (if (setq but (when normalized-file (ebut:get key nil normalized-file)))
+        (hbut:act but)
       (hypb:error "(link-to-ebut): No button `%s' in `%s'"
-		  (ebut:key-to-label key)
-		  key-file))))
+                  (ebut:key-to-label key)
+                  key-file))))
 
 (defact link-to-elisp-doc (symbol)
   "Display documentation for SYMBOL."
@@ -326,74 +321,80 @@ KEY-FILE defaults to the current buffer's file name."
 	((not (or (boundp symbol) (fboundp symbol)))
 	 (hypb:error "(link-to-elisp-doc): `%s' not defined" symbol))
 	(t (let ((temp-buffer-show-function 'switch-to-buffer))
-	     (hpath:display-buffer (current-buffer))
+	     (hpath:display-buffer (help-buffer))
 	     (describe-symbol symbol)))))
 
+(defun  hactypes:link-to-file-interactively ()
+  "Get a path to link to and return it as a one item list."
+  (let ((prev-reading-p hargs:reading-type)
+	(existing-buf t)
+	path-buf)
+    (unwind-protect
+	(let* ((default-directory (or (hattr:get 'hbut:current 'dir)
+				      (file-name-directory
+				       (or (hattr:get 'hbut:current 'loc) ""))
+				      default-directory))
+	       (file-path (or (car hargs:defaults) default-directory))
+	       (file-point (cadr hargs:defaults))
+	       (hargs:reading-type 'file)
+	       ;; If reading interactive inputs from a key series
+	       ;; (puts key events into the unread queue), then don't
+	       ;; insert default-directory into the minibuffer
+	       ;; prompt, allowing time to remove any extra pathname
+	       ;; quotes added in the key series.
+	       (insert-default-directory (not unread-command-events))
+	       ;; Remove any double quotes and whitespace at the
+	       ;; start and end of the path that interactive use may
+	       ;; have introduced.
+	       (path (hpath:trim (read-file-name "Path to link to: "
+						 file-path file-path)))
+	       (orig-path path)
+	       path-line-and-col
+	       line-num
+	       column-num
+	       normalized-path)
+	  ;; Handle if :line:column are included in path.
+	  (setq path-line-and-col (hpath:file-line-and-column path))
+	  (when path-line-and-col
+	    (setq path (nth 0 path-line-and-col)
+		  line-num (nth 1 path-line-and-col)
+		  column-num (nth 2 path-line-and-col)))
+	  ;; Ensure any variables and heading suffixes following
+	  ;; [#,] are removed before doing path matching.
+	  (setq normalized-path (or (hpath:is-p path) path))
+	  (when (not (or (file-name-absolute-p path)
+			 (string-match "\\`\\$\{" path)))
+	    (setq path (concat default-directory path)))
+	  (setq existing-buf (get-file-buffer normalized-path)
+		path-buf (or existing-buf
+			     (and (file-readable-p normalized-path)
+				  (prog1 (set-buffer (find-file-noselect normalized-path t))
+				    (when (integerp file-point)
+				      (goto-char (min (point-max) file-point)))))))
+	  (if (and path-buf (not line-num))
+	      (with-current-buffer path-buf
+		(setq hargs:reading-type 'character)
+		(if (y-or-n-p
+		     (format "y = Display at present position (line %d); n = no position? "
+			     (count-lines 1 (point))))
+		    (list path (point))
+		  (list path)))
+	    (if path-buf
+		(delq nil (list path (save-excursion
+				       (goto-char (point-min))
+				       (forward-line (1- line-num))
+				       (when column-num (move-to-column column-num))
+				       (point))))
+	      (list (or path orig-path)))))
+      (setq hargs:reading-type prev-reading-p)
+      (when (and path-buf (not existing-buf))
+	(kill-buffer path-buf)))))
+
 (defact link-to-file (path &optional point)
   "Display a file given by PATH scrolled to optional POINT.
 If POINT is given, display the buffer with POINT at the top of
-the window."
-  (interactive
-   (let ((prev-reading-p hargs:reading-p)
-	 (existing-buf t)
-	 path-buf)
-     (unwind-protect
-	 (let* ((default-directory (or (hattr:get 'hbut:current 'dir) default-directory))
-		(file-path (or (car defaults) default-directory))
-		(file-point (cadr defaults))
-		(hargs:reading-p 'file)
-		;; If reading interactive inputs from a key series
-		;; (puts key events into the unread queue), then don't
-		;; insert default-directory into the minibuffer
-		;; prompt, allowing time to remove any extra pathname
-		;; quotes added in the key series.
-		(insert-default-directory (not unread-command-events))
-		;; Remove any double quotes and whitespace at the
-		;; start and end of the path that interactive use may
-		;; have introduced.
-		(path (hpath:trim (read-file-name "Path to link to: "
-						  file-path file-path)))
-		(orig-path path)
-		path-line-and-col
-		line-num
-		column-num
-		normalized-path)
-	   ;; Handle if :line:column are included in path.
-	   (setq path-line-and-col (hpath:file-line-and-column path))
-	   (when path-line-and-col
-	     (setq path (nth 0 path-line-and-col)
-		   line-num (nth 1 path-line-and-col)
-		   column-num (nth 2 path-line-and-col)))
-	   ;; Ensure any variables and heading suffixes following
-	   ;; [#,] are removed before doing path matching.
-	   (setq normalized-path (hpath:is-p path))
-	   (when (not (or (file-name-absolute-p path)
-			  (string-match "\\`\\$\{" path)))
-	     (setq path (concat default-directory path)))
-	   (setq existing-buf (get-file-buffer normalized-path)
-		 path-buf (or existing-buf
-			      (and (file-readable-p normalized-path)
-				   (prog1 (set-buffer (find-file-noselect normalized-path t))
-				     (when (integerp file-point)
-				       (goto-char (min (point-max) file-point)))))))
-	   (if (and path-buf (not line-num))
-	       (with-current-buffer path-buf
-		 (setq hargs:reading-p 'character)
-		 (if (y-or-n-p
-		      (format "y = Display at present position (line %d); n = no position? "
-			      (count-lines 1 (point))))
-		     (list path (point))
-		   (list path)))
-	     (if path-buf
-		 (delq nil (list path (save-excursion
-					(goto-char (point-min))
-					(forward-line (1- line-num))
-					(when column-num (move-to-column column-num))
-					(point))))
-	       (list (or path orig-path)))))
-       (setq hargs:reading-p prev-reading-p)
-       (when (and path-buf (not existing-buf))
-	 (kill-buffer path-buf)))))
+the window or as close as possible."
+  (interactive (hactypes:link-to-file-interactively))
   (if path
       (progn
 	;; Remove any double quotes and whitespace at the start and end of
@@ -434,12 +435,12 @@ the window."
       (hpath:find-line path line-num))
     (move-to-column column-num)))
 
-(defact link-to-gbut (key &optional key-file)
+(defact link-to-gbut (key &optional _key-file)
   "Perform an action given by an existing global button, specified by KEY.
 Optional second arg, KEY-FILE, is not used but is for calling
 compatibility with the `hlink' function."
   (interactive
-   (let ((gbut-file (hpath:validate (hpath:substitute-value gbut:file)))
+   (let ((gbut-file (hpath:validate (hpath:substitute-value (gbut:file))))
 	 but-lbl)
      (if (not (file-readable-p gbut-file))
 	 (hypb:error "(link-to-gbut): You cannot read `%s'" gbut-file)
@@ -448,7 +449,7 @@ compatibility with the `hlink' function."
 	       (while (string-equal "" (setq but-lbl
 					     (hargs:read-match
 					      "Global button to link to: "
-					      (mapcar 'list (gbut:label-list))
+					      (mapcar #'list (gbut:label-list))
 					      nil t nil 'gbut)))
 		 (beep))
 	       (hbut:label-to-key but-lbl))))))
@@ -462,6 +463,9 @@ item-name is available.  Filename may be given without the .info
 suffix."
   (interactive "+XInfo (file)index-item-name to link to: ")
   (require 'info)
+  (when (stringp index-item)
+    ;; Remove any tabs or newlines that might be in index-item.
+    (setq index-item (replace-regexp-in-string "[ \t\n\r\f]+" " " index-item t t)))
   (if (and (stringp index-item) (string-match "^(\\([^\)]+\\))\\(.*\\)" index-item))
       (id-info-item index-item)
     (hypb:error "(link-to-Info-index-entry): Invalid Info index item: `%s'" index-item)))
@@ -473,48 +477,60 @@ button creation, completion for both filename and node names is
 available.  Filename may be given without the .info suffix."
   (interactive "+IInfo (file)nodename to link to: ")
   (require 'info)
+  (when (stringp string)
+    ;; Remove any tabs or newlines that might be in string.
+    (setq string (replace-regexp-in-string "[ \t\n\r\f]+" " " string t t)))
   (if (and (stringp string) (string-match "^(\\([^\)]+\\))\\(.*\\)" string))
       (id-info string)
     (hypb:error "(link-to-Info-node): Invalid Info node: `%s'" string)))
 
-(defact link-to-ibut (key &optional key-file point)
-  "Perform an action given by an implicit button, specified by KEY, optional KEY-FILE and POINT.
-KEY-FILE defaults to the current buffer's file and POINT to the current point.
+(defact link-to-ibut (key &optional but-src point)
+  "Perform an action given by an implicit button, specified by KEY, optional BUT-SRC and POINT.
+BUT-SRC defaults to the current buffer's file or if there is no
+attached file, then to its buffer name.  POINT defaults to the
+current point.
 
-When creating the button, point must be on the implicit button to which to link
-and its buffer must have a file attached."
+When the button with this action type is created, point must be
+on the implicit button to which to link."
   (interactive
    (let ((ibut-key (ibut:at-p t)))
-     (if (and ibut-key buffer-file-name)
-	 (list ibut-key buffer-file-name (point))
-       ;; TODO: If default is null below and are creating, rather than modifying,
-       ;; the link, it would be better to throw an error than create
-       ;; an invalid link, but it is difficult to tell which operation
-       ;; is in progress, so ignore this for now.  -- RSW, 01-25-2020
+     (cond (ibut-key
+	    (list ibut-key (or buffer-file-name (buffer-name)) (point)))
+	   ;; TODO: If default is null below and are creating, rather than editing
+	   ;; the link, it would be better to throw an error than create
+	   ;; an invalid link, but it is difficult to tell which operation
+	   ;; is in progress, so ignore this for now.  -- RSW, 01-25-2020
 
-       ;; When not on an ibut and moddifying the link, use existing arguments
-       (if (and (boundp 'defaults) defaults (listp defaults))
-	   defaults
-	 (list nil nil nil)))))
+	   ;; When not on an ibut and editing the link, use existing arguments
+	   ((and (bound-and-true-p hargs:defaults) (listp hargs:defaults) hargs:defaults)
+	    hargs:defaults)
+	   (t
+	    (hypb:error "(link-to-ibut): Point must be on an implicit button to create a link-to-ibut")))))
+  (when (null key)
+    (hypb:error "(link-to-ibut): Point must be on an implicit button to create a link-to-ibut"))
   (let (but
 	normalized-file)
-    (if key-file
-	(unless (called-interactively-p 'interactive)
-	  (setq normalized-file (hpath:normalize key-file)))
-      (setq normalized-file buffer-file-name))
+    (cond (but-src
+	   (unless (and (get-buffer but-src)
+			(not (buffer-file-name (get-buffer but-src))))
+	     (setq normalized-file (hpath:normalize but-src))))
+	  (t (setq normalized-file buffer-file-name)))
     (save-excursion
       (save-restriction
-	(when key-file
-	  (set-buffer (get-file-buffer normalized-file)))
+	(when but-src
+	  (set-buffer (or (get-buffer but-src) (get-file-buffer normalized-file))))
 	(widen)
-	(if (integerp point) (goto-char (min point (point-max))))
+	(when (or (not normalized-file) (hmail:editor-p) (hmail:reader-p))
+	  (hmail:msg-narrow))
+	(when (integerp point)
+	  (goto-char (min point (point-max))))
 	(setq but (ibut:to key))))
     (cond (but
 	   (hbut:act but))
 	  (key
 	   (hypb:error "(link-to-ibut): No implicit button `%s' found in `%s'"
 		       (ibut:key-to-label key)
-		       (or key-file (buffer-name))))
+		       (or but-src (buffer-name))))
 	  (t
 	   (hypb:error "(link-to-ibut): Link reference is null/empty")))))
 
@@ -593,8 +609,8 @@ Return t if found, signal an error if not."
   "Retrieve and display an Internet rfc given by RFC-NUM.
 RFC-NUM may be a string or an integer."
   (interactive "nRFC number to retrieve: ")
-  (if (or (stringp rfc-num) (integerp rfc-num))
-      (hpath:find (hpath:rfc rfc-num))))
+  (when (or (stringp rfc-num) (integerp rfc-num))
+    (browse-url-emacs (hpath:rfc rfc-num))))
 
 (defact link-to-string-match (string n source &optional buffer-p)
   "Find STRING's Nth occurrence in SOURCE and display location at window top.
@@ -605,17 +621,30 @@ Return t if found, nil if not."
   (funcall (actype:action 'link-to-regexp-match)
 	   (regexp-quote string) n source buffer-p))
 
-(defact link-to-texinfo-node (nodename)
-  "Display the Texinfo node with NODENAME (a string) from the current buffer."
-  (interactive "sTexinfo nodename to link to: ")
+(defact link-to-texinfo-node (file node)
+  "Display the Texinfo FILE and NODE (a string).
+FILE may be a string or nil, in which case the current buffer is used."
+  (interactive "fTexinfo file to link to: \nsNode within file to link to: ")
+  (when (stringp node)
+    ;; Remove any tabs or newlines that might be in node name.
+    (setq node (replace-regexp-in-string "[ \t\n\r\f]+" " " node t t)))
   (let (node-point)
+    (if file
+        (set-buffer (find-find-noselect (hpath:substitute-value file)))
+      (setq file buffer-file-name))
     (save-excursion
       (goto-char (point-min))
-      (if (re-search-forward (format "^@node[ \t]+%s *[,\n\r]" nodename) nil t)
+      (if (re-search-forward (format "^@node[ \t]+%s *[,\n\r]" node) nil t)
 	  (setq node-point (match-beginning 0))
-	(hypb:error "(link-to-texinfo-node): Non-existent node: `%s'"
-		    nodename)))
-    (hact 'link-to-file buffer-file-name node-point)))
+	(hypb:error "(link-to-texinfo-node): Non-existent node: \"%s%s\""
+                    (if file
+                        (format "(%s)" (file-name-nondirectory file))
+                      "")
+		    node)))
+    (if file
+        (hact 'link-to-file file node-point)
+      (hypb:error "(link-to-texinfo-node): Non-existent node: \"%s\""
+		  node))))
 
 (defact link-to-web-search (service-name search-term)
   "Search web SERVICE-NAME for SEARCH-TERM.
@@ -633,9 +662,9 @@ Uses `hpath:display-where' setting to control where the man page is displayed."
   (let ((Man-notify-method 'meek))
     (hpath:display-buffer (man topic))))
 
-(defact rfc-toc (&optional buf-name opoint)
+(defact rfc-toc (&optional buf-name opoint sections-start)
   "Compute and display summary of an Internet rfc in BUF-NAME.
-Assumes point has already been moved to start of region to summarize.
+Assume point has already been moved to start of region to summarize.
 Optional OPOINT is point to return to in BUF-NAME after displaying summary."
   (interactive)
   (if buf-name
@@ -651,7 +680,7 @@ Optional OPOINT is point to return to in BUF-NAME after displaying summary."
 	(temp-buffer-show-function 'switch-to-buffer)
 	occur-buffer)
     (hpath:display-buffer (current-buffer))
-    (occur sect-regexp)
+    (occur sect-regexp nil (list (cons sections-start (point-max))))
     (setq occur-buffer (set-buffer "*Occur*"))
     (rename-buffer (format "*%s toc*" buf-name))
     (re-search-forward "^[ ]*[0-9]+:" nil t)
