@@ -1,10 +1,11 @@
-;;; hargs.el --- GNU Hyperbole user input functions
+;;; hargs.el --- GNU Hyperbole user input functions    -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    31-Oct-91 at 23:17:35
+;; Last-Mod:     20-Feb-22 at 22:15:24 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -25,6 +26,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
+(require 'etags)                        ; For `find-tag--default'
 (require 'hpath)
 (require 'hypb)
 (require 'set)
@@ -33,8 +35,11 @@
 ;;; Public variables
 ;;; ************************************************************************
 
-(defvar hargs:reading-p nil
-  "Is either a symbol representing the type of object Hyperbole is prompting the user to input or nil.")
+(defvar hargs:defaults nil
+  "Default arguments read from an existing Hyperbole button when editing it.")
+
+(defvar hargs:reading-type nil
+  "Symbol representing the type of object Hyperbole is prompting the user to input.")
 
 (add-hook 'completion-setup-hook #'hargs:set-string-to-complete)
 (add-hook 'minibuffer-exit-hook  #'hargs:unset-string-to-complete)
@@ -43,52 +48,23 @@
 ;;; Private functions
 ;;; ************************************************************************
 
-;;; From etags.el, so don't have to load the whole thing.
-(unless (fboundp 'find-tag-default)
-  (defun find-tag-default ()
-    (or (and (boundp 'find-tag-default-hook)
-	     (not (memq find-tag-default-hook '(nil find-tag-default)))
-	     (condition-case data
-		 (funcall find-tag-default-hook)
-	       (error
-		(message "value of find-tag-default-hook signalled error: %s"
-			 data)
-		(sit-for 1)
-		nil)))
-	(save-excursion
-	  (unless (memq (char-syntax (preceding-char)) '(?w ?_))
-	    (while (not (looking-at "\\sw\\|\\s_\\|\\'"))
-	      (forward-char 1)))
-	  (while (looking-at "\\sw\\|\\s_")
-	    (forward-char 1))
-	  (when (re-search-backward "\\sw\\|\\s_" nil t)
-	    (forward-char 1)
-	    (regexp-quote (buffer-substring (point)
-					    (progn (forward-sexp -1)
-						   (while (looking-at "\\s'")
-						     (forward-char 1))
-						   (point)))))))))
+(defalias 'hargs:find-tag-default #'find-tag--default)
 
-(unless (fboundp 'find-tag--default)
-  (defun find-tag--default ()
-    (funcall (or (when (fboundp find-tag-default-function) find-tag-default-function)
-		 (get major-mode 'find-tag-default-function)
-		 'find-tag-default))))
-(defalias 'hargs:find-tag-default 'find-tag--default)
-
-
-(defun hargs:action-get (action modifying)
+(defun hargs:action-get (action editing-flag)
   "Interactively get list of arguments for ACTION's parameters.
-Current button is being modified when MODIFYING is non-nil.
+Current button is being edited when EDITING-FLAG is t.
 Return nil if ACTION is not a list or `byte-code' object, has no
 interactive form or takes no arguments."
-  (and (or (hypb:emacs-byte-code-p action) (listp action))
-       (let ((interactive-form (action:commandp action)))
-	 (when interactive-form
-	   (action:path-args-rel
-	    (hargs:iform-read interactive-form modifying))))))
+  (save-excursion
+    (and (or (hypb:emacs-byte-code-p action) (listp action))
+	 (let ((interactive-form (action:commandp action)))
+	   (when interactive-form
+	     (hpath:relative-arguments
+	      (hargs:iform-read interactive-form editing-flag)))))))
 
 (defun hargs:buffer-substring (start end)
+  "Return the buffer substring sans any properties between START and END positions.
+Convert NUL characters to colons for use with grep lines."
   (let ((string (buffer-substring-no-properties start end)))
     ;; This may trigger on a colored grep-like output line which has
     ;; an embedded null character with a display text property that
@@ -104,47 +80,59 @@ START-DELIM and END-DELIM are strings that specify the argument
 delimiters.  With optional START-REGEXP-FLAG non-nil, START-DELIM is
 treated as a regular expression.  END-REGEXP-FLAG is similar.
 With optional LIST-POSITIONS-FLAG, return list of (string-matched start-pos end-pos).
-With optional EXCLUDE-REGEXP, any matched string is ignored if it this regexp."
+With optional EXCLUDE-REGEXP, any matched string is ignored if it matches this regexp."
   (let* ((opoint (point))
+	 (line-begin (line-beginning-position))
+	 ;; This initial limit if the forward search limit for start delimiters
 	 (limit (if start-regexp-flag opoint
 		  (+ opoint (1- (length start-delim)))))
-	 (start-search-func (if start-regexp-flag 're-search-forward
-			      'search-forward))
-	 (end-search-func (if end-regexp-flag 're-search-forward
-			    'search-forward))
+	 (forward-search-func (if start-regexp-flag 're-search-forward
+				'search-forward))
+	 (reverse-search-func (if end-regexp-flag 're-search-backward
+				'search-backward))
 	 (count 0)
 	 first
 	 start
 	 end)
-    (save-excursion
-      (beginning-of-line)
-      (while (and (setq start (funcall start-search-func start-delim limit t))
-		  (setq count (1+ count))
-		  (< (point) opoint)
-		  ;; This is not to find the real end delimiter but to find
-		  ;; end delimiters that precede the current argument and are
-		  ;; therefore false matches, hence the search is limited to
-		  ;; prior to the original point.
-		  (funcall end-search-func end-delim opoint t)
-		  (setq count (1+ count)))
-	(setq first (or first start)
-	      start nil))
-      (when (and (not start) (> count 0) (zerop (% count 2)))
-	;; Since strings can span lines but this function matches only
-	;; strings that start on the current line, when start-delim and
-	;; end-delim are the same and there are an even number of
-	;; delimiters in the search range, causing the end-delim
-	;; search to match to what should probably be the start-delim,
-	;; assume point is within a string and not between two other strings.
-	;; -- RSW, 02-05-2019
-	(setq start (if (string-equal start-delim end-delim)
-			(point)
-		      first)))
-      (when start
+    (if (string-equal start-delim end-delim)
+	(save-excursion
+	  (beginning-of-line)
+	  (while (and (setq start (funcall forward-search-func start-delim limit t))
+		      (setq count (1+ count))
+		      (< (point) opoint)
+		      ;; This is not to find the real end delimiter but to find
+		      ;; end delimiters that precede the current argument and are
+		      ;; therefore false matches, hence the search is limited to
+		      ;; prior to the original point.
+		      (funcall forward-search-func end-delim opoint t)
+		      (setq count (1+ count)))
+	    (setq first (or first start)
+		  start nil))
+	  (when (and (not start) (> count 0) (zerop (% count 2)))
+	    ;; Since strings can span lines but this function matches only
+	    ;; strings that start on the current line, when start-delim and
+	    ;; end-delim are the same and there are an even number of
+	    ;; delimiters in the search range, causing the end-delim
+	    ;; search to match to what should probably be the start-delim,
+	    ;; assume point is within a string and not between two other strings.
+	    ;; -- RSW, 02-05-2019
+	    (setq start (if (string-equal start-delim end-delim)
+			    (point)
+			  first))))
+      ;;
+      ;; Start and end delims are different, so don't have to worry
+      ;; about whether in or outside two of the same delimiters and
+      ;; can match much more simply.
+      (save-excursion
+	(setq start (when (funcall reverse-search-func start-delim line-begin t)
+		      (match-end 0)))))
+
+    (when start
+      (save-excursion
 	(forward-line 2)
 	(setq limit (point))
 	(goto-char opoint)
-	(and (funcall end-search-func end-delim limit t)
+	(and (funcall forward-search-func end-delim limit t)
 	     (setq end (match-beginning 0))
 	     ;; Ignore any preceding backslash, e.g. when a double-quoted
 	     ;; string is embedded within a doc string, except when
@@ -206,19 +194,26 @@ element of the list is always the symbol 'args."
 		"(hargs:get): Bad interactive-entry command character: `%c'"
 		cmd))))))
 
-(defun hargs:make-iform-vector (iform-alist)
-  "Return a vector built from IFORM-ALIST used for looking up interactive command code characters."
+(defmacro hargs:make-iform-vector (&rest iform-alist)
+  "Return a vector of interactive command code characters.
+IFORM-ALIST is a list of elements of the form
+    (INTERACTIVE-CMD-CHR  (ARGUMENT-TYPE . GET-ARGUMENT-FORM))
+GET-ARGUMENT-FORM is executed in a context where it has access to
+two variables `prompt' and `default'."
   ;; Vector needs to have 1 more elts than the highest char code for
   ;; interactive commands.
-  (let* ((size (1+ (car (sort (mapcar 'car iform-alist) '>))))
-	 (vec (make-vector size nil)))
-    (mapc (lambda (elt)
-	    (aset vec (car elt)
-		  `(lambda (prompt default)
-		     (setq hargs:reading-p ',(cadr elt))
-		     ,(cddr elt))))
-	  iform-alist)
-    vec))
+  (let ((size (1+ (car (sort (mapcar #'car iform-alist) #'>))))
+        (vecsym (make-symbol "vec")))
+    `(let ((,vecsym (make-vector ',size nil)))
+       ,@(mapcar (lambda (elt)
+		   `(aset ,vecsym ',(car elt)
+		          (lambda (prompt default)
+		            (ignore prompt default) ;Don't warn if not used.
+		            ;; FIXME: Why `setq' instead of let-binding?
+		            (setq hargs:reading-symbol ',(cadr elt))
+		            ,(cddr elt))))
+		 iform-alist)
+       ,vecsym)))
 
 (defun hargs:prompt (prompt default &optional default-prompt)
   "Return string of PROMPT including DEFAULT.
@@ -280,35 +275,37 @@ that point is within is returned or nil if none."
 ;;; Public functions
 ;;; ************************************************************************
 
-(defun hargs:actype-get (actype &optional modifying)
-  "Interactively gets and return list of arguments for ACTYPE's parameters.
-Current button is being modified when MODIFYING is non-nil."
-  (hargs:action-get (actype:action-body actype) modifying))
+(defun hargs:actype-get (actype &optional editing-flag)
+  "Interactively get and return list of arguments for ACTYPE's parameters.
+Current button is being edited when EDITING-FLAG is non-nil."
+  (when editing-flag
+    (setq editing-flag t))
+  (hargs:action-get (actype:action-body actype) editing-flag))
 
 (defun hargs:at-p (&optional no-default)
-  "Return thing at point, if of hargs:reading-p type, or default.
+  "Return thing at point, if of hargs:reading-type type, or default.
 If optional argument NO-DEFAULT is non-nil, nil is returned instead of any
 default values.
 
 Caller should have checked whether an argument is presently being read
-and has set `hargs:reading-p' to an appropriate argument type.
+and has set `hargs:reading-type' to an appropriate argument type.
 Handles all of the interactive argument types that `hargs:iform-read' does."
-  (cond ((and (eq hargs:reading-p 'kcell)
+  (cond ((and (eq hargs:reading-type 'kcell)
 	      (eq major-mode 'kotl-mode)
 	      (not (looking-at "^$")))
 	 (kcell-view:label))
-	((and (eq hargs:reading-p 'klink)
+	((and (eq hargs:reading-type 'klink)
 	      (not (looking-at "^$")))
 	 (if (eq major-mode 'kotl-mode)
 	     (kcell-view:reference
 	      nil (and (boundp 'default-dir) default-dir))
-	   (let ((hargs:reading-p 'file))
+	   (let ((hargs:reading-type 'file))
 	     (list (hargs:at-p)))))
-	((eq hargs:reading-p 'kvspec)
+	((eq hargs:reading-type 'kvspec)
 	 (read-string "Koutline view spec: "
 		      (when (boundp 'kvspec:current) kvspec:current)))
 	((eolp) nil)
-	((and (eq hargs:reading-p 'hmenu)
+	((and (eq hargs:reading-type 'hmenu)
 	      (eq (selected-window) (minibuffer-window)))
 	   (char-to-string
 	    (save-excursion
@@ -316,26 +313,27 @@ Handles all of the interactive argument types that `hargs:iform-read' does."
 		     ;; In menu prefix
 		     1)
 		    ((search-backward " " nil t)
-		     ;; On a menu item
+		     ;; On a menu item, return the first capitalized
+		     ;; char or if none, the first character
 		     (skip-chars-forward " ")
-		     (following-char))
+		     (let (case-fold-search)
+		       (if (looking-at "[^ \t\nA-Z]*[A-Z]")
+			   (char-before (match-end 0))
+			 (following-char))))
 		    ;; At the end of the menu
 		    (t 0)))))
 	((hargs:completion t))
-	((eq hargs:reading-p 'ebut) (ebut:label-p 'as-label))
-	((eq hargs:reading-p 'ibut) (ibut:label-p 'as-label))
-	((eq hargs:reading-p 'gbut)
-	 (when (eq (current-buffer) (get-file-buffer gbut:file))
+	((eq hargs:reading-type 'ebut) (ebut:label-p 'as-label))
+	((eq hargs:reading-type 'ibut) (ibut:label-p 'as-label))
+	((eq hargs:reading-type 'gbut)
+	 (when (eq (current-buffer) (get-file-buffer (gbut:file)))
 	   (hbut:label-p 'as-label)))
-	((eq hargs:reading-p 'hbut) (hbut:label-p 'as-label))
+	((eq hargs:reading-type 'hbut) (hbut:label-p 'as-label))
 	((hbut:label-p) nil)
-	((eq hargs:reading-p 'file)
+	((eq hargs:reading-type 'file)
 	 (cond ((derived-mode-p 'dired-mode)
 		(let ((file (dired-get-filename nil t)))
 		  (and file (hpath:absolute-to file))))
-	       ((eq major-mode 'monkey-mode)
-		(let ((file (monkey-filename t)))
-		  (and file (hpath:absolute-to file))))
 	       ;; Delimited file name.
 	       ((hpath:at-p 'file))
 	       ;; Unquoted remote file name.
@@ -343,17 +341,12 @@ Handles all of the interactive argument types that `hargs:iform-read' does."
 	       ;; Possibly non-existent file name
 	       ((when no-default (hpath:at-p 'file 'non-exist)))
 	       (no-default nil)
-	       ((buffer-file-name))
-	       ))
-	((eq hargs:reading-p 'directory)
+	       ((buffer-file-name))))
+	((eq hargs:reading-type 'directory)
 	 (cond ((derived-mode-p 'dired-mode)
 		(let ((dir (dired-get-filename nil t)))
 		  (and dir (setq dir (hpath:absolute-to dir))
 		       (file-directory-p dir) dir)))
-	       ((eq major-mode 'monkey-mode)
-		(let ((dir (monkey-filename t)))
-		  (and dir (setq dir (hpath:absolute-to dir))
-		       (file-directory-p dir) dir)))
 	       ;; Delimited directory name.
 	       ((hpath:at-p 'directory))
 	       ;; Unquoted remote directory name.
@@ -361,22 +354,20 @@ Handles all of the interactive argument types that `hargs:iform-read' does."
 	       ;; Possibly non-existent directory name
 	       ((when no-default (hpath:at-p 'directory 'non-exist)))
 	       (no-default nil)
-	       (default-directory)
-	       ))
-	((eq hargs:reading-p 'string)
+	       (default-directory)))
+	((eq hargs:reading-type 'string)
 	 (or (hargs:delimited "\"" "\"") (hargs:delimited "'" "'")
-	     (hargs:delimited "`" "'")
-	     ))
-	((or (eq hargs:reading-p 'actype)
-	     (eq hargs:reading-p 'actypes))
+	     (hargs:delimited "`" "'")))
+	((or (eq hargs:reading-type 'actype)
+	     (eq hargs:reading-type 'actypes))
 	 (let ((name (hargs:find-tag-default)))
 	   (car (set:member name (htype:names 'actypes)))))
-	((or (eq hargs:reading-p 'ibtype)
-	     (eq hargs:reading-p 'ibtypes))
+	((or (eq hargs:reading-type 'ibtype)
+	     (eq hargs:reading-type 'ibtypes))
 	 (let ((name (hargs:find-tag-default)))
 	   (car (set:member name (htype:names 'ibtypes)))))
-	((eq hargs:reading-p 'sexpression) (hargs:sexpression-p))
-	((memq hargs:reading-p '(Info-index-item Info-node))
+	((eq hargs:reading-type 'sexpression) (hargs:sexpression-p))
+	((memq hargs:reading-type '(Info-index-item Info-node))
 	 (when (eq major-mode 'Info-mode)
 	   (let ((file (Info-current-filename-sans-extension))
 		 (node (cond ((Info-note-at-p))
@@ -391,21 +382,24 @@ Handles all of the interactive argument types that `hargs:iform-read' does."
 		   (file
 		    (concat "(" file ")" node))
 		   (t node)))))
-	((eq hargs:reading-p 'mail)
+	((eq hargs:reading-type 'mail)
 	 (and (hmail:reader-p) buffer-file-name
 	      (prin1-to-string (list (rmail:msg-id-get) buffer-file-name))))
-	((eq hargs:reading-p 'symbol)
+	((eq hargs:reading-type 'symbol)
 	 (let ((sym (hargs:find-tag-default)))
 	   (when (or (fboundp sym) (boundp sym)) sym)))
-	((eq hargs:reading-p 'buffer)
-	 (hargs:find-tag-default))
-	((eq hargs:reading-p 'character)
+	((eq hargs:reading-type 'buffer)
+	 (let ((tag (hargs:find-tag-default)))
+	   (if (member tag (mapcar #'buffer-name (buffer-list)))
+	       tag
+	     (buffer-name))))
+	((eq hargs:reading-type 'character)
 	 (following-char))
-	((eq hargs:reading-p 'key)
+	((eq hargs:reading-type 'key)
 	 (require 'hib-kbd)
 	 (let ((key-seq (hbut:label-p 'as-label "{" "}")))
 	   (when key-seq (kbd-key:normalize key-seq))))
-	((eq hargs:reading-p 'integer)
+	((eq hargs:reading-type 'integer)
 	 (save-excursion (skip-chars-backward "-0-9")
 			 (when (looking-at "-?[0-9]+")
 			   (read (current-buffer)))))))
@@ -416,7 +410,10 @@ Also insert unless optional NO-INSERT is non-nil.
 Insert in minibuffer if active or in other window if minibuffer is inactive."
   (interactive '(nil))
   (when (or (string-match "[* ]Completions\\*\\'" (buffer-name))
-	    (eq major-mode 'completion-mode))
+	    (eq major-mode 'completion-mode)
+	    (and (boundp 'which-key--buffer)
+		 (eq (window-buffer action-key-depress-window) which-key--buffer)
+		 (eq (window-buffer action-key-release-window) which-key--buffer)))
     (let ((opoint (point))
 	  (owind (selected-window)))
       (when (re-search-backward "^\\|\t\\| [ \t]" nil t)
@@ -434,8 +431,7 @@ Insert in minibuffer if active or in other window if minibuffer is inactive."
 		     ;; newline or two whitespace characters.
 		     (looking-at
 		      "[^ \t\n]+\\( [^ \t\n]+\\)*\\( [ \t\n]\\|[\t\n]\\|\\'\\)"))
-	    (setq entry (buffer-substring (match-beginning 0)
-					  (match-beginning 2)))
+	    (setq entry (hypb:get-completion))
 	    (select-window insert-window)
 	    (let ((str (or hargs:string-to-complete
 			   (buffer-substring
@@ -462,8 +458,9 @@ Insert in minibuffer if active or in other window if minibuffer is inactive."
 			       (1+ (match-beginning 0)))
 			      entry))))
 		(or no-insert
-		    (if entry (progn (erase-buffer)
-				     (insert entry)))))
+		    (when entry
+		      (erase-buffer)
+		      (insert entry))))
 	       ;; In buffer, non-minibuffer completion.
 	       ;; Only insert entry if last buffer line does
 	       ;; not end in entry.
@@ -479,38 +476,32 @@ Insert in minibuffer if active or in other window if minibuffer is inactive."
 	    (delete-window))
 	  entry)))))
 
-(defun hargs:iform-read (iform &optional modifying)
-  "Read action arguments according to IFORM, a list with car = 'interactive.
-Optional MODIFYING non-nil indicates current button is being modified, so
-button's current values should be presented as defaults.  Otherwise, uses
-hargs:defaults as list of defaults, if any.
+(defun hargs:iform-read (iform &optional default-args)
+  "Read action arguments according to IFORM, a list with car = `interactive'.
+With optional DEFAULT-ARGS equal to t, the current button is being edited, so
+its attribute values should be presented as defaults.  Otherwise, use
+DEFAULT-ARGS as a list of defaults to present when reading arguments.
 See also documentation for `interactive'."
   ;; This is mostly a translation of `call-interactively' to Lisp.
   ;;
-  ;; Save this now, since use of minibuffer will clobber it.
+  ;; Save the prefix arg now, since use of minibuffer will clobber it
   (setq prefix-arg current-prefix-arg)
   (if (not (and (listp iform) (eq (car iform) 'interactive)))
       (error "(hargs:iform-read): arg must be a list whose car = 'interactive")
     (setq iform (car (cdr iform)))
     (unless (or (null iform) (and (stringp iform) (equal iform "")))
-      (let ((prev-reading-p hargs:reading-p))
+      (let ((prev-reading-p hargs:reading-type))
 	(unwind-protect
 	    (progn
-	      (setq hargs:reading-p t)
+	      (when (eq default-args t)
+		(setq default-args (hattr:get 'hbut:current 'args)
+		      ;; Set hargs:defaults global used by "hactypes.el"
+		      hargs:defaults default-args))
+	      (setq hargs:reading-type t)
 	      (if (not (stringp iform))
-		  (let ((defaults (if modifying
-				      (hattr:get 'hbut:current 'args)
-				    (and (boundp 'hargs:defaults)
-					 (listp hargs:defaults)
-					 hargs:defaults))))
-		    (eval iform))
+		  (eval iform)
 		(let ((i 0) (start 0) (end (length iform))
-		      (ientry) (results) (val) (default)
-		      (defaults (if modifying
-				    (hattr:get 'hbut:current 'args)
-				  (and (boundp 'hargs:defaults)
-				       (listp hargs:defaults)
-				       hargs:defaults))))
+		      (ientry) (results) (val) (default))
 		  ;;
 		  ;; Handle special initial interactive string chars.
 		  ;;
@@ -544,12 +535,12 @@ See also documentation for `interactive'."
 		    (setq start (match-end 0)
 			  ientry (substring iform i (match-beginning 0))
 			  i start
-			  default (car defaults)
+			  default (car default-args)
 			  default (if (or (null default) (stringp default))
 				      default
 				    (prin1-to-string default))
 			  val (hargs:get ientry default (car results))
-			  defaults (cdr defaults)
+			  default-args (cdr default-args)
 			  results (cond ((or (null val) (not (listp val)))
 					 (cons val results))
 					;; Is a list of args?
@@ -558,7 +549,7 @@ See also documentation for `interactive'."
 					(t ;; regular list value
 					 (cons val results)))))
 		  (nreverse results))))
-	  (setq hargs:reading-p prev-reading-p))))))
+	  (setq hargs:reading-type prev-reading-p))))))
 
 (defun hargs:read (prompt &optional predicate default err val-type)
   "PROMPT without completion for a value matching PREDICATE and return it.
@@ -569,15 +560,15 @@ Optional VAL-TYPE is a symbol indicating the type of value to be read.  If
 VAL-TYPE equals `sexpression', then return that type; otherwise return the
 string read or nil."
   (let ((bad-val) (val) (stringify)
-	(prev-reading-p hargs:reading-p) (read-func)
+	(prev-reading-p hargs:reading-type) (read-func)
 	(owind (selected-window))
 	(obuf (current-buffer)))
     (unwind-protect
 	(progn
 	  (cond ((or (null val-type) (eq val-type 'sexpression))
 		 (setq read-func 'read-minibuffer
-		       hargs:reading-p 'sexpression))
-		(t (setq read-func 'read-string hargs:reading-p val-type
+		       hargs:reading-type 'sexpression))
+		(t (setq read-func 'read-string hargs:reading-type val-type
 			 stringify t)))
 	  (while (progn (and default (not (stringp default))
 			     (setq default (prin1-to-string default)))
@@ -597,10 +588,15 @@ string read or nil."
 	      (message err)
 	      (sit-for 3)))
 	  val)
-      (setq hargs:reading-p prev-reading-p)
+      (setq hargs:reading-type prev-reading-p)
       (select-window owind)
       (switch-to-buffer obuf))))
 
+(defun hargs:read-buffer-name (prompt)
+  "Use PROMPT to read an existing buffer name and return it."
+  (hargs:read-match prompt (mapcar #'buffer-name (buffer-list)) nil t nil 'buffer))
+
+
 (defun hargs:read-match (prompt collection
 			 &optional predicate must-match initial-input val-type)
   "PROMPT with completion for a value in COLLECTION and return it.
@@ -612,28 +608,28 @@ means value returned must be from COLLECTION.  Optional INITIAL-INPUT
 is a string inserted after PROMPT as the default value.  Optional
 VAL-TYPE is a symbol indicating the type of value to be read."
   (unless (and must-match (null collection))
-    (let ((prev-reading-p hargs:reading-p)
+    (let ((prev-reading-p hargs:reading-type)
 	  (completion-ignore-case t)
 	  (owind (selected-window))
 	  (obuf (current-buffer))
 	  result)
       (unwind-protect
 	  (progn
-	    (setq hargs:reading-p (or val-type t)
+	    (setq hargs:reading-type (or val-type t)
 		  result (completing-read prompt collection predicate must-match initial-input))
 	    (if (and (equal result "") initial-input)
 		initial-input
 	      result))
-	(setq hargs:reading-p prev-reading-p)
+	(setq hargs:reading-type prev-reading-p)
 	(select-window owind)
 	(switch-to-buffer obuf)))))
 
-(defun hargs:select-p (&optional value assist-flag)
+(defun hargs:select-p (&optional value assist-bool)
   "Return optional VALUE or value selected at point if any, else nil.
 If value is the same as the contents of the minibuffer, it is used as
 the current minibuffer argument, otherwise, the minibuffer is erased
 and value is inserted there.
-Optional ASSIST-FLAG non-nil triggers display of Hyperbole menu item
+Optional ASSIST-BOOL non-nil triggers display of Hyperbole menu item
 help when appropriate."
     (when (and (> (minibuffer-depth) 0) (or value (setq value (hargs:at-p))))
       (let ((owind (selected-window)) (back-to)
@@ -647,8 +643,9 @@ help when appropriate."
 	      (cond
 	       ;;
 	       ;; Selecting a menu item
-	       ((eq hargs:reading-p 'hmenu)
-		(if assist-flag (setq hargs:reading-p 'hmenu-help))
+	       ((eq hargs:reading-type 'hmenu)
+		(when assist-bool
+		  (setq hargs:reading-type 'hmenu-help))
 		(hui:menu-enter str-value))
 	       ;;
 	       ;; Enter existing value into the minibuffer as the desired parameter.
@@ -666,182 +663,171 @@ help when appropriate."
 ;;; Private variables
 ;;; ************************************************************************
 
-(defvar hargs:iforms nil
-  "Alist of (interactive-cmd-chr . (argument-type . get-argument-form)) elts.")
-(setq   hargs:iforms
-	'(
-	  ;; Get function symbol.
-	  (?a . (symbol .
-		 (intern (completing-read prompt obarray 'fboundp t default))))
-	  ;; Get name of existing buffer.
-	  (?b . (buffer .
+(defconst hargs:iform-vector
+  (hargs:make-iform-vector
+   ;; Get function symbol.
+   (?a . (symbol .
+		 (intern (completing-read prompt obarray #'fboundp t default))))
+   ;; Get name of existing buffer.
+   (?b . (buffer .
 		 (progn
 		   (or default (setq default (other-buffer (current-buffer))))
 		   (read-buffer prompt default t))))
-	  ;; Get name of possibly nonexistent buffer.
-	  (?B . (buffer .
+   ;; Get name of possibly nonexistent buffer.
+   (?B . (buffer .
 		 (progn
 		   (or default (setq default (other-buffer (current-buffer))))
 		   (read-buffer prompt default nil))))
-	  ;; Get character.
-	  (?c . (character .
-		 (progn (message
-			 (if default
-			     (hargs:prompt prompt
-					   (if (integerp default)
-					       (char-to-string default)
-					     default)
-					   "Curr:")
-			   prompt))
-			(char-to-string (read-char)))))
-	  ;; Get symbol for interactive function, a command.
-	  (?C . (symbol .
+   ;; Get character.
+   (?c . (character .
+		    (progn (message
+			    (if default
+			        (hargs:prompt prompt
+					      (if (integerp default)
+					          (char-to-string default)
+					        default)
+					      "Curr:")
+			      prompt))
+			   (char-to-string (read-char)))))
+   ;; Get symbol for interactive function, a command.
+   (?C . (symbol .
 		 (intern
-		  (completing-read prompt obarray 'commandp t default))))
-	  ;; Get value of point; does not do I/O.
-	  (?d . (integer . (point)))
-	  ;; Get directory name.
-	  (?D . (directory .
-		 (progn
-		   (or default (setq default default-directory))
-		   (read-file-name prompt default default 'existing))))
-	  ;; Get existing file name.
-	  (?f . (file .
-		 (read-file-name prompt default default
-				 (if (eq system-type 'vax-vms)
-				     nil 'existing))))
-	  ;; Get possibly nonexistent file name.
-	  (?F . (file . (read-file-name prompt default default nil)))
-	  ;; Get key sequence.
-	  (?k . (key .
-		 (key-description (read-key-sequence
-				   (if default
-				       (hargs:prompt prompt default "Curr:")
-				     prompt)))))
-	  ;; Get key sequence without converting uppercase or shifted
-	  ;; function keys to their unshifted equivalents.
-	  (?K . (key .
-		 (key-description (read-key-sequence
-				   (if default
-				       (hargs:prompt prompt default "Curr:")
-				     prompt)
-				   nil t))))
-	  ;; Get value of mark.  Does not do I/O.
-	  (?m . (integer . (marker-position (hypb:mark-marker t))))
-	  ;; Get numeric prefix argument or a number from the minibuffer.
-	  (?N . (integer .
-		 (if prefix-arg
-		     (prefix-numeric-value prefix-arg)
-		   (let ((arg))
-		     (while (not (integerp
-				  (setq arg (read-minibuffer prompt default))))
-		       (beep))
-		     arg))))
-	  ;; Get number from minibuffer.
-	  (?n . (integer .
-		 (let ((arg))
-		   (while (not (integerp
-				(setq arg (read-minibuffer prompt default))))
-		     (beep))
-		   arg)))
-	  ;; Get numeric prefix argument.  No I/O.
-	  (?p . (prefix-arg .
-		 (prefix-numeric-value prefix-arg)))
-	  ;; Get prefix argument in raw form.  No I/O.
-	  (?P . (prefix-arg . prefix-arg))
-	  ;; Get region, point and mark as 2 args.  No I/O
-	  (?r . (region .
-		 (if (marker-position (hypb:mark-marker t))
-		     (list 'args (min (point) (hypb:mark t))
-			   (max (point) (hypb:mark t)))
+		  (completing-read prompt obarray #'commandp t default))))
+   ;; Get value of point; does not do I/O.
+   (?d . (integer . (point)))
+   ;; Get directory name.
+   (?D . (directory .
+		    (progn
+		      (or default (setq default default-directory))
+		      (read-file-name prompt default default 'existing))))
+   ;; Get existing file name.
+   (?f . (file .
+	       (read-file-name prompt default default
+			       (if (eq system-type 'vax-vms)
+				   nil 'existing))))
+   ;; Get possibly nonexistent file name.
+   (?F . (file . (read-file-name prompt default default nil)))
+   ;; Get key sequence.
+   (?k . (key .
+	      (key-description (read-key-sequence
+				(if default
+				    (hargs:prompt prompt default "Curr:")
+				  prompt)))))
+   ;; Get key sequence without converting uppercase or shifted
+   ;; function keys to their unshifted equivalents.
+   (?K . (key .
+	      (key-description (read-key-sequence
+				(if default
+				    (hargs:prompt prompt default "Curr:")
+				  prompt)
+				nil t))))
+   ;; Get value of mark.  Does not do I/O.
+   (?m . (integer . (marker-position (mark-marker))))
+   ;; Get numeric prefix argument or a number from the minibuffer.
+   (?N . (integer .
+		  (if prefix-arg
+		      (prefix-numeric-value prefix-arg)
+		    (let ((arg))
+		      (while (not (integerp
+				   (setq arg (read-minibuffer prompt default))))
+		        (beep))
+		      arg))))
+   ;; Get number from minibuffer.
+   (?n . (integer .
+		  (let ((arg))
+		    (while (not (integerp
+				 (setq arg (read-minibuffer prompt default))))
+		      (beep))
+		    arg)))
+   ;; Get numeric prefix argument.  No I/O.
+   (?p . (prefix-arg .
+		     (prefix-numeric-value prefix-arg)))
+   ;; Get prefix argument in raw form.  No I/O.
+   (?P . (prefix-arg . prefix-arg))
+   ;; Get region, point and mark as 2 args.  No I/O
+   (?r . (region .
+		 (if (marker-position (mark-marker))
+		     (list 'args (min (point) (mark t))
+			   (max (point) (mark t)))
 		   (list 'args nil nil))))
-	  ;; Get string.
-	  (?s . (string . (read-string prompt default)))
-	  ;; Get symbol.
-	  (?S . (symbol .
+   ;; Get string.
+   (?s . (string . (read-string prompt default)))
+   ;; Get symbol.
+   (?S . (symbol .
 		 (read-from-minibuffer
 		  prompt default minibuffer-local-ns-map 'sym)))
-	  ;; Get variable name: symbol that is user-variable-p.
-	  (?v . (symbol . (read-variable
-			   (if default
-			       (hargs:prompt prompt default "Curr:")
-			     prompt))))
-	  ;; Get Lisp expression but don't evaluate.
-	  (?x . (sexpression . (read-minibuffer prompt default)))
-	  ;; Get Lisp expression and evaluate.
-	  (?X . (sexpression . (eval-minibuffer prompt default)))
-	  ))
-
-(defvar hargs:iform-vector nil
+   ;; Get variable name: symbol that is user-variable-p.
+   (?v . (symbol . (read-variable
+		    (if default
+			(hargs:prompt prompt default "Curr:")
+		      prompt))))
+   ;; Get Lisp expression but don't evaluate.
+   (?x . (sexpression . (read-minibuffer prompt default)))
+   ;; Get Lisp expression and evaluate.
+   (?X . (sexpression . (eval-minibuffer prompt default))))
   "Vector of forms for each interactive command character code.")
-(setq   hargs:iform-vector (hargs:make-iform-vector hargs:iforms))
 
-(defvar hargs:iforms-extensions nil
-  "Hyperbole extension alist of (interactive-cmd-chr . (argument-type . get-argument-form)) elts.")
-(setq   hargs:iforms-extensions
-	'(
-	  ;; Get existing Info node name, possibly prefixed with its (filename).
-	  (?I . (Info-node .
-	         (let ((prev-reading-p hargs:reading-p))
-		   (unwind-protect
-		       (progn (require 'info)
-			      (setq hargs:reading-p 'Info-node)
-			      ;; Prevent empty completions list from
-			      ;; triggering an error in Info-read-node-name.
-			      (unless Info-current-file-completions
-				(condition-case nil
-				    (Info-build-node-completions)
-				  (error (setq Info-current-file-completions '(("None"))))))
-			      (Info-read-node-name prompt))
-		     (setq hargs:reading-p prev-reading-p)))))
-	  ;; Get kcell from koutline.
-	  (?K . (kcell . (hargs:read-match
-			  prompt
-			  ;; Match to visible cell labels only
-			  (kview:map-tree (lambda (view) (kcell-view:label)) kview t t)
-			  nil t (kcell-view:visible-label) 'kcell)))
-	  ;; Get kcell or path reference for use in a link.
-	  (?L . (klink . (hargs:read prompt nil default nil 'klink)))
-	  ;; Get existing mail msg date and file.
-	  (?M . (mail . (progn
-			  (while
-			      (or (not (listp
-					(setq default
-					      (read-minibuffer
-					       (hargs:prompt
-						prompt ""
-						"list of (date mail-file)")
-					       default))))
-				  (/= (length default) 2)
-				  (not (and (stringp (car (cdr default)))
-					    (file-exists-p
-					     (car (cdr default))))))
-			    (beep))
-			  default)))
-	  ;; Get a Koutline viewspec.
-	  (?V . (kvspec . (hargs:read prompt nil nil nil 'kvspec)))
-	  ;; Get existing Info index item name, possibly prefixed with its (filename).
-	  (?X . (Info-index-item .
-	         (let ((prev-reading-p hargs:reading-p))
-		   (unwind-protect
-		       (let (file item)
-			 (require 'info)
-			 (setq hargs:reading-p 'Info-index-item
-			       item (Info-read-index-item-name prompt))
-			 (if (string-match "^(\\([^\)]+\\))\\(.*\\)" item)
-			     item
-			   (if (setq file (Info-current-filename-sans-extension))
-			       (format "(%s)%s" file item)
-			     item)))
-		     (setq hargs:reading-p prev-reading-p)))))))
-
-(defvar hargs:iform-extensions-vector nil
+(defconst hargs:iform-extensions-vector
+  (hargs:make-iform-vector
+   ;; Get existing Info node name, possibly prefixed with its (filename).
+   (?I . (Info-node .
+	            (let ((prev-reading-p hargs:reading-symbol))
+		      (unwind-protect
+		          (progn (require 'info)
+			         (setq hargs:reading-symbol 'Info-node)
+			         ;; Prevent empty completions list from
+			         ;; triggering an error in Info-read-node-name.
+			         (unless Info-current-file-completions
+				   (condition-case nil
+				       (Info-build-node-completions)
+				     (error (setq Info-current-file-completions '(("None"))))))
+			         (Info-read-node-name prompt))
+		        (setq hargs:reading-symbol prev-reading-p)))))
+   ;; Get kcell from koutline.
+   (?K . (kcell . (hargs:read-match
+		   prompt
+		   ;; Match to "0" and visible cell labels only
+		   (cons "0"
+			 (kview:map-tree (lambda (_kview) (kcell-view:label)) kview t t))
+		   nil t (kcell-view:visible-label) 'kcell)))
+   ;; Get kcell or path reference for use in a link.
+   (?L . (klink . (hargs:read prompt nil default nil 'klink)))
+   ;; Get existing mail msg date and file.
+   (?M . (mail . (progn
+		   (while
+		       (or (not (listp
+				 (setq default
+				       (read-minibuffer
+					(hargs:prompt
+					 prompt ""
+					 "list of (date mail-file)")
+					default))))
+			   (/= (length default) 2)
+			   (not (and (stringp (car (cdr default)))
+				     (file-exists-p
+				      (car (cdr default))))))
+		     (beep))
+		   default)))
+   ;; Get a Koutline viewspec.
+   (?V . (kvspec . (hargs:read prompt nil nil nil 'kvspec)))
+   ;; Get existing Info index item name, possibly prefixed with its (filename).
+   (?X . (Info-index-item .
+	                  (let ((prev-reading-p hargs:reading-symbol))
+		            (unwind-protect
+		                (let (file item)
+			          (require 'info)
+			          (setq hargs:reading-symbol 'Info-index-item
+			                item (Info-read-index-item-name prompt))
+			          (if (string-match "^(\\([^\)]+\\))\\(.*\\)" item)
+			              item
+			            (if (setq file (Info-current-filename-sans-extension))
+			                (format "(%s)%s" file item)
+			              item)))
+		              (setq hargs:reading-symbol prev-reading-p))))))
   "Vector of forms for each interactive command character code.")
-(setq   hargs:iform-extensions-vector
-	(hargs:make-iform-vector hargs:iforms-extensions))
 
 (defvar hargs:string-to-complete nil
-  "The string in the minibuffer the last time a completions buffer was generated, or nil.")
+  "Minibuffer content the last time a completions buffer was generated, or nil.")
 
 (provide 'hargs)
 
