@@ -1,10 +1,11 @@
-;;; hbdata.el --- GNU Hyperbole button attribute accessor functions
+;;; hbdata.el --- GNU Hyperbole button attribute accessor functions  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     2-Apr-91
+;; Last-Mod:     17-Apr-22 at 12:38:53 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -50,6 +51,7 @@
 ;;; ************************************************************************
 
 (require 'hbmap)
+(require 'hgnus)
 
 ;;; ************************************************************************
 ;;; Public functions
@@ -64,15 +66,23 @@
 
 (defun hbdata:actype (hbdata)
   "Return the action type in HBDATA as a string."
-  (let ((nm (symbol-name (nth 3 hbdata))))
+  (let ((nm (symbol-name (nth 3 hbdata)))
+	actype-sym)
     (and nm (if (or (= (length nm) 2) (string-match "::" nm))
-		nm (concat "actypes::" nm)))))
+		nm
+	      ;; RSW 2020-11-01 - Updated to handle programmatically created
+	      ;; explicit buttions whose action types may be non-actypes,
+	      ;; regular Elisp functions.
+	      (setq actype-sym (intern-soft (concat "actypes::" nm)))
+              (if (and actype-sym (fboundp actype-sym))
+		  (symbol-name actype-sym)
+		nm)))))
 
 (defun hbdata:args (hbdata)
   "Return the list of any arguments given in HBDATA."
   (nth 4 hbdata))
 
-(defun hbdata:categ (hbdata)
+(defun hbdata:categ (_hbdata)
   "Return the category of HBDATA's button."
   'explicit)
 
@@ -90,7 +100,7 @@
 
 (defun hbdata:loc-p (hbdata)
   "[Hyp V1] Return 'L iff HBDATA referent is within a local file system.
-Returns 'R if remote and nil if irrelevant for button action type."
+Return 'R if remote and nil if irrelevant for button action type."
   (nth 1 hbdata))
 
 (defun hbdata:modifier (hbdata)
@@ -135,12 +145,11 @@ Search is case-insensitive.  Return list with elements:
 ;;; ------------------------------------------------------------------------
 
 (defun hbdata:build (&optional mod-lbl-key but-sym)
-  "Construct button data from optional MOD-LBL-KEY and BUT-SYM; modifies BUT-SYM attributes.
+  "Construct button data from optional MOD-LBL-KEY and BUT-SYM; modify BUT-SYM attributes.
 MOD-LBL-KEY nil means create a new entry, otherwise modify existing one.
-Nil BUT-SYM means use 'hbut:current'.  If successful, returns a cons of
+Nil BUT-SYM means use 'hbut:current'.  If successful, return a cons of
  (button-data . button-instance-str), else nil."
-  (let* ((but)
-	 (b (hattr:copy (or but-sym 'hbut:current) 'but))
+  (let* ((b (hattr:copy (or but-sym 'hbut:current) 'but))
 	 (l (hattr:get b 'loc))
 	 (key (or mod-lbl-key (hattr:get b 'lbl-key)))
 	 (new-key (if mod-lbl-key (hattr:get b 'lbl-key) key))
@@ -164,10 +173,11 @@ Nil BUT-SYM means use 'hbut:current'.  If successful, returns a cons of
 	      (when (setq lbl-instance (hbdata:instance-last new-key loc dir))
 		(setq lbl-instance (concat ebut:instance-sep
 					   (int-to-string (1+ lbl-instance))))
-		;; This line is needed to ensure that the highest
+		;; This expression is needed to ensure that the highest
 		;; numbered instance of a label appears before
 		;; other instances, so 'hbdata:instance-last' will work.
-		(if (hbdata:to-entry-buf loc dir) (forward-line 1))))
+		(when (hbdata:to-entry-buf loc dir)
+		  (forward-line 1))))
 	  (let ((inst-num (hbdata:instance-last new-key loc dir)))
 	    (setq lbl-instance (if inst-num
 				   (hbdata:instance-next
@@ -177,38 +187,39 @@ Nil BUT-SYM means use 'hbut:current'.  If successful, returns a cons of
 	(hattr:set b 'lbl-key (concat new-key lbl-instance))
 	(hattr:set b 'loc loc)
 	(hattr:set b 'dir dir)
-	(let ((hbdata (list (hattr:get b 'lbl-key)
-			    (hattr:get b 'action)
-			    ;; Hyperbole V1 referent compatibility, always nil in V2
-			    (hattr:get b 'referent)
-			    ;; Save actype without class prefix.
-			    (let ((actype (hattr:get b 'actype)))
-			      (and actype (symbolp actype)
-				   (setq actype (symbol-name actype))
-				   (intern
-				    (substring actype (if (string-match "::" actype)
-							  (match-end 0) 0)))))
-			    (let ((mail-dir (and (fboundp 'hmail:composing-dir)
-						 (hmail:composing-dir l)))
-				  (args (hattr:get b 'args)))
-			      ;; Replace matches for variable values with their variable names in any pathname args.
-			      (hattr:set b 'args
-					 (mapcar 'hpath:substitute-var
-						 (if mail-dir
-						     ;; Make pathname args absolute for outgoing mail and news messages.
-						     (action:path-args-abs args mail-dir)
-						   args))))
-			    (hattr:set b 'creator (or creator hyperb:user-email))
-			    (hattr:set b 'create-time (or create-time (htz:date-sortable-gmt)))
-			    (hattr:set b 'modifier modifier)
-			    (hattr:set b 'mod-time mod-time))))
+	(let* ((actype)
+	       (hbdata (list (hattr:get b 'lbl-key)
+			     (hattr:get b 'action)
+			     ;; Hyperbole V1 referent compatibility, always nil in V2
+			     (hattr:get b 'referent)
+			     ;; Save actype without class prefix.
+			     (and (setq actype (hattr:get b 'actype))
+				  (symbolp actype)
+				  (setq actype (symbol-name actype))
+				  (intern
+				   (substring actype (if (string-match "::" actype)
+							 (match-end 0) 0))))
+			     (let ((mail-dir (and (fboundp 'hmail:composing-dir)
+						  (hmail:composing-dir l)))
+				   (args (hattr:get b 'args)))
+			       ;; Replace matches for variable values with their variable names in any pathname args.
+			       (hattr:set b 'args
+					  (mapcar #'hpath:substitute-var
+						  (if mail-dir
+						      ;; Make pathname args absolute for outgoing mail and news messages.
+						      (hpath:absolute-arguments actype args mail-dir)
+						    args))))
+			     (hattr:set b 'creator (or creator hyperb:user-email))
+			     (hattr:set b 'create-time (or create-time (htz:date-sortable-gmt)))
+			     (hattr:set b 'modifier modifier)
+			     (hattr:set b 'mod-time mod-time))))
 	  ;; Ensure modified attributes are saved to `but-sym' or hbut:current.
 	  (hattr:copy b (or but-sym 'hbut:current))
 	  (cons hbdata lbl-instance))))))
 
 (defun hbdata:get-entry (lbl-key key-src &optional directory)
   "Return button data entry given by LBL-KEY, KEY-SRC and optional DIRECTORY.
-Returns nil if no matching entry is found.
+Return nil if no matching entry is found.
 A button data entry is a list of attribute values.  Use methods from
 class 'hbdata' to operate on the entry."
   (hbdata:apply-entry
@@ -230,7 +241,7 @@ Nil if LBL-KEY is nil."
 (defun hbdata:instance-last (lbl-key key-src &optional directory)
   "Return highest instance number for repeated button label.
 1 if not repeated, nil if no instance.
-Takes arguments LBL-KEY, KEY-SRC and optional DIRECTORY."
+Utilize arguments LBL-KEY, KEY-SRC and optional DIRECTORY."
   (hbdata:apply-entry
    (lambda ()
      (if (looking-at "[0-9]+")
@@ -239,9 +250,10 @@ Takes arguments LBL-KEY, KEY-SRC and optional DIRECTORY."
    lbl-key key-src directory nil 'instance))
 
 (defun hbdata:delete-entry (lbl-key key-src &optional directory)
-  "Deletes button data entry given by LBL-KEY, KEY-SRC and optional DIRECTORY.
-Returns entry deleted (a list of attribute values) or nil.
-Use methods from class 'hbdata' to operate on the entry."
+  "Delete button data entry given by LBL-KEY, KEY-SRC and optional DIRECTORY.
+Return entry deleted (a list of attribute values) or nil.
+Use methods from class 'hbdata' to operate on the entry.
+If the hbdata buffer is blank/empty, kill it and remove the associated file."
   (hbdata:apply-entry
    (lambda ()
      (prog1 (read (current-buffer))
@@ -249,22 +261,23 @@ Use methods from class 'hbdata' to operate on the entry."
 	     (kill))
 	 (beginning-of-line)
 	 (hbdata:delete-entry-at-point)
-	 (if (looking-at empty-file-entry)
-	     (let ((end (point))
-		   (empty-hbdata-file "[ \t\n\r]*\\'"))
-	       (forward-line -1)
-	       (if (eq (following-char) ?\")
-		   ;; Last button entry for filename, so del filename.
-		   (progn (forward-line -1) (delete-region (point) end)))
-	       (save-excursion
-		 (goto-char (point-min))
-		 (if (looking-at empty-hbdata-file)
-		     (setq kill t)))
-	       (if kill
-		   (let ((fname buffer-file-name))
-		     (erase-buffer) (save-buffer) (kill-buffer nil)
-		     (hbmap:dir-remove (file-name-directory fname))
-		     (call-process "rm" nil 0 nil "-f" fname))))))))
+	 (when (looking-at empty-file-entry)
+	   (let ((end (point))
+		 (empty-hbdata-file "[ \t\n\r]*\\'"))
+	     (forward-line -1)
+	     (when (eq (following-char) ?\")
+		 ;; Last button entry for filename, so del filename.
+		 (forward-line -1)
+		 (delete-region (point) end))
+	     (save-excursion
+	       (goto-char (point-min))
+	       (when (looking-at empty-hbdata-file)
+		 (setq kill t)))
+	     (when kill
+	       (let ((fname buffer-file-name))
+		 (erase-buffer) (save-buffer) (kill-buffer nil)
+		 (hbmap:dir-remove (file-name-directory fname))
+		 (delete-file fname))))))))
    lbl-key key-src directory))
 
 (defun hbdata:delete-entry-at-point ()
@@ -272,7 +285,7 @@ Use methods from class 'hbdata' to operate on the entry."
 
 (defun hbdata:to-entry (but-key key-src &optional directory instance)
   "Return button data entry indexed by BUT-KEY, KEY-SRC, optional DIRECTORY.
-Returns nil if entry is not found.  Leaves point at start of entry when
+Return nil if entry is not found.  Leave point at start of entry when
 successful or where entry should be inserted if unsuccessful.
 A button entry is a list.  Use methods from class 'hbdata' to operate on the
 entry.  Optional INSTANCE non-nil means search for any button instance matching
@@ -285,108 +298,120 @@ but-key."
 	  but-key key-src directory 'create instance)))
     (hbdata:to-entry-buf key-src directory)
     (forward-line 1)
-    (if pos-entry-cons
-	(progn
-	  (goto-char (car pos-entry-cons))
-	  (cdr pos-entry-cons)))))
+    (when pos-entry-cons
+      (goto-char (car pos-entry-cons))
+      (cdr pos-entry-cons))))
 
 ;;; ************************************************************************
 ;;; Private functions
 ;;; ************************************************************************
 
-(defun hbdata:apply-entry (function lbl-key key-src &optional directory
-			   create instance)
-  "Invoke FUNCTION with point at hbdata entry given by LBL-KEY, KEY-SRC, optional DIRECTORY.
-With optional CREATE, if no such line exists, inserts a new file entry at the
+(defun hbdata:apply-entry (func lbl-key key-src &optional directory
+			   create-flag instance-flag)
+  "Invoke FUNC with point at hbdata entry given by LBL-KEY, KEY-SRC, optional DIRECTORY.
+With optional CREATE-FLAG, if no such line exists, insert a new file entry at the
 beginning of the hbdata file (which is created if necessary).
-INSTANCE non-nil means search for any button instance matching LBL-KEY and
-call FUNCTION with point right after any 'ebut:instance-sep' in match.
-Returns value of evaluation when a matching entry is found or nil."
-  (let ((found)
-	(rtn)
-	(opoint)
-	(end-func))
+INSTANCE-FLAG non-nil means search for any button instance matching LBL-KEY and
+call FUNC with point right after any 'ebut:instance-sep' in match.
+Return value of evaluation when a matching entry is found or nil."
+  (let (found
+	rtn
+	opoint
+	end-func)
     (save-excursion
-      (unwind-protect
-	  (progn
-	    (if (not (bufferp key-src))
-		nil
-	      (set-buffer key-src)
-	      (cond ((hmail:editor-p)
-		     (setq end-func (lambda ()
-				      (hmail:msg-narrow))))
-		    ((and (hmail:lister-p)
-			  (progn (rmail:summ-msg-to) (rmail:to)))
-		     (setq opoint (point)
-			   key-src (current-buffer)
-			   end-func (lambda ()
-				      (hmail:msg-narrow)
-				      (goto-char opoint)
-				      (lmail:to))))
-		    ((and (hnews:lister-p)
-			  (progn (rnews:summ-msg-to) (rnews:to)))
-		     (setq opoint (point)
-			   key-src (current-buffer)
-			   end-func (lambda ()
-				      (hmail:msg-narrow)
-				      (goto-char opoint)
-				      (lnews:to))))))
-	    (setq found (hbdata:to-entry-buf key-src directory create)))
-	(if found
+      (save-restriction
+	(unwind-protect
+	    (progn
+	      (when (get-buffer key-src)
+		(set-buffer key-src)
+		(unless buffer-file-name
+		  (cond ((hmail:editor-p)
+			 (setq end-func (lambda ()
+					  (hmail:msg-narrow))))
+			((and (hmail:lister-p)
+			      (progn (rmail:summ-msg-to) (rmail:to)))
+			 (setq opoint (point)
+			       key-src (current-buffer)
+			       end-func (lambda ()
+					  (hmail:msg-narrow)
+					  (goto-char opoint)
+					  (lmail:to))))
+			((and (hnews:lister-p)
+			      (progn (rnews:summ-msg-to) (rnews:to)))
+			 (setq opoint (point)
+			       key-src (current-buffer)
+			       end-func (lambda ()
+					  (hmail:msg-narrow)
+					  (goto-char opoint)
+					  (lnews:to))))
+			;; Any non-file buffer
+			(t
+			 (setq opoint (point)
+			       key-src (current-buffer)
+			       end-func (lambda ()
+					  (widen)
+					  (goto-char opoint)
+					  (narrow-to-region (point-min)
+							    (hmail:hbdata-start))))))))
+	      (setq found (hbdata:to-entry-buf key-src directory create-flag)))
+	  (when found
 	    (let ((case-fold-search t)
 		  (qkey (regexp-quote lbl-key))
 		  (end (save-excursion (if (search-forward "\n\^L" nil t)
 					   (point) (point-max)))))
-	      (if (if instance
+	      (if (if instance-flag
 		      (re-search-forward
 		       (concat "\n(\"" qkey "["
 			       ebut:instance-sep "\"]") end t)
 		    (search-forward (concat "\n(\"" lbl-key "\"") end t))
 		  (progn
-		    (or instance (beginning-of-line))
+		    (unless instance-flag
+		      (beginning-of-line))
 		    (let (buffer-read-only)
-		      (setq rtn (funcall function)))))))
-	(if end-func (funcall end-func))))
+		      (setq rtn (funcall func)))))))
+	  (when end-func (funcall end-func)))))
     rtn))
 
 (defun hbdata:to-hbdata-buffer (dir &optional create)
   "Read in the file containing DIR's button data, if any, and return buffer.
-If it does not exist and optional CREATE is non-nil, creates a new
-one and returns buffer, otherwise returns nil."
+If it does not exist and optional CREATE is non-nil, create a new
+one and return buffer, otherwise return nil."
   (let* ((file (expand-file-name hattr:filename (or dir default-directory)))
 	 (existing-file (or (file-exists-p file) (get-file-buffer file)))
 	 (buf (or (get-file-buffer file)
 		  (and (or create existing-file)
 		       (find-file-noselect file)))))
-    (if buf
-	(progn (set-buffer buf)
-	       (or (verify-visited-file-modtime (get-file-buffer file))
-		   (cond ((yes-or-no-p
-			   "Hyperbole button data file has changed, read new contents? ")
-			  (revert-buffer t t)
-			  )))
-	       (or (= (point-max) 1) (eq (char-after 1) ?\^L)
-		   (error "File %s is not a valid Hyperbole button data table" file))
-	       (or (equal (buffer-name) file) (rename-buffer file))
-	       (setq buffer-read-only nil)
-	       (or existing-file (hbmap:dir-add (file-name-directory file)))
-	       buf))))
+    (when buf
+      (set-buffer buf)
+      (unless (verify-visited-file-modtime (get-file-buffer file))
+	(cond ((yes-or-no-p
+		"Hyperbole button data file has changed, read new contents? ")
+	       (revert-buffer t t))))
+      (or (= (point-max) 1) (eq (char-after 1) ?\^L)
+	  (error "File %s is not a valid Hyperbole button data table" file))
+      (unless (equal (buffer-name) file)
+	(rename-buffer file))
+      (setq buffer-read-only nil)
+      (unless existing-file
+	(hbmap:dir-add (file-name-directory file)))
+      buf)))
 
 
 (defun hbdata:to-entry-buf (key-src &optional directory create)
   "Move point to end of line in but data buffer matching KEY-SRC.
-Uses hbdata file in KEY-SRC's directory, or optional DIRECTORY or if nil, uses
+Use hbdata file in KEY-SRC's directory, or optional DIRECTORY or if nil, use
 `default-directory'.
-With optional CREATE, if no such line exists, inserts a new file entry at the
+With optional CREATE, if no such line exists, insert a new file entry at the
 beginning of the hbdata file (which is created if necessary).
-Returns non-nil if KEY-SRC is found or created, else nil."
+Return non-nil if KEY-SRC is found or created, else nil."
   (let ((rtn) (ln-dir))
-    (if (bufferp key-src)
+    (if (and (get-buffer key-src)
+	     (setq rtn (set-buffer key-src))
+	     (not buffer-file-name))
 	;; Button buffer has no file attached
-	(progn (setq rtn (set-buffer key-src)
-		     buffer-read-only nil)
-	       (if (not (hmail:hbdata-to-p))
-		   (insert "\n" hmail:hbdata-sep "\n"))
+	(progn (setq buffer-read-only nil)
+	       (unless (hmail:hbdata-to-p)
+		 (insert "\n" hmail:hbdata-sep "\n"))
 	       (backward-char 1))
       (setq directory (or (file-name-directory key-src) directory))
       (let ((ln-file) (link-p key-src))
@@ -419,16 +444,12 @@ a button instance string to append to button label or t when first instance.
 On failure, return nil."
   (let ((cons (hbdata:build orig-lbl-key but-sym))
 	entry lbl-instance)
-    (if (or (and buffer-file-name
-		 (not (file-writable-p buffer-file-name)))
-	    (null cons))
-	nil
+    (unless (or (and buffer-file-name (not (file-writable-p buffer-file-name)))
+		(null cons))
       (setq entry (car cons) lbl-instance (cdr cons))
       (prin1 entry (current-buffer))
       (terpri (current-buffer))
-      (or lbl-instance t)
-      )))
-
+      (or lbl-instance t))))
 
 ;;; ************************************************************************
 ;;; Private variables
