@@ -1,35 +1,44 @@
-;;; hib-kbd.el --- Implicit button type for key sequences delimited with {}.
+;;; hib-kbd.el --- Implicit button type for key sequences delimited with {}.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    22-Nov-91 at 01:37:57
+;; Last-Mod:     24-Apr-22 at 17:34:04 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
 ;;
 ;;; Commentary:
 ;;
-;;   A press of the Action Key on any sequence of keys delimited by braces
-;;   executes its command binding or Hyperbole minibuffer menu binding.
+;;   A press of the Action Key on any series of key sequences delimited by
+;;   curly braces executes all of the associated commands.  Key sequences
+;;   can include Hyperbole minibuffer menu sequences.
 ;;
-;;   A press of the Assist Key on any sequence of keys delimited by braces
-;;   displays the documentation for it.
+;;   A press of the Assist Key on any series of key sequences delimited by
+;;   curly braces displays the documentation for it.
 ;;
 ;;   Sequences of keys should be in human readable string form with spaces
 ;;   between each key, may contain any number of individual key sequences
 ;;   and the whole thing should be delimited by braces, e.g. {M-x apropos
 ;;   RET hyperbole RET}.  Forms such as {\C-b}, {\^b}, and {^b} will not be
 ;;   recognized.
+;;
+;;   Programmatically, to execute a key series given as a string, use:
+;;   (kbd-key:execute "{key series}").
 
 ;;; Code:
 ;;; ************************************************************************
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(require 'hactypes)
+(require 'hactypes) ;; This invokes (require 'hargs)
+(require 'seq)
 
+;;; ************************************************************************
+;;; Public variables
+;;; ************************************************************************
 (defvar kbd-key:named-key-list
   '("add" "backspace" "begin" "bs" "clear" "decimal" "delete" "del"
     "divide" "down" "end" "enter" "esc" "home" "left" "insert"
@@ -45,92 +54,149 @@
   "Regexp that matches to any of the dedicated keyboard key names in lower or uppercase.")
 
 (defvar kbd-key:modified-key-regexp
-  (concat "\\(\\([ACHMS]-\\|kp-\\)+\\)\\s-*\\(<?\\<" kbd-key:named-key-regexp "\\>>?"
-	  "\\|<?[fF][0-9][0-9]?>?\\|<[a-zA-Z0-9]+>\\|.\\)")
+  (concat "\\(\\[?\\([ACHMS]-\\|kp-\\)+\\)\\s-*\\(\\(<?\\<" kbd-key:named-key-regexp "\\>>?"
+	  "\\|<?[fF][0-9][0-9]?>?\\|<[a-zA-Z0-9]+>\\|.\\)\\]?\\)")
   "Regexp matching to a single modified keyboard key within a human-readable string.
 Group 1 matches to the set of modifier keys.  Group 3 matches to the unmodified key.")
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(declare-function helm-mode "ext:helm")
+
 ;;; ************************************************************************
 ;;; Public implicit button types
 ;;; ************************************************************************
 
 (defact kbd-key (key-series)
-  "Execute a normalized key sequence without curly braces, {}.
-KEY-SERIES must be a string of one of the following:
+  "Execute a normalized KEY-SERIES (series of key sequences) without curly braces, {}.
+Each key sequence within KEY-SERIES must be a string of one of the following:
   a Hyperbole minibuffer menu item key sequence,
   a HyControl key sequence,
   a M-x extended command,
   or a valid key sequence together with its interactive arguments.
 
 Return t if the sequence appears to be valid, else nil."
-  (interactive "kKey sequence to execute (no {}): ")
+  (interactive "sKey series to execute (no {}): ")
   (kbd-key:act key-series))
 
 (defib kbd-key ()
-  "Execute a key sequence found around point, delimited by curly braces, {}, if any.
+  "Execute a key series (series of key sequences) around point, delimited by curly braces, {}.
 Key sequences should be in human readable form, e.g. {C-x C-b}, or what `key-description' returns.
 Forms such as {\C-b}, {\^b}, and {^M} will not be recognized.
 
-Any key sequence must be a string of one of the following:
+Any key sequence within the series must be a string of one of the following:
   a Hyperbole minibuffer menu item key sequence,
   a HyControl key sequence,
   a M-x extended command,
   or a valid key sequence together with its interactive arguments."
   (unless (or (br-in-browser)
 	      (and (looking-at "[{}]") (/= ?\\ (preceding-char))))
-    ;; handle long series, e.g. eval-elisp actions
-    (let* ((hbut:max-len (max 3000 (hbut:max-len)))
-	   (seq-and-pos (or (hbut:label-p t "{`" "'}" t)
-			    (hbut:label-p t "{" "}" t)
-			    ;; Regular dual single quotes (Texinfo smart quotes)
-			    (hbut:label-p t "``" "''" t)
-			    ;; Typical GNU manual key sequences; note
-			    ;; these are special quote marks, not the
-			    ;; standard ASCII characters.
-			    (hbut:label-p t "‘" "’" t)))
-	   ;; This excludes delimiters
-	   (key-series (car seq-and-pos))
-	   (start (cadr seq-and-pos))
-	   binding)
-      ;; Match only when start delimiter is preceded by whitespace,
-      ;; double quotes or is the 1st buffer character, so do not
-      ;; match to things like ${variable}.
-      (when (memq (char-before start) '(nil ?\ ?\t ?\n ?\j ?\f ?\"))
-	(when (and (stringp key-series)
-		   (not (eq key-series "")))
-	  (setq key-series (kbd-key:normalize key-series)
-		binding (kbd-key:binding key-series)))
-	(and (stringp key-series)
-	     (or (and binding (not (integerp binding)))
-		 (kbd-key:special-sequence-p key-series))
-	     (ibut:label-set seq-and-pos)
-	     (hact 'kbd-key key-series))))))
+    ;; Temporarily make open and close braces have list syntax for
+    ;; matching purposes.
+    (let ((open-brace-syntax (hypb:get-raw-syntax-descriptor ?\{))
+	  (close-brace-syntax (hypb:get-raw-syntax-descriptor ?\})))
+      (unwind-protect
+	  (progn (modify-syntax-entry ?\{ "(\}" (syntax-table))
+		 (modify-syntax-entry ?\} ")\}" (syntax-table))
+		 ;; Handle long series, e.g. eval-elisp actions
+		 (let* ((hbut:max-len (max 3000 (hbut:max-len)))
+			(seq-and-pos (or ;; (kbd) calls but only if point is between double quotes
+ 				      (and (hbut:label-p t "(kbd \"" "\"\)" t)
+					   (hbut:label-p t "\"" "\"\)" t))
+				      (hbut:label-p t "{`" "'}" t)
+				      (hbut:label-p t "{" "}" t)
+				      ;; Regular dual single quotes (Texinfo smart quotes)
+				      (hbut:label-p t "``" "''" t)
+				      ;; Typical GNU manual key sequences; note
+				      ;; these are special quote marks, not the
+				      ;; standard ASCII characters.
+				      (hbut:label-p t "‘" "’" t)))
+			;; This excludes delimiters
+			(key-series (car seq-and-pos))
+			(start (cadr seq-and-pos))
+			binding)
+		   ;; Match only when start delimiter is preceded by whitespace,
+		   ;; double quotes or is the 1st buffer character, so do not
+		   ;; match to things like ${variable}.
+		   (when (memq (char-before start) '(nil ?\ ?\t ?\n ?\r ?\f ?\"))
+		     (when (and (stringp key-series)
+				(not (eq key-series "")))
+		       ;; Replace any ${} internal or env vars; leave
+		       ;; $VAR untouched for the shell to evaluate.
+		       (let ((hpath:variable-regexp "\\${\\([^}]+\\)}"))
+			 (setq key-series (hpath:substitute-value key-series)))
+
+		       (setq key-series (kbd-key:normalize key-series)
+			     binding (kbd-key:binding key-series)))
+		     (and (stringp key-series)
+			  (or (and binding (not (integerp binding)))
+			      (kbd-key:special-sequence-p key-series))
+			  (ibut:label-set seq-and-pos)
+			  (hact 'kbd-key key-series)))))
+	(hypb:set-raw-syntax-descriptor ?\{ open-brace-syntax)
+	(hypb:set-raw-syntax-descriptor ?\} close-brace-syntax)))))
 
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
 
 (defun kbd-key:act (key-series)
-  "Execute the command binding for normalized KEY-SERIES.
-Returns t if KEY-SERIES has a binding, else nil."
-  (interactive "kKeyboard key to execute (no {}): ")
+  "Execute the normalized KEY-SERIES.
+Return t if KEY-SERIES appears valid, else nil."
+  (interactive "sKey series to execute (no {}): ")
   (setq current-prefix-arg nil) ;; Execution of the key-series may set it.
   (let ((binding (kbd-key:binding key-series)))
     (cond ((null binding)
-	   ;; If this is a special key seqence, execute it by adding
-	   ;; its keys to the stream of unread command events.
-	   (when (kbd-key:special-sequence-p key-series)
-             (kbd-key:key-series-to-events key-series)
-	     t))
+	   (if (kbd-key:special-sequence-p key-series)
+	       (kbd-key:execute-special-series key-series)
+	     (kbd-key:key-series-to-events key-series))
+	   t)
 	  ((memq binding '(action-key action-mouse-key hkey-either))
 	   (beep)
 	   (message "(kbd-key:act): This key does what the Action Key does.")
 	   t)
-	  (t (call-interactively binding) t))))
+	  ((not (integerp binding))
+	   (call-interactively binding)
+	   t))))
+
+(defun kbd-key:execute (key-series)
+   "Execute a possibly non-normalized KEY-SERIES with or without curly brace delimiters.
+Return t if KEY-SERIES is a valid key series that is executed, else nil."
+  (interactive "sKey series to execute: ")
+  (when (and key-series
+	     (setq key-series (kbd-key:is-p key-series)))
+    (hact #'kbd-key:act key-series)))
+
+(defun kbd-key:execute-special-series (key-series)
+  "Execute key series."
+  (if (memq (key-binding [?\M-x]) #'(execute-extended-command counsel-M-x))
+      (kbd-key:key-series-to-events key-series)
+    ;; Disable helm while processing M-x commands; helm
+    ;; gobbles final RET key.  Counsel works without modification.
+    (let ((orig-binding (global-key-binding [?\M-x]))
+	  (helm-flag (when (boundp 'helm-mode) helm-mode))
+	  (minibuffer-completion-confirm))
+      (unwind-protect
+	  (progn
+	    (when helm-flag (helm-mode -1))
+	    (global-set-key [?\M-x] 'execute-extended-command)
+	    (kbd-key:key-series-to-events key-series))
+	(kbd-key:key-series-to-events
+	 (format "M-: SPC (kbd-key:maybe-enable-helm SPC %s SPC #'%S) RET"
+		 helm-flag orig-binding))))))
+
+(defun kbd-key:maybe-enable-helm (helm-flag orig-M-x-binding)
+  "Enable helm-mode if HELM-FLAG is non-nil.  Restore M-x binding to ORIG-M-X-BINDING."
+  (when helm-flag (helm-mode 1))
+  (global-set-key [?\M-x] orig-M-x-binding))
 
 (defun kbd-key:key-series-to-events (key-series)
-  "Insert the key-series as a series of keyboard events into Emacs' unread input stream."
-  (setq unread-command-events (nconc unread-command-events (listify-key-sequence (kbd-key:kbd key-series)))))
+  "Insert the key-series as a series of keyboard events into Emacs' unread input stream.
+Emacs then executes them when its command-loop regains control."
+  (setq unread-command-events (nconc unread-command-events
+				     (listify-key-sequence
+				      (kbd-key:kbd key-series)))))
 
 (defun kbd-key:doc (key-series &optional full)
   "Show first line of doc for binding of keyboard KEY-SERIES in minibuffer.
@@ -161,9 +227,40 @@ With optional prefix arg FULL, display full documentation for command."
 (defun kbd-key:help (but)
   "Display documentation for binding of keyboard key given by BUT's label."
   (let ((kbd-key (hbut:key-to-label (hattr:get but 'lbl-key))))
-    (when kbd-key
+    (when (and kbd-key (not (string-empty-p kbd-key)))
       (kbd-key:doc kbd-key t))))
 
+(defun kbd-key:is-p (str)
+  "If STR is a curly-brace {} delimited key series, return the non-delimited, normalized form, else nil.
+Key sequences should be in human readable form, e.g. {C-x C-b}, or what `key-description' returns.
+Forms such as {\C-b}, {\^b}, and {^M} will not be recognized.
+
+Any key sequence within the series must be a string of one of the following:
+  a Hyperbole minibuffer menu item key sequence,
+  a HyControl key sequence,
+  a M-x extended command,
+  or a valid key sequence together with its interactive arguments."
+  ;; Handle long series, e.g. eval-elisp actions
+  (let* ((hbut:max-len (max 3000 (hbut:max-len)))
+	 ;; STR must include delimiters but they are stripped from `key-series'.
+	 (key-series (or (kbd-key:remove-delimiters str "{`" "'}")
+			 (kbd-key:remove-delimiters str "{" "}")
+			 ;; Regular dual single quotes (Texinfo smart quotes)
+			 (kbd-key:remove-delimiters str "``" "''")
+			 ;; Typical GNU manual key sequences; note
+			 ;; these are special quote marks, not the
+			 ;; standard ASCII characters.
+			 (kbd-key:remove-delimiters str "‘" "’")))
+	 binding)
+    (when (and (stringp key-series)
+	       (not (eq key-series "")))
+      (setq key-series (kbd-key:normalize key-series)
+	    binding (kbd-key:binding key-series)))
+    (and (stringp key-series)
+	 (or (and binding (not (integerp binding)))
+	     (kbd-key:special-sequence-p key-series))
+	 key-series)))
+
 (defun kbd-key:normalize (key-series)
   "Normalize a human-readable string of keyboard keys, KEY-SERIES (without any surrounding {}).
 Return the normalized but still human-readable format.
@@ -182,8 +279,9 @@ keyboad input queue, as if they had been typed by the user."
 	   (let ((norm-key-series (copy-sequence key-series))
 		 (case-fold-search nil)
 		 (case-replace t)
-		 (substring)
-		 (arg))
+		 ;; (substring)
+		 ;; (arg)
+		 )
 	     (setq norm-key-series (kbd-key:mark-spaces-to-keep norm-key-series "(" ")")
 		   norm-key-series (kbd-key:mark-spaces-to-keep norm-key-series "\\[" "\\]")
 		   norm-key-series (kbd-key:mark-spaces-to-keep norm-key-series "<" ">")
@@ -226,12 +324,18 @@ keyboad input queue, as if they had been typed by the user."
 	     norm-key-series)))
 	(t (error "(kbd-key:normalize): requires a string argument, not `%s'" key-series))))
 
+(defun kbd-key:remove-delimiters (str start-delim end-delim)
+  "Return STR sans START-DELIM and END-DELIM (strings) iff it starts and ends with these delimiters."
+  (when (and (string-match (format "\\`%s" (regexp-quote start-delim)) str)
+	     (string-match (format "%s\\'"  (regexp-quote end-delim)) str))
+    (string-trim str start-delim end-delim)))
+
 ;;; ************************************************************************
 ;;; Private functions
 ;;; ************************************************************************
 
 (defun kbd-key:binding (key-series)
-  "Return any existing key binding for KEY-SERIES or nil."
+  "Return key binding for KEY-SERIES if it is a single key sequence or nil."
   ;; This custom function is used to prevent the (kbd) call from
   ;; mistakenly removing angle brackets from Hyperbole implicit button
   ;; names, like: <[td]>.
@@ -263,12 +367,13 @@ For an approximate inverse of this, see `key-description'."
 	  (setq times (string-to-number (substring word 0 (match-end 1))))
 	  (setq word (substring word (1+ (match-end 1)))))
 	(cond ((string-match "^<<.+>>$" word)
-	       (setq key (vconcat (if (eq (key-binding [?\M-x])
-					  'execute-extended-command)
-				      [?\M-x]
-				    (or (car (where-is-internal
-					      'execute-extended-command))
-					[?\M-x]))
+	       (setq key (vconcat (cond ((memq (key-binding [?\M-x])
+					       kbd-key:extended-command-binding-list)
+					 [?\M-x])
+					((seq-filter
+					  (lambda (elt) (car (where-is-internal elt)))
+					  kbd-key:extended-command-binding-list)
+					 [?\M-x]))
 				  (substring word 2 -2) "\r")))
 	      ((and (string-match "^\\(\\([ACHMsS]-\\)*\\)<\\(.+\\)>$" word)
 		    (progn
@@ -358,8 +463,8 @@ Allows for multiple key sequences strung together."
 (defun kbd-key:hyperbole-mini-menu-key-p (key-series)
   "Return t if normalized KEY-SERIES appears to invoke a Hyperbole menu item or sequence of keys, else nil.
 Also, initialize `kbd-key:mini-menu-key' to the key sequence that invokes the Hyperbole minibuffer menu."
-  (when key-series
-    (unless kbd-key:mini-menu-key
+  (when (stringp key-series)
+    (unless (and (stringp kbd-key:mini-menu-key) (not (string-empty-p kbd-key:mini-menu-key)))
       (setq kbd-key:mini-menu-key (regexp-quote (kbd-key:normalize (key-description (car (where-is-internal 'hyperbole)))))))
     (when (string-match kbd-key:mini-menu-key key-series)
       t)))
@@ -409,8 +514,11 @@ a M-x extended command,
 ;;; ************************************************************************
 
 (defconst kbd-key:extended-command-prefix
-  (kbd-key:normalize (key-description (where-is-internal 'execute-extended-command (current-global-map) t)))
-  "Normalized prefix string that invokes an extended command; typically ESC x.")
+  (format "\\_<%s\\_>" (kbd-key:normalize "M-x"))
+  "Normalized prefix regular expression that invokes an extended command; by default, M-x.")
+
+(defconst kbd-key:extended-command-binding-list '(execute-extended-command helm-M-x counsel-M-x)
+  "List of commands that may be bound to M-x to invoke extended/named commands.")
 
 (defvar kbd-key:mini-menu-key nil
   "The key sequence that invokes the Hyperbole minibuffer menu.")
