@@ -1,10 +1,11 @@
-;;; hib-social.el --- Implicit button type for social media/git hashtag and username references
+;;; hib-social.el --- Implicit button type for social media/git hashtag and username references  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    20-Jul-16 at 22:41:34
+;; Last-Mod:     13-Feb-22 at 19:49:11 by Mats Lidell
 ;;
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -179,7 +180,7 @@
 ;;; ************************************************************************
 
 (defcustom hibtypes-social-default-service "twitter"
-  "Lowercase string matching the name of the default social media service to use when none is specified."
+  "Lowercase string matching the social media service to use as a default."
   :type '(radio (const "facebook")
 		(const "git")
 		(const "github")
@@ -189,7 +190,7 @@
   :group 'hyperbole-button)
 
 (defcustom hibtypes-social-display-function #'browse-url
-  "Function of one argument, a url, to display when a social media reference is activated."
+  "Function of one arg, a url, to display upon activation of a social media reference."
   :type 'function
   :group 'hyperbole-button)
 
@@ -198,6 +199,11 @@
   :type 'string
   :group 'hyperbole-button)
 
+(defcustom hibtypes-git-use-magit-flag nil
+  "If magit is available, when activating a git directory button, use Magit rather than Dired."
+  :type 'boolean
+  :group 'hyperbole-button)
+
 (defcustom hibtypes-github-default-project nil
   "Default project name to associate with any Github commit link."
   :type 'string
@@ -218,6 +224,10 @@
   :type 'string
   :group 'hyperbole-button)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(declare-function magit-status-setup-buffer "ext:magit")
 ;;; ************************************************************************
 ;;; Private variables
 ;;; ************************************************************************
@@ -226,10 +236,9 @@
   '(("\\`\\(fb\\|facebook\\)\\'"  . "https://www.facebook.com/hashtag/%s")
     ("\\`\\(gh\\|github\\)\\'"    . "https://github.com/%s/%s/%s%s")
     ("\\`\\(gl\\|gitlab\\)\\'"    . "https://www.gitlab.com/%s/%s/%s%s")
-    ("\\`\\(gt\\|git\\)\\'"       . "(cd %s; git %s %s)")
+    ("\\`\\(gt\\|git\\)\\'"       . "(cd %s && git %s %s)")
     ("\\`\\(in\\|instagram\\)\\'" . "https://www.instagram.com/explore/tags/%s/")
-    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=%%23%s&src=hashtag")
-)
+    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=%%23%s&src=hashtag"))
   "Alist of (social-media-service-regexp  . expression-to-display-hashtag-reference) elements.")
 
 (defconst hibtypes-social-username-alist
@@ -237,8 +246,7 @@
     ("\\`\\(gh\\|github\\)\\'"    . "https://github.com/%s/")
     ("\\`\\(gl\\|gitlab\\)\\'"    . "https://www.gitlab.com/%s/")
     ("\\`\\(in\\|instagram\\)\\'" . "https://www.instagram.com/%s/")
-    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=@%s")
-    )
+    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=@%s"))
   "Alist of (social-media-service-regexp  . url-with-%s-for-username) elements.")
 
 ;; Assume at least a 2-character project name
@@ -602,11 +610,18 @@ PROJECT value is provided, it defaults to the value of
 ;;; Local git repository commit references
 
 (defib git-commit-reference ()
-  "Display the diff for a git commit reference, e.g. \"commit a55e21\", typically produced by git log."
-  (if (save-excursion
-	(beginning-of-line)
-	(looking-at "\\s-*commit \\([0-9a-f]+\\)$"))
-      (hact #'git-reference (match-string-no-properties 1))))
+  "Display the changeset for a git commit reference, e.g. \"commit a55e21\", typically produced by git log.
+Hyperbole also includes two commands, `hypb:fgrep-git-log' and
+`hypb:grep-git-log' to list git commit references whose changesets
+contain either the string (fgrep) or regular expression (grep) given.
+Then an Action Key displays the associated changeset.
+"
+  (when (save-excursion
+	  (beginning-of-line)
+	  (looking-at "\\(^\\|\\s-+\\)\\(commit #?\\([0-9a-f][0-9a-f][0-9a-f][0-9a-f]+\\)\\)\\(\\s-\\|$\\)"))
+    (save-match-data
+      (ibut:label-set (match-string-no-properties 2) (match-beginning 2) (match-end 2)))
+    (hact #'git-reference (match-string-no-properties 3))))
 
 (defvar hibtypes-git-repos-cache
   (expand-file-name "Local-Git-Repos" hbmap:dir-user)
@@ -623,10 +638,11 @@ PROJECT value is provided, it defaults to the value of
   "Store cache of local git repo directories in `hibtypes-git-repos-cache'.
 With optional PROMPT-FLAG non-nil, prompt user whether to build the cache before building.
 Return t if built, nil otherwise."
+  (interactive)
   (when (or (not prompt-flag)
 	    (y-or-n-p "Find all local git repositories (will take some time)? "))
     (message "Please wait while all local git repositories are found...")
-    (unless (zerop (shell-command (format "%s -r '/\.git$' | sed -e 's+/.git$++' > %s"
+    (unless (zerop (shell-command (format "%s -r '/\\.git$' | sed -e 's+/.git$++' > %s"
 					  (hibtypes-git-get-locate-command)
 					  hibtypes-git-repos-cache)))
       (error "(hibtypes-git-build-repos-cache): Cache build failed; `locate-command' must accept `-r' argument for regexp matching"))
@@ -636,6 +652,7 @@ Return t if built, nil otherwise."
 (defun hibtypes-git-add-project-to-repos-cache (project)
   "Locate PROJECT directory and add to the cache of local git repo directories in `hibtypes-git-repos-cache'.
 Return the project directory found or nil if none."
+  (interactive "sProject: ")
   (message "Please wait while %s's local git repository is found..." project)
   (let ((project-dir (shell-command-to-string
 		      (format "%s -l1 /%s/.git | sed -e 's+/.git++' | tr -d '\n'"
@@ -644,13 +661,14 @@ Return the project directory found or nil if none."
     (message "")
     (when (and (> (length project-dir) 0) (= ?/ (aref project-dir 0)))
       ;; project-dir a directory, prepend it to the cache file...
-      (shell-command-to-string (format "echo -e \"%s\n$(cat %s)\" > %s"
-				       project-dir hibtypes-git-repos-cache
-				       hibtypes-git-repos-cache))
+      (with-current-buffer (find-file-noselect hibtypes-git-repos-cache)
+        (goto-char (point-min))
+        (insert (concat project-dir "\n"))
+        (save-buffer))
       ;; ...and return it.
       project-dir)))
 
-(defun hibtypes-git-build-or-add-to-repos-cache (project &optional prompt-flag)
+(defun hibtypes-git-build-or-add-to-repos-cache (project &optional _prompt-flag)
   "Store cache of local git repo directories in `hibtypes-git-repos-cache'.
 With optional PROMPT-FLAG non-nil, prompt user whether to build the cache before building.
 Return t if built, nil otherwise."
@@ -675,7 +693,7 @@ Return t if built, nil otherwise."
 ;;  1. If within a git repo directory, use that repo unless specified in path
 ;;  2. If project name is given or is default, see if assocated repo dir is in cache and use it.
 ;;  3. Prompt to rebuild locate db and then goto 2 if yes else quit
-;;  4. Run: (cd <dir-found>; git <cmd> <item>)
+;;  4. Run: (cd <dir-found> && git <cmd> <item>)
 ;;  5. Otherwise, do nothing.
 ;;
 ;; Don't make this a defact or its arguments may be improperly expanded as pathnames.
@@ -780,6 +798,8 @@ PROJECT value is provided, it defaults to the value of
 		       (setq project-dir (and project (hibtypes-git-project-directory project)))
 		     (error "(git-reference): No git directory found for project `%s'" project)))
 		 (when (equal project-dir "") (setq project-dir nil))
+		 ;; Eliminate ~ that Windows shell can't handle in shell paths
+		 (when project-dir (setq project-dir (expand-file-name project-dir)))
 		 (cond ((and project-dir (file-readable-p project-dir) (file-directory-p project-dir))
 			(if reference
 			    (if (and (equal ref-type "commits") (fboundp 'vc-print-root-log))
@@ -799,12 +819,16 @@ PROJECT value is provided, it defaults to the value of
 							  (if (equal project "") "" " ")
 							  project ref-type
 							  (if (equal reference "") "" " ")
-							  reference)
+							  (substring reference nil (min 9 (length reference))))
 				  (princ (format "Command: %s\n\n" cmd))
 				  (princ (shell-command-to-string cmd)))))
 			  ;; Project-only reference, run dired on the project home directory
-			  (hpath:display-buffer (dired-noselect
-						 (file-name-as-directory project-dir)))))
+			  (if (and hibtypes-git-use-magit-flag (fboundp #'magit-status-setup-buffer))
+			      (hpath:display-buffer (save-window-excursion
+						      (magit-status-setup-buffer
+						       (file-name-as-directory project-dir))))
+			    (hpath:display-buffer (dired-noselect
+						   (file-name-as-directory project-dir))))))
 		       (t (if project-dir
 			      (error "(git-reference): git project `%s' directory is unreadable or invalid: \"%s\""
 				     project project-dir)
