@@ -1,10 +1,11 @@
-;;; hmail.el --- GNU Hyperbole buttons embedded in e-mail messages
+;;; hmail.el --- GNU Hyperbole buttons embedded in e-mail messages  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     9-Oct-91 at 18:38:05
+;; Last-Mod:      6-Mar-22 at 22:29:00 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the HY-COPY (Hyperbole) or BR-COPY (OO-Browser) file for license
 ;; information.
 ;;
@@ -78,14 +79,15 @@ MSG-END."
     (if (search-backward hmail:hbdata-sep msg-start t) (1- (point)) msg-end)))
 
 (defun hmail:hbdata-to-p ()
-  "Move point to Hyperbole but data start in an e-mail msg.
-Return t if button data is found."
+  "When in a buffer with embedded Hyperbole button data, move point to the start of the button data.
+Return t if button data is found, else nil."
   (and (cond ((memq major-mode (list hmail:reader hmail:modifier))
 	      (hmail:msg-narrow) t)
 	     ((or (hmail:lister-p) (hnews:lister-p)) t)
 	     ((memq major-mode (list hmail:composer hnews:reader
 				     hnews:composer))
-	      (widen) t))
+	      (widen) t)
+	     ((not buffer-file-name)))
        (progn
 	 (goto-char (point-max))
 	 (if (search-backward hmail:hbdata-sep nil t)
@@ -113,11 +115,11 @@ Optional SUBJECT and HELP message may also be given."
   (interactive "sDeliver e-mail to: \nSubject: ")
   (require 'hactypes) ;; Needed in case EXPR calls hact.
   (unless (or (stringp help) (stringp subject))
-    (setq subject "Be explicit here.  Make a statement or ask a question."))
+    (setq subject "Delete this text but write a detailed subject.  Make a statement or ask a question."))
   (hmail:invoke address nil subject)
   (eval expr)
-  (if (re-search-backward "^Subject: " nil t)
-      (goto-char (match-end 0)))
+  (when (re-search-backward "^Subject: " nil t)
+    (goto-char (match-end 0)))
   (message (if (stringp help)
 	       help
 	     "Replace subject, compose message, and then mail.")))
@@ -156,9 +158,12 @@ Optional SUBJECT and HELP message may also be given."
 Optional arguments are ADDRESS, CC list and SUBJECT of mail."
   ;; Next 3 lines prevent blank lines between fields due to
   ;; fill-region-as-paragraph within mail-setup.
-  (if (equal address "") (setq address nil))
-  (if (equal cc "") (setq cc nil))
-  (if (equal subject "") (setq subject nil))
+  (when (equal address "")
+    (setq address nil))
+  (when (equal cc "")
+    (setq cc nil))
+  (when (equal subject "")
+    (setq subject nil))
   (compose-mail address subject (if cc (list (cons "CC" cc)))))
 
 
@@ -181,7 +186,8 @@ Optional arguments are ADDRESS, CC list and SUBJECT of mail."
   "Narrows buffer to displayable part of current message.
 Its displayable part begins at optional MSG-START and ends at or before
 MSG-END."
-  (if (hmail:reader-p) (rmail:msg-widen))
+  (when (hmail:reader-p)
+    (rmail:msg-widen))
   (setq msg-start (or msg-start (point-min))
 	msg-end (or msg-end (point-max)))
   (narrow-to-region msg-start (hmail:hbdata-start msg-start msg-end)))
@@ -197,15 +203,16 @@ non-nil.  Optional BUF contains the region and defaults to the current
 buffer.  It may be a buffer or buffer name."
   (interactive (list (region-beginning) (region-end) (current-buffer)
 		     (y-or-n-p "Include invisible text? ")))
-  (or buf (setq buf (current-buffer)))
-  (if (stringp buf) (setq buf (get-buffer buf)))
+  (unless buf
+    (setq buf (current-buffer)))
+  (when (stringp buf)
+    (setq buf (get-buffer buf)))
   (let (mail-buf)
     (hmail:invoke)
     (setq mail-buf (current-buffer))
     (save-excursion
-      (if (search-forward mail-header-separator nil t)
-	  ;; Within header, so move to body
-	  (goto-char (point-max)))
+      (rfc822-goto-eoh)
+      (forward-line)
       (set-buffer buf)
       (hypb:insert-region mail-buf start end invisible-flag))))
 
@@ -244,8 +251,7 @@ Signals error when current mail reader is not supported."
 		;; Found matching msg
 		(buffer-substring (match-beginning 2) (match-end 2))))
 	;; (rmail:msg-hdrs-full toggled)
-	()
-	))))
+	()))))
 
 ;;; ------------------------------------------------------------------------
 ;;; Each mail reader-specific Hyperbole support module must also define
