@@ -1,10 +1,11 @@
-;;; hmouse-drv.el --- Smart Key/Mouse driver functions.
+;;; hmouse-drv.el --- Smart Key/Mouse driver functions.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    04-Feb-90
+;; Last-Mod:     17-Apr-22 at 15:11:15 by Bob Weiner
 ;;
-;; Copyright (C) 1989-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1989-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -19,10 +20,6 @@
 (require 'hui-window)
 (require 'hypb)
 
-;; Quiet byte compiler warnings for these free variables.
-(defvar hkey-action)
-(defvar pred-value)
-
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
@@ -109,6 +106,30 @@ Default is nil."
   "Used to pass the value of a selected region between a Smart Key depress and release.
 This permits the Smart Keys to behave as paste keys.")
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function br-in-view-window-p "ext:br")
+(declare-function br-next-listing-window "ext:br")
+(declare-function br-to-view-window "ext:br")
+
+(declare-function ace-window "ext:ace-window")
+(declare-function ace-window-display-mode "ext:ace-window")
+(declare-function aw-select "ext:ace-window")
+
+(defvar aw-dispatch-alist)
+(defvar aw-dispatch-always)
+(defvar aw-frame-size)
+(defvar aw-keys)
+
+;; window-jump
+(declare-function window-jump "ext:window-jump")
+(defvar	wj-vec-left)
+(defvar	wj-vec-right)
+(defvar	wj-vec-down)
+(defvar	wj-vec-up)
+
 ;;; ************************************************************************
 ;;; Private variables
 ;;; ************************************************************************
@@ -135,6 +156,7 @@ This permits the Smart Keys to behave as paste keys.")
 
 ;;; Smart Key Depress Functions
 (defun action-key-depress (&rest args)
+  "Register depress of the Hyperbole Action Mouse Key."
   (interactive)
   (cond (assist-key-depressed-flag
 	 (or action-key-help-flag
@@ -155,6 +177,7 @@ This permits the Smart Keys to behave as paste keys.")
   (run-hooks 'action-key-depress-hook))
 
 (defun assist-key-depress (&rest args)
+  "Register depress of the Hyperbole Assist Mouse Key."
   (interactive)
   (cond (action-key-depressed-flag
 	 (or assist-key-help-flag
@@ -175,10 +198,12 @@ This permits the Smart Keys to behave as paste keys.")
   (run-hooks 'assist-key-depress-hook))
 
 (defun action-key-depress-emacs (event)
+  "Handle depress event of the Hyperbole Action Mouse Key."
   (interactive "e")
   (action-key-depress event))
 
 (defun assist-key-depress-emacs (event)
+  "Handle depress event of the Hyperbole Assist Mouse Key."
   (interactive "e")
   (assist-key-depress event))
 
@@ -190,7 +215,7 @@ EVENT will be passed to 'hmouse-function'."
   (apply #'action-mouse-key (hmouse-key-release-args-emacs event)))
 
 (defun assist-mouse-key-emacs (event)
-  "Set point to the current mouse cursor position and execute 'action-key'.
+  "Set point to the current mouse cursor position and execute 'assist-key'.
 EVENT will be passed to 'hmouse-function'."
   (interactive "e")
   (apply #'assist-mouse-key (hmouse-key-release-args-emacs event)))
@@ -312,7 +337,7 @@ bound to a valid function."
 	(funcall assist-key-default-function)
 	t)))
 
-(defun hkey-either (arg)
+(defun hkey-either (&optional arg)
   "Execute `action-key' or with non-nil ARG execute `assist-key'."
   (interactive "P")
   (when (and (featurep 'hycontrol)
@@ -359,20 +384,22 @@ Then whenever point is on an item you want displayed in another
 window, use {M-o i <id-of-window-to-display-item-in>} and watch the
 magic happen."
   (require 'ace-window)
-  (when key (global-set-key key 'ace-window))
+  (when key (hkey-set-key key 'ace-window))
   (setq aw-keys '(?a ?s ?d ?f ?g ?h ?j ?k ?l)
 	;; allows {i} operation to work when only 2 windows exist
 	aw-dispatch-always t)
   ;; New ace-window frames (window id = z) inherit the size of the
   ;; prior selected frame; same as HyWindow.
   (setq aw-frame-size '(0 . 0)
-	aw-dispatch-alist (delq (assq ?t aw-dispatch-alist)
+	aw-dispatch-alist (delq (assq ?i aw-dispatch-alist)
 				(delq (assq ?r aw-dispatch-alist)
-				      (delq (assq ?i aw-dispatch-alist) aw-dispatch-alist))))
-  (push '(?i hkey-drag-item "Hyperbole: Drag Item") aw-dispatch-alist)
-  ;; Ace-window includes ?m as the swap windows key, so it is not added here.
-  (push '(?r hkey-replace "Hyperbole: Replace Here") aw-dispatch-alist)
+				      (delq (assq ?t aw-dispatch-alist)
+					    (delq (assq ?w aw-dispatch-alist) aw-dispatch-alist)))))
+  (push '(?w hkey-window-link "Hyperbole: Window Link") aw-dispatch-alist)
   (push '(?t hkey-throw   "Hyperbole: Throw") aw-dispatch-alist)
+  (push '(?r hkey-replace "Hyperbole: Replace Here") aw-dispatch-alist)
+  ;; Ace-window includes ?m as the swap windows key, so it is not added here.
+  (push '(?i hkey-drag-item "Hyperbole: Drag Item") aw-dispatch-alist)
   (ace-window-display-mode 1))
 
 ;;;###autoload
@@ -411,9 +438,6 @@ Works only when running under a window system, not from a dumb terminal."
 After the drag, the selected window remains the same as it was before
 the drag.
 
-Optional prefix arg non-nil means emulate Assist Key rather than the
-Action Key.
-
 Works only when running under a window system, not from a dumb terminal."
   (let ((start-window (selected-window)))
     (unwind-protect
@@ -559,6 +583,20 @@ The selected window does not change."
 	     ;; Don't use sit-for here because it can be interrupted early.
 	     (sleep-for display-delay))))))
 
+;;;###autoload
+(defun hkey-window-link (release-window)
+  "Create a new Hyperbole explicit link button in the selected window, linked to point in RELEASE-WINDOW, interactively chosen via ace-window.
+The selected window does not change."
+  (interactive
+   (list (let ((mode-line-text (concat " Ace - Hyperbole: " (nth 2 (assq ?w aw-dispatch-alist)))))
+	   (aw-select mode-line-text))))
+  (let ((start-window (selected-window)))
+    (unwind-protect
+	(hui:link-directly start-window release-window)
+      ;; Leave start-window selected
+      (when (window-live-p start-window)
+	(hypb:select-window-frame start-window)))))
+
 (defun hkey-insert-region (depress-window release-window throw-region-flag display-delay)
   "Throw any active (highlighted) region from DEPRESS-WINDOW to RELEASE-WINDOW.
 If THROW-REGION-FLAG is non-nil, the region is thrown even if not
@@ -568,24 +606,30 @@ Highlight the thrown region for DISPLAY-DELAY seconds.
 
 Return t if thrown, else nil."
   (when (or (use-region-p) throw-region-flag)
-    (if (> (region-end) (region-beginning))
-	;; Non-empty region
-	(if (eq (window-buffer depress-window) (window-buffer release-window))
-	    (user-error "(hkey-insert-region): Can't throw region from and to the same buffer")
-	  (let* ((orig-buf (current-buffer))
-		 (orig-start (region-beginning))
-		 (orig-end (region-end))
-		 (len (- orig-end orig-start))
-		 insert-start
-		 insert-end)
-	    (select-window release-window 'mark-for-redisplay)
-	    (setq insert-start (point)
-		  insert-end (+ insert-start len))
-	    (insert-buffer-substring orig-buf orig-start orig-end)
-	    (hmouse-pulse-region insert-start insert-end)
-	    (sit-for display-delay)
-	    t))
-      (user-error "(hkey-insert-region): Can't throw an empty region"))))
+    (cond ((or (not (window-live-p depress-window))
+	       (not (window-live-p release-window)))
+	   (user-error "(hkey-insert-region): Invalid window: depress window: '%s'; release window: '%s'"
+		       depress-window release-window))
+	  ((> (region-end) (region-beginning))
+	   ;; Non-empty region
+	   (if (and (eq (window-buffer depress-window) (window-buffer release-window))
+		    (<= (region-beginning) (window-point release-window))
+		    (>= (region-end) (window-point release-window)))
+	       (user-error "(hkey-insert-region): Can't throw region to a point within the region")
+	     (let* ((orig-buf (current-buffer))
+		    (orig-start (region-beginning))
+		    (orig-end (region-end))
+		    (len (- orig-end orig-start))
+		    insert-start
+		    insert-end)
+	       (select-window release-window 'mark-for-redisplay)
+	       (setq insert-start (point)
+		     insert-end (+ insert-start len))
+	       (insert-buffer-substring orig-buf orig-start orig-end)
+	       (hmouse-pulse-region insert-start insert-end)
+	       (sit-for display-delay)
+	       t)))
+	  (t (user-error "(hkey-insert-region): Can't throw an empty region")))))
 
 ;;;###autoload
 (defun hkey-buffer-to (from-window to-window)
@@ -690,13 +734,13 @@ hkey-swap and hkey-throw."
 		   until (and (mouse-event-p start-event)
 			      (not (string-match "\\`down-" (symbol-name (car start-event)))))
 		   finally return (posn-window (event-start start-event))))
-    (message "Click on the %s start window...Now on the end window..." func)
+    (message "Now click on the %s end window..." func)
     (setq end-window
 	  (cl-loop do (setq end-event (read-event))
 		   until (and (mouse-event-p end-event)
 			      (not (string-match "\\`down-" (symbol-name (car end-event)))))
 		   finally return (posn-window (event-start end-event))))
-    (message "Click on the %s start window...Now on the end window...Done" func)
+    (message "Done")
     (with-selected-window start-window
       (funcall func end-window))))
 
@@ -795,107 +839,136 @@ frame instead."
       ;; Drag frame when the window is on the bottom of its frame.
       (mouse-drag-frame start-event 'move)))))
 
-(defun hkey-debug ()
-  (message (format "(HyDebug) %sContext: %s; %s: %s; Buf: %s; Mode: %s; MinibufDepth: %s"
-		   (cond ((eq pred-value 'hbut:current)
-			  (format "ButType: %s; ButLabel: %s; "
-				  (hattr:get  'hbut:current 'categ)
-				  (hypb:format-quote (hbut:label 'hbut:current))))
-			 ((functionp pred-value)
-			  (format "Selection Func: %s; " pred-value))
-			 (t ""))
-		   pred
-		   (if assist-flag "Assist" "Action")
-		   (hypb:format-quote (format "%s" hkey-action))
-		   (current-buffer) major-mode (minibuffer-depth))))
+(defun hkey-debug (pred pred-value hkey-action)
+  (message "(HyDebug) %sContext: %s; %s: %s; Buf: %s; Mode: %s; MinibufDepth: %s"
+	   (cond ((eq pred-value 'hbut:current)
+		  (format "ButType: %s; ButLabel: %s; "
+			  (hattr:get 'hbut:current 'categ)
+			  (hypb:format-quote (hbut:label 'hbut:current))))
+		 ((functionp pred-value)
+		  (format "Selection Func: %s; " pred-value))
+		 (t ""))
+	   pred
+	   (if assist-flag "Assist" "Action")
+	   (if (hattr:get  'hbut:current 'actype)
+	       (cons (hattr:get  'hbut:current 'actype)
+		     (hattr:get  'hbut:current 'args))
+	     (hypb:format-quote (format "%s" hkey-action)))
+	   (current-buffer)
+	   major-mode
+	   (minibuffer-depth)))
 
-(defun hkey-execute (assist-flag)
-  "Evaluate Action Key form (or Assist Key form with ASSIST-FLAG non-nil) for first non-nil predicate from `hkey-alist'.
-Non-nil ASSIST-FLAG means evaluate second form, otherwise evaluate first form.
+(defun hkey-execute (assisting)
+  "Evaluate Action Key form (or Assist Key form with ASSISTING non-nil) for first non-nil predicate from `hkey-alist'.
+Non-nil ASSISTING means evaluate second form, otherwise evaluate first form.
 Return non-nil iff a non-nil predicate is found."
   ;; Keep in mind that hkey-alist may be set to hmouse-alist here, with additional predicates.
   (let ((hkey-forms hkey-alist)
-	(pred-value) (hkey-action) hkey-form pred)
+	(pred-point (point-marker))
+	pred-value hkey-action hkey-form pred)
     (while (and (null pred-value) (setq hkey-form (car hkey-forms)))
-      (if (setq hkey-action (if assist-flag (cddr hkey-form) (cadr hkey-form))
+      (if (setq hkey-action (if assisting (cddr hkey-form) (cadr hkey-form))
 		pred (car hkey-form)
 		pred-value (eval pred))
-	  ;; Conditionally debug after Smart Key release and evaluation
-	  ;; of matching predicate but before hkey-action is executed.
-	  (progn (when hkey-debug (hkey-debug))
-		 (eval hkey-action))
+	  (progn
+	    ;; Any Smart Key predicate should leave point unchanged.
+	    ;; Trigger an error if not.
+	    (unless (equal (point-marker) pred-point)
+	      (hypb:error "(Hyperbole): `%s' predicate failed to restore point to %s" pred pred-point))
+	    (set-marker pred-point nil)
+	    ;; Conditionally debug after Smart Key release and evaluation
+	    ;; of matching predicate but before hkey-action is executed.
+	    (when hkey-debug
+	      (hkey-debug pred pred-value hkey-action))
+	    (if hkey-debug
+		(let ((debug-on-error t)
+		      (debug-on-quit t))
+		  (eval hkey-action))
+	      (eval hkey-action)))
 	(setq hkey-forms (cdr hkey-forms))))
     pred-value))
 
-(defun hkey-help (&optional assist-flag)
+(defun hkey-help (&optional assisting)
   "Display help for the Action Key command in current context.
-With optional ASSIST-FLAG non-nil, display help for the Assist Key command.
+With optional ASSISTING prefix arg non-nil, display help for the Assist Key command.
 Return non-nil iff associated help documentation is found."
   (interactive "P")
-  (unless (or action-key-depressed-flag action-key-help-flag)
-    (action-key-clear-variables))
-  (unless (or assist-key-depressed-flag assist-key-help-flag)
-    (assist-key-clear-variables))
-  (let ((hkey-forms hmouse-alist)
-	hkey-form pred-value call calls cmd-sym doc)
+  (let* ((mouse-flag (when (mouse-event-p last-command-event)
+		       (or action-key-depress-position assist-key-depress-position)))
+	 (mouse-drag-flag (hmouse-drag-p))
+	 (hkey-forms (if mouse-flag hmouse-alist hkey-alist))
+	 (hrule:action 'actype:identity)
+	 hkey-form pred-value call calls cmd-sym doc)
     (while (and (null pred-value) (setq hkey-form (car hkey-forms)))
       (or (setq pred-value (eval (car hkey-form)))
 	  (setq hkey-forms (cdr hkey-forms))))
     (if pred-value
-	(setq call (if assist-flag (cdr (cdr hkey-form))
+	(setq call (if assisting (cdr (cdr hkey-form))
 		     (cadr hkey-form))
 	      cmd-sym (if (eq (car call) #'funcall)
 			  (cadr call)
 			(car call)))
-      (setq cmd-sym (if assist-flag assist-key-default-function action-key-default-function)
+      (setq cmd-sym (if assisting assist-key-default-function action-key-default-function)
 	    call cmd-sym))
     (if (and (consp call) (eq (car call) 'call-interactively))
-	(if (consp (cadr call))
-	    (setq cmd-sym (if (memq (caadr call) '(function quote))
-			      (cadadr call)
-			    (caadr call)))))
+	(when (consp (cadr call))
+	  (setq cmd-sym (if (memq (caadr call) '(function quote))
+			    (cadadr call)
+			  (caadr call)))))
     (setq calls (if (and (consp call) (eq (car call) 'or))
-		    (mapcar 'identity (cdr call))
+		    (mapcar #'identity (cdr call))
 		  (list cmd-sym)))
 
+    (unless (or action-key-depressed-flag action-key-help-flag)
+      (action-key-clear-variables))
+    (unless (or assist-key-depressed-flag assist-key-help-flag)
+      (assist-key-clear-variables))
+
     (setq hkey-help-msg
 	  (if (and cmd-sym (symbolp cmd-sym))
 	      (progn
 		(let* ((condition (car hkey-form))
 		       (temp-buffer-show-hook
-			 (lambda (buf)
-			   (set-buffer buf)
-			   (help-mode)
-			   (let ((owind (selected-window)))
-			     (if (br-in-browser)
-				 (save-excursion
-				   (br-to-view-window)
-				   (select-window (previous-window))
-				   (display-buffer buf 'other-win))
-			       (display-buffer buf 'other-win))
-			     (if (or (and (boundp 'help-window-select)
-					  help-window-select)
-				     (and (boundp 'help-selects-help-window)
-					  help-selects-help-window))
-				 (select-window (get-buffer-window buf))
-			       (select-window owind)))))
+			(lambda (buf)
+			  (set-buffer buf)
+			  (help-mode)
+			  (let ((owind (selected-window)))
+			    (if (br-in-browser)
+				(save-excursion
+				  (br-to-view-window)
+				  (select-window (previous-window))
+				  (display-buffer buf 'other-win))
+			      (display-buffer buf 'other-win))
+			    (if (or (and (boundp 'help-window-select)
+					 help-window-select)
+				    (and (boundp 'help-selects-help-window)
+					 help-selects-help-window))
+				(select-window (get-buffer-window buf))
+			      (select-window owind)))))
 		       (temp-buffer-show-function temp-buffer-show-hook))
 		  (with-output-to-temp-buffer
 		      (hypb:help-buf-name
-		       (format "%s Key" (if assist-flag "Assist" "Action")))
-		    (princ (format "A click of the %s Key"
-				   (if assist-flag "Assist" "Action")))
+		       (format "%s %sKey"
+			       (if assisting "Assist" "Action")
+			       (if mouse-flag "Mouse " "")))
+		    (princ (format "A %s of the %s %sKey"
+				   (if mouse-flag
+				       (if mouse-drag-flag "drag" "click")
+				     "press")
+				   (if assisting "Assist" "Action")
+				   (if mouse-flag "Mouse " "")))
 		    (terpri)
 		    (princ "WHEN  ")
 		    (princ
-		      (or condition
-			  "there is no matching context"))
+		     (or condition
+			 "there is no matching context"))
 		    (terpri)
 
 		    (mapc (lambda (c)
-			    (if (> (length calls) 1)
-				;; Is an 'or' set of calls
-				(princ "'OR' "))
+			    (when (and (> (length calls) 1)
+				       (not (eq (car calls) c)))
+			      ;; Is an 'or' set of calls
+			      (princ "OR "))
 			    (princ "CALLS ") (princ (if (consp c) c (list c)))
 			    (when (and (fboundp (setq call (if (consp c) (car c) c)))
 				       (setq doc (documentation call)))
@@ -917,23 +990,22 @@ Return non-nil iff associated help documentation is found."
 				     (actype:doc 'hbut:current t)))
 		      (hattr:report
 		       (nthcdr 2 (hattr:list 'hbut:current))))
-		    (terpri)
-		    ))
+		    (terpri)))
 		"")
 	    (message "No %s Key command for current context."
-		     (if assist-flag "Assist" "Action"))))
+		     (if assisting "Assist" "Action"))))
     doc))
 
 (defun hkey-assist-help ()
   "Display doc associated with Assist Key command in current context.
 Return non-nil iff associated documentation is found."
   (interactive)
-  (hkey-help 'assist))
+  (hkey-help t))
 
 ;; Overload help-mode quit-window function to support Hyperbole
 ;; hkey--wconfig window configurations.
 (unless (eq (symbol-function #'quit-window) #'hkey-help-hide)
-  (defalias 'hkey-quit-window (hypb:function-copy #'quit-window)))
+  (defalias 'hkey-quit-window (symbol-function #'quit-window)))
 
 ;;;###autoload
 (defun hkey-help-hide (&optional kill window)
@@ -945,7 +1017,8 @@ details."
   (let ((buf (current-buffer)))
     (if (window-configuration-p hkey--wconfig)
 	(progn (set-window-configuration hkey--wconfig)
-	       (if kill (kill-buffer buf)
+	       (if kill
+		   (kill-buffer buf)
 		 (bury-buffer buf)))
       (hkey-quit-window kill window)))
   (setq hkey--wconfig nil))
@@ -1004,30 +1077,31 @@ the current window.  By default, it is displayed according to the setting of
 	      (when (derived-mode-p 'help-mode)
 		(local-set-key "q" #'hkey-help-hide)))))
       ;; If in an *Org Help* buffer, reselect the Org buffer.
-      (if org-help (select-window owind))
+      (when org-help
+	(select-window owind))
       ;; If in a *Completions* buffer, re-select the window that
       ;; generated the completions.
-      (if (buffer-live-p completion-reference-buffer)
-	  (select-window (get-buffer-window completion-reference-buffer t))))))
+      (when (buffer-live-p completion-reference-buffer)
+	(select-window (get-buffer-window completion-reference-buffer t))))))
 
-(defun hkey-mouse-help (assist-flag args)
+(defun hkey-mouse-help (assisting args)
   "If a Smart Key help flag is set and the other Smart Key is not down, show help.
-Takes two args:  ASSIST-FLAG should be non-nil iff command applies to the Assist Key.
+Takes two args:  ASSISTING should be non-nil iff command applies to the Assist Key.
 ARGS is a list of arguments passed to `hmouse-function'.
 Return t if help is displayed, nil otherwise."
   (let ((help-shown)
-	(other-key-released (not (if assist-flag
+	(other-key-released (not (if assisting
 				     action-key-depressed-flag
 				   assist-key-depressed-flag))))
     (unwind-protect
 	(setq help-shown
 	      (cond ((and  action-key-help-flag other-key-released)
 		     (setq action-key-help-flag nil)
-		     (hmouse-function #'hkey-help assist-flag args)
+		     (hmouse-function #'hkey-help assisting args)
 		     t)
 		    ((and  assist-key-help-flag other-key-released)
 		     (setq assist-key-help-flag nil)
-		     (hmouse-function #'hkey-assist-help assist-flag args)
+		     (hmouse-function #'hkey-assist-help assisting args)
 		     t)))
       (when help-shown
 	;; Then both Smart Keys have been released.
@@ -1043,8 +1117,8 @@ Action Key.
 
 Only works when running under a window system, not from a dumb terminal."
   (interactive "P")
-  (or (hyperb:window-system)
-      (hypb:error "(hkey-operate): Drag actions require mouse support"))
+  (unless (hyperb:window-system)
+    (hypb:error "(hkey-operate): Drag actions require mouse support"))
   (if arg
       (if assist-key-depressed-flag
 	  (progn (assist-mouse-key)
@@ -1072,11 +1146,10 @@ the current window.  By default, it is displayed in another window."
   (interactive)
   (let* ((doc-file (hypb:hkey-help-file))
 	 (buf-name (hypb:help-buf-name "Smart Keys"))
-	 (wind (get-buffer-window buf-name))
-	 owind)
+	 (wind (get-buffer-window buf-name)))
     (when (file-readable-p doc-file)
-      (if (br-in-browser)
-	  (br-to-view-window))
+      (when (br-in-browser)
+	(br-to-view-window))
       (if wind
 	  (select-window wind)
 	(hkey-help-show buf-name current-window)
@@ -1088,8 +1161,8 @@ the current window.  By default, it is displayed in another window."
 
 
 (defun hkey-toggle-debug (&optional arg)
-  "Toggle whether conflicting local key bindings are overridden by Hyperbole.
-With optional ARG, override them iff ARG is positive."
+  "Toggle whether Hyperbole logs Smart Key events for later analysis/submission using {C-h h m c}.
+With optional ARG, enable iff ARG is positive."
   (interactive "P")
   (if (or (and arg (<= (prefix-numeric-value arg) 0))
 	  (and (not (and arg (> (prefix-numeric-value arg) 0)))
@@ -1102,7 +1175,8 @@ With optional ARG, override them iff ARG is positive."
 (defun hmouse-depress-inactive-minibuffer-p (event)
   "Return the minibuffer window if the last Smart Mouse Key depress EVENT was in it and it was inactive, else nil."
   (let ((window (posn-window (event-start event))))
-    (if (framep window) (setq window (frame-selected-window window)))
+    (when(framep window)
+      (setq window (frame-selected-window window)))
     (and (window-minibuffer-p window)
 	 (not (minibuffer-window-active-p window))
 	 window)))
@@ -1273,9 +1347,9 @@ window, return nil.  Considers all windows on the selected frame's display."
 	       pos-x pos-y))
     (when edges (list window (cons pos-x pos-y)))))
 
-(defun hmouse-key-release-window ()
-  "Return the window of the current mouse position if any, else nil."
-  (ignore-errors (hmouse-window-at-absolute-pixel-position nil t)))
+(defun hmouse-key-release-window (release-position)
+  "Return the Emacs window of last Action/Assist Mouse Key RELEASE-POSITION if any, else nil."
+  (ignore-errors (hmouse-window-at-absolute-pixel-position release-position t)))
 
 (defun hmouse-key-release-args-emacs (event)
   "For GNU Emacs, return a possibly modified version of EVENT as a list.
@@ -1309,22 +1383,16 @@ compute the actual release location and include that."
 
 (defun hmouse-use-region-p ()
   "Return t if there is a non-empty, highlighted region, else nil."
-  (cond
-   ;; Newer GNU Emacs
-   ((fboundp 'use-region-p)
-    (let ((use-empty-active-region))
-      (use-region-p)))
-   ;; InfoDock and XEmacs
-   ((fboundp 'region-exists-p)
-    (and (fboundp 'region-active-p) (region-active-p) (region-exists-p)))
-   ;; Older GNU Emacs
-   ((boundp 'transient-mark-mode)
-    (and transient-mark-mode mark-active))))
+  ;; Newer GNU Emacs
+  (if (fboundp 'use-region-p)
+      (let ((use-empty-active-region))
+	(use-region-p))
+    ;; Older GNU Emacs
+    (and transient-mark-mode mark-active)))
 
-(defun hmouse-save-region (&optional frame)
+(defun hmouse-save-region ()
   "Save to `hkey-region' and return any active region within the current buffer.
-Under InfoDock and XEmacs, `zmacs-region' must be t; under GNU Emacs,
-`transient-mark-mode' must be t or the function does nothing."
+`transient-mark-mode' must be t or this sets `hkey-region' to nil."
   (setq hkey-region
 	(when (hmouse-use-region-p)
 	  (buffer-substring (region-beginning) (region-end)))))
@@ -1353,41 +1421,32 @@ return current point as a marker."
   "Set point to cursor position using SET-POINT-ARG-LIST and return t.
 If 'hmouse-set-point-command' is not bound to a function, this does nothing
 and returns nil."
-  (if (fboundp hmouse-set-point-command)
-      (or (if set-point-arg-list
-	      (funcall hmouse-set-point-command set-point-arg-list)
-	    (funcall hmouse-set-point-command))
-	  t)))
+  (when (fboundp hmouse-set-point-command)
+    (or (if set-point-arg-list
+	    (funcall hmouse-set-point-command set-point-arg-list)
+	  (funcall hmouse-set-point-command))
+	t)))
 
 ;; "hsettings.el" contains documentation for this variable.
-(or (boundp 'smart-scroll-proportional)
-    (defvar smart-scroll-proportional t
-      "*Non-nil means Smart Keys should scroll relative to current line when pressed at the end of a line.
+(unless (boundp 'smart-scroll-proportional)
+  (defvar smart-scroll-proportional t
+    "*Non-nil means Smart Keys should scroll relative to current line when pressed at the end of a line.
 Action Key moves current line to top of window.  Assist Key moves current
 line to bottom of window.  Repeated presses then scroll up or down a
 windowful.  Nil value instead ignores current line and always scrolls up or
 down a windowful."))
 
-;; The smart keys scroll buffers when pressed at the end of lines.
-;; These next two functions do the scrolling and keep point at the end
-;; of line to simplify repeated scrolls when using keyboard smart keys.
-;;
-;; These functions may also be used to test whether the scroll action would
-;; be successful: no action is taken if it would fail (because the beginning
-;; or end of a buffer is already showing) and nil is returned.
-;; t is returned whenever scrolling is performed.
-
-(defun hmouse-function (func assist-flag set-point-arg-list)
-  "Execute FUNC for Action Key (Assist Key with ASSIST-FLAG non-nil) and set point from SET-POINT-ARG-LIST.
+(defun hmouse-function (func assisting set-point-arg-list)
+  "Execute FUNC for Action Key (Assist Key with ASSISTING non-nil) and set point from SET-POINT-ARG-LIST.
 FUNC may be nil in which case no function is called.
 SET-POINT-ARG-LIST is passed to the call of the command bound to
 `hmouse-set-point-command'.  Return nil if `hmouse-set-point-command' variable
 is not bound to a valid function."
   (when (fboundp hmouse-set-point-command)
-    (if assist-flag
-	(setq assist-key-release-window (hmouse-key-release-window)
+    (if assisting
+	(setq assist-key-release-window (hmouse-key-release-window assist-key-release-position)
 	      assist-key-release-prev-point (point-marker))
-      (setq action-key-release-window (hmouse-key-release-window)
+      (setq action-key-release-window (hmouse-key-release-window action-key-release-position)
 	    action-key-release-prev-point (point-marker)))
     (and (eq major-mode 'br-mode)
 	 (setq action-mouse-key-prev-window
@@ -1398,7 +1457,7 @@ is not bound to a valid function."
 		 (selected-window))))
     (setq action-mouse-key-prefix-arg current-prefix-arg)
     (let ((release-args (hmouse-set-point set-point-arg-list)))
-      (if assist-flag
+      (if assisting
 	  (setq assist-key-release-args release-args)
 	(setq action-key-release-args release-args)))
     (when func
@@ -1407,6 +1466,15 @@ is not bound to a valid function."
 	    action-mouse-key-prefix-arg nil))
     t))
 
+;; The smart keys scroll buffers when pressed at the end of lines.
+;; These next two functions do the scrolling and keep point at the end
+;; of line to simplify repeated scrolls when using keyboard smart keys.
+;;
+;; These functions may also be used to test whether the scroll action would
+;; be successful: no action is taken if it would fail (because the beginning
+;; or end of a buffer is already showing) and nil is returned.
+;; t is returned whenever scrolling is performed.
+
 (defun smart-scroll-down ()
   "Scroll down according to value of smart-scroll-proportional.
 If smart-scroll-proportional is nil or if point is on the bottom window line,
@@ -1454,7 +1522,9 @@ not."
 	  (setq rtn nil)
 	(scroll-up)))
     (end-of-line)
-    (or rtn (progn (beep) (message "End of buffer")))
+    (unless rtn
+      (beep)
+      (message "End of buffer"))
     rtn))
 
 (provide 'hmouse-drv)
