@@ -1,10 +1,11 @@
-;;; hmouse-info.el --- Walks through Info networks using one key.
+;;; hmouse-info.el --- Walks through Info networks using one key.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    04-Apr-89
+;; Last-Mod:     17-Apr-22 at 11:16:29 by Bob Weiner
 ;;
-;; Copyright (C) 1989-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1989-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -36,16 +37,17 @@ the Top node in filename.  Signal an error if a filename without an
 index is given."
   (let ((completion-ignore-case t)
 	(Info-history-list nil)
-	Info-complete-menu-buffer
 	item-name)
     (save-window-excursion
       (info)
       (setq Info-complete-menu-buffer (clone-buffer)))
     (unwind-protect
-	(progn (while (equal "" (setq item-name
-				      (completing-read prompt #'Info-read-index-item-name-1 nil t))))
-	       item-name)
-      (kill-buffer Info-complete-menu-buffer))))
+	(progn
+	  (while (equal "" (setq item-name
+				 (completing-read prompt #'Info-read-index-item-name-1 nil t))))
+	  item-name)
+      (kill-buffer Info-complete-menu-buffer)
+      (makunbound 'Info-complete-menu-buffer))))
 
 ;;;###autoload
 (defun smart-info ()
@@ -72,7 +74,8 @@ or a Menu; otherwise returns nil."
     ;; If at end of node, go to next node
     ;;
     ((last-line-p)
-     (if (fboundp 'Info-global-next) (Info-global-next)
+     (if (fboundp 'Info-global-next)
+	 (Info-global-next)
        (Info-next)))
     ((and (fboundp 'Info-mouse-follow-link)
 	  (mouse-event-p action-key-release-args)
@@ -230,7 +233,7 @@ entry and returns t; otherwise returns nil."
 
 ;;; Much of this is derived in part from "info.el".
 
-(defun Info-build-menu-item-completions (string predicate action)
+(defun Info-build-menu-item-completions (string _predicate action)
   ;; See comments in `Info-complete-menu-item' for free variables used.
   (with-current-buffer Info-complete-menu-buffer
     (save-excursion
@@ -340,7 +343,7 @@ This works regardless of the current buffer."
 ;;;###autoload
 (defun Info-note-at-p ()
   "Return the name of the Info cross-reference note at point, or nil if none."
-  (let ((note-name) (opoint (point)))
+  (let ((opoint (point)))
     (save-excursion
       (skip-chars-forward "^:")
       (if (and (re-search-backward
@@ -370,7 +373,6 @@ See `completing-read' for a description of arguments and usage."
    ;; If a file name was given, complete index-items in the file.
    ((string-match "\\`(\\([^)]+\\))" string)
     (let ((file0 (match-string 0 string))
-	  (file1 (match-string 1 string))
 	  (index-item (substring string (match-end 0)))
 	  Info-complete-nodes
 	  completions)
