@@ -1,10 +1,11 @@
-;;; hmouse-sh.el --- System-dependent Smart Mouse Key bindings.
+;;; hmouse-sh.el --- System-dependent Smart Mouse Key bindings.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     3-Sep-91 at 21:40:58
+;; Last-Mod:     24-Jan-22 at 00:18:46 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -20,6 +21,16 @@
 
 (require 'hvar)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(defvar br-env-spec)
+(defvar br-lang-prefix)
+(defvar company-active-map)
+(defvar java-class-def-name-grpn)
+(defvar java-class-def-regexp)
+(defvar jedi-mode)
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
@@ -31,7 +42,7 @@
 ;;   bottom-divider
 
 (defun hmouse-bind-key-emacs (mouse-key-number depress-cmd release-cmd)
-  "Ensure MOUSE-KEY-NUMBER (1-5), e.g. 1 for [mouse-1], is globally bound to DEPRESS-CMD and RELEASE-CMD (includes depresses and drags).
+  "Ensure MOUSE-KEY-NUMBER (1-5), e.g. 1 for [mouse-1], is bound to DEPRESS-CMD and RELEASE-CMD (includes depresses and drags).
 Use nil as cmd values to unbind a key.  Works under GNU Emacs only."
   (hmouse-set-key-list
    depress-cmd
@@ -75,8 +86,7 @@ Use nil as cmd values to unbind a key.  Works under GNU Emacs only."
 	   [left-fringe down-mouse-5]
 	   [right-fringe down-mouse-5]
 	   [vertical-line down-mouse-5]
-	   [mode-line down-mouse-5])
-	  )))
+	   [mode-line down-mouse-5]))))
 	   
   (hmouse-set-key-list
    release-cmd
@@ -145,11 +155,10 @@ Use nil as cmd values to unbind a key.  Works under GNU Emacs only."
 	   [vertical-line drag-mouse-5]
 	   [vertical-line mouse-5]
 	   [mode-line drag-mouse-5]
-	   [mode-line mouse-5])
-	  ))))
+	   [mode-line mouse-5])))))
 
 (defun hmouse-bind-shifted-key-emacs (shifted-mouse-key-number depress-cmd release-cmd)
-  "Ensure SHIFTED-MOUSE-KEY-NUMBER (1-5), e.g. 1 for [Smouse-1], is globally bound to DEPRESS-CMD and RELEASE-CMD (includes depresses and drags).
+  "Ensure SHIFTED-MOUSE-KEY-NUMBER (1-5), e.g. 1 for [Smouse-1], is bound to DEPRESS-CMD and RELEASE-CMD (includes depresses and drags).
 Use nil as CMD value to unbind the key.  Works under GNU Emacs only."
   (hmouse-set-key-list
    depress-cmd
@@ -183,8 +192,7 @@ Use nil as CMD value to unbind the key.  Works under GNU Emacs only."
 	   [left-fringe S-down-mouse-5]
 	   [right-fringe S-down-mouse-5]
 	   [vertical-line S-down-mouse-5]
-	   [mode-line S-down-mouse-5])
-	  )))
+	   [mode-line S-down-mouse-5]))))
 	  
   (hmouse-set-key-list
    release-cmd
@@ -253,11 +261,10 @@ Use nil as CMD value to unbind the key.  Works under GNU Emacs only."
 	   [vertical-line S-drag-mouse-5]
 	   [vertical-line S-mouse-5]
 	   [mode-line S-drag-mouse-5]
-	   [mode-line S-mouse-5])
-	  ))))
+	   [mode-line S-mouse-5])))))
 	   
 (defun hmouse-get-bindings (hmouse-middle-flag)
-  "Return the list of current bindings of mouse keys used by Hyperbole.
+  "Return the list of active bindings of mouse keys used by Hyperbole.
 If HMOUSE-MIDDLE-FLAG is non-nil, includes the middle mouse key binding as well.
 These may be the bindings prior to initializing Hyperbole or the Hyperbole bindings."
   ;; Do nothing when running in batch mode.
@@ -267,7 +274,7 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
      ;; Get mouse bindings under Emacs or XEmacs, even if not under a
      ;; window system since they can have frames on ttys and windowed
      ;; displays at the same time.
-     (mapcar (lambda (key) (cons key (global-key-binding key)))
+     (mapcar (lambda (key) (cons key (key-binding key)))
 	     (if (eq window-system 'dps)
 		 ;; NEXTSTEP offers only 2 shift-mouse buttons which we use
 		 ;; as the Smart Keys.
@@ -294,8 +301,7 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 		   [mode-line S-down-mouse-1] [mode-line S-drag-mouse-1]
 		   [mode-line S-mouse-1]
 		   [mode-line S-down-mouse-2] [mode-line S-drag-mouse-2]
-		   [mode-line S-mouse-2]
-		   )
+		   [mode-line S-mouse-2])
 	       ;; X, macOS or MS Windows
 	       '([S-down-mouse-2] [S-drag-mouse-2] [S-mouse-2]
 		 [S-down-mouse-3] [S-drag-mouse-3] [S-mouse-3]
@@ -320,11 +326,10 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 		 [mode-line S-down-mouse-2] [mode-line S-drag-mouse-2]
 		 [mode-line S-mouse-2]
 		 [mode-line S-down-mouse-3] [mode-line S-drag-mouse-3]
-		 [mode-line S-mouse-3]
-		 )))
+		 [mode-line S-mouse-3])))
      (nconc
       (mapcar (lambda (key)
-		(cons key (global-key-binding key)))
+		(cons key (key-binding key)))
 	      '([(shift button2)] [(shift button2up)]
 		[(shift button3)] [(shift button3up)]))
       (when (boundp 'mode-line-map)
@@ -335,7 +340,7 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 (defun hmouse-get-unshifted-bindings ()
   "Return the list of middle mouse key bindings prior to their use as Smart Keys."
   ;; Do nothing when running in batch mode.
-  (mapc (lambda (key) (cons key (global-key-binding key)))
+  (mapc (lambda (key) (cons key (key-binding key)))
 	(unless (eq window-system 'dps)
 	  ;; X, macOS or MS Windows
 	  '([down-mouse-2] [drag-mouse-2] [mouse-2]
@@ -359,20 +364,17 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 	    [mode-line down-mouse-2] [mode-line drag-mouse-2]
 	    [mode-line mouse-2]
 	    [mode-line down-mouse-3] [mode-line drag-mouse-3]
-	    [mode-line mouse-3]
-	    )))
+	    [mode-line mouse-3])))
   (nconc
    (mapcar (lambda (key)
-	     (cons key (global-key-binding key)))
+	     (cons key (key-binding key)))
 	   '([button2] [button2up]
-	     [button3] [button3up]
-	     ))
+	     [button3] [button3up]))
    (when (boundp 'mode-line-map)
      (mapcar (function
 	      (lambda (key)
 		(cons key (lookup-key mode-line-map key))))
-	     '([button3] [button3up])))
-   ))
+	     '([button3] [button3up])))))
 
 ;; Based on a function from Emacs mouse.el.
 (defun hmouse-posn-set-point (position)
@@ -388,6 +390,28 @@ Select the corresponding window as well."
   (when (numberp (posn-point position))
     (goto-char (posn-point position))))
 
+;; Based on mouse-drag-region from Emacs mouse.el.
+(defun hmouse-drag-region (start-event)
+  "Set the region to the text that the mouse is dragged over.
+If not the start of a region drag-and-drop, then depress the Action Key.
+Highlight the drag area as you move the mouse.
+This must be bound to a button-down mouse event.
+In Transient Mark mode, the highlighting remains as long as the mark
+remains active.  Otherwise, it remains until the next input event.
+
+When the region already exists and `mouse-drag-and-drop-region'
+is non-nil, this moves the entire region of text to where mouse
+is dragged over to."
+  (interactive "e")
+  (if (and mouse-drag-and-drop-region
+           (not (member 'triple (event-modifiers start-event)))
+           (equal (mouse-posn-property (event-start start-event) 'face) 'region))
+      (mouse-drag-and-drop-region start-event)
+    ;; Give temporary modes such as isearch a chance to turn off.
+    (run-hooks 'mouse-leave-buffer-hook)
+    (action-key-depress start-event)
+    (mouse-drag-track start-event)))
+
 ;; Based on a function from Emacs mouse.el.
 (defun hmouse-move-point-emacs (event &optional promote-to-region)
   "Move point to the position clicked on with the mouse.
@@ -402,7 +426,8 @@ point determined by `mouse-select-region-move-to-beginning'."
       (with-selected-frame start-w-or-f (setq start-w-or-f (selected-window))))
     (when (framep end-w-or-f)
       (with-selected-frame end-w-or-f (setq end-w-or-f (selected-window))))
-    (if (and (window-minibuffer-p start-w-or-f)
+    (if (and (window-valid-p start-w-or-f)
+	     (window-minibuffer-p start-w-or-f)
 	     (not (minibuffer-window-active-p start-w-or-f)))
 	;; Select the ending frame only, not the window pressed within.
 	(select-frame (window-frame end-w-or-f))
@@ -416,19 +441,21 @@ point determined by `mouse-select-region-move-to-beginning'."
 		     (exchange-point-and-mark))))
 	;; Use event-end in case called from mouse-drag-region.
 	;; If EVENT is a click, event-end and event-start give same value.
-	(if (and (window-minibuffer-p end-w-or-f)
+	(if (and (window-valid-p end-w-or-f)
+		 (window-minibuffer-p end-w-or-f)
 		 (not (minibuffer-window-active-p end-w-or-f)))
 	    ;; Select the ending frame only, not the window pressed within.
 	    (select-frame (window-frame end-w-or-f))
 	  (condition-case ()
 	      (hmouse-posn-set-point (event-end event))
-	    (error (select-frame (window-frame end-w-or-f)))))))))
+	    (error (when (window-valid-p end-w-or-f)
+		     (select-frame (window-frame end-w-or-f))))))))))
 
 (defun hmouse-move-point-eterm (arg-list)
   (apply 'mouse-move-point arg-list))
 
 (defun hmouse-set-key-list (binding key-list)
-  (mapc (lambda (key) (hkey-global-set-key key binding)) key-list)
+  (mapc (lambda (key) (hkey-set-key key binding)) key-list)
   nil)
 
 (defun hmouse-shifted-setup (hmouse-middle-flag)
@@ -449,7 +476,7 @@ point determined by `mouse-select-region-move-to-beginning'."
     ;; this.
     (setq kmacro-call-mouse-event nil)
     ;;
-    (setq hmouse-set-point-command 'hmouse-move-point-emacs)
+    (setq hmouse-set-point-command #'hmouse-move-point-emacs)
     (if (eq window-system 'dps)
 	;; NEXTSTEP offers only 2 shift-mouse buttons which we use as the Smart Keys.
 	(progn
@@ -471,16 +498,15 @@ point determined by `mouse-select-region-move-to-beginning'."
 With optional MIDDLE-KEY-ONLY-FLAG non-nil, bind only the middle mouse key."
   (interactive)
   ;; Globally Emacs uses key-translation-map to link mouse-1 to
-  ;; do whatever mouse-2 does but because Hyperbole uses both down
-  ;; and up bindings on mouse2, this does not work to follow links.
-  ;; Disable use of mouse following on button1 and use button2 only
-  ;; for that.  Use mouse1 only for setting point and drag selecting
-  ;; regions.
-  ;;
-  ;; Don't bind mouse-1 to action-key-depress-emacs to allow it to
-  ;; follow links because that will disable the region highlighting
-  ;; that we would rather allow.`
-  (customize-set-variable 'mouse-1-click-follows-link nil)
+  ;; do whatever mouse-2 does when 'mouse-1-click-follows-link' is
+  ;; non-nil (the default) and point is on an Emacs button.
+  ;; Since Hyperbole rebinds mouse-2 here, this does not work by
+  ;; default.  We fix that here by overriding the setting of
+  ;; down-mouse-1 to include the down Action Key command plus the
+  ;; original mouse-1 command.  Then when mouse-1 is released on an
+  ;; Emacs button, the Action Key command is run which handles
+  ;; activating the button/following a link.
+  (global-set-key [down-mouse-1] 'hmouse-drag-region)
   ;;
   ;; Unbind Emacs push-button mouse keys since Hyperbole handles them.
   (define-key button-map [mouse-2] nil)
@@ -489,19 +515,8 @@ With optional MIDDLE-KEY-ONLY-FLAG non-nil, bind only the middle mouse key."
   ;; Remove push-button help echo string for mouse-2 key.
   (put 'default-button 'help-echo nil)
   ;;
-  ;; In Info-mode, Emacs uses key-translation-map to link mouse-1 to
-  ;; do whatever mouse-2 does but because Hyperbole uses both down
-  ;; and up bindings on mouse2, this does not work.  So we rebind
-  ;; mouse-1 in Info mode to be an actual Action Mouse Key (which
-  ;; makes it follow Info links/cross-references properly, doing a
-  ;; superset of what it did before).
-  (var:add-and-run-hook 'Info-mode-hook
- 			(lambda ()
-			  (define-key Info-mode-map [down-mouse-1] 'action-key-depress-emacs)
-			  (define-key Info-mode-map [mouse-1] 'action-mouse-key-emacs)
-			  (define-key Info-mode-map [double-down-mouse-1] 'action-key-depress-emacs)
-			  (define-key Info-mode-map [double-mouse-1] 'action-mouse-key-emacs)
-			  (define-key Info-mode-map [mouse-2] nil)))
+  ;; Remove mouse-2 binding In Info-mode since Hyperbole handles it.
+  (var:add-and-run-hook 'Info-mode-hook (lambda () (define-key Info-mode-map [mouse-2] nil)))
   ;;
   (unless (eq window-system 'dps)
     ;; X, macOS or MS Windows
