@@ -1,10 +1,11 @@
-;;; hmouse-tag.el --- Smart Key support of programming language constructs
+;;; hmouse-tag.el --- Smart Key support of programming language constructs  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    24-Aug-91
+;; Last-Mod:     12-Feb-22 at 10:42:20 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -128,6 +129,32 @@ Keyword matched is grouping 1.  Referent is grouping 2.")
   :type '(file :must-match t)
   :group 'hyperbole-commands)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function br-edit "ext:br")
+(declare-function br-edit-feature "ext:br-ftr")
+(declare-function python-import-file "ext:br-python-ft")
+(declare-function python-to-definition "ext:br-python-ft")
+
+(declare-function epc:manager-server-process "ext:epc")
+
+(declare-function java-to-definition "ext:br-java-ft")
+
+(declare-function jedi:-get-servers-in-use "ext:jedi-core")
+(declare-function jedi:goto--line-column "ext:jedi-core")
+(declare-function jedi:goto-definition "ext:jedi-core")
+
+(declare-function objc-to-definition "ext:br-objc-ft")
+
+(defvar br-env-spec)
+(defvar br-lang-prefix)
+(defvar buffer-tag-table)
+(defvar jedi-mode)
+(defvar java-class-def-name-grpn)
+(defvar java-class-def-regexp)
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
@@ -154,7 +181,7 @@ If:
 	    (progn
 	      (smart-tags-display tag next)
 	      (message "Found definition for `%s'" tag))
-	  (error (message "`%s' not found in tag tables" tag)
+	  (error (message "`%s' definition not found in identifier lookup/tag tables" tag)
 		 (beep))))))
 
 ;;;###autoload
@@ -211,7 +238,7 @@ Otherwise:
       (error
        (if (or (not smart-c-use-lib-man)
 	       (not (file-readable-p "~/.CLIBS-LIST")))
-	   (progn (message "`%s' not found in tag tables" tag)
+	   (progn (message "`%s' definition not found in identifier lookup/tag tables" tag)
 		  (beep)
 		  nil)
 	 (message "Checking if `%s' is a C++ library function..." tag)
@@ -219,7 +246,7 @@ Otherwise:
 	     (progn (message "Displaying C++ library man page for `%s'" tag)
 		    (manual-entry tag)
 		    t)
-	   (message "`%s' not found in tag tables or C++ libraries" tag)
+	   (message "`%s' definition not found in identifier lookup/tag tables or C++ libraries" tag)
 	   (beep)
 	   nil))))))
 
@@ -291,13 +318,13 @@ If:
 	  (error
 	   (if (or (not smart-c-use-lib-man)
 		   (not (file-readable-p "~/.CLIBS-LIST")))
-	       (progn (message "`%s' not found in tag tables" tag)
+	       (progn (message "`%s' definition not found in identifier lookup/tag tables" tag)
 		      (beep))
 	     (message "Checking if `%s' is a C library function..." tag)
 	     (if (smart-library-symbol tag)
 		 (progn (message "Displaying C library man page for `%s'" tag)
 			(manual-entry tag))
-	       (message "`%s' not found in tag tables or C libraries" tag)
+	       (message "`%s' definition not found in identifier lookup/tag tables or C libraries" tag)
 	       (beep))))))))
 
 (defconst smart-c-keywords
@@ -349,7 +376,7 @@ If:
 (defun smart-emacs-lisp-mode-p ()
   "Return t if in a mode which use Emacs Lisp symbols."
   ;; Beyond Lisp files, Emacs Lisp symbols appear frequently in Byte-Compiled
-  ;; buffers, debugger buffers, and Help buffers.
+  ;; buffers, debugger buffers, program ChangeLog buffers, and Help buffers.
   (or (memq major-mode #'(emacs-lisp-mode lisp-interaction-mode debugger-mode))
       (string-match "\\`\\*Compile-Log\\(-Show\\)?\\*" (buffer-name))
       (and (or (memq major-mode #'(help-mode change-log-mode))
@@ -374,7 +401,7 @@ in the current directory or any of its ancestor directories."
 	  (smart-tags-display tag next)
 	  (message "Found definition for `%s'" tag))
       (error
-       (message "`%s' not found in tag tables" tag)
+       (message "`%s' definition not found in identifier lookup/tag tables" tag)
        (beep)))))
 
 (defconst smart-fortran-keywords
@@ -443,11 +470,11 @@ Otherwise:
 	(progn
 	  (smart-tags-display tag next)
 	  (message "Found definition for `%s'" tag))
-      (error (progn (message "`%s' not found in tag tables" tag)
+      (error (progn (message "`%s' definition not found in identifier lookup/tag tables" tag)
 		    (beep))))))
 
 ;;; The following should be called only if the OO-Browser is available.
-(defun smart-java-oo-browser (&optional junk)
+(defun smart-java-oo-browser (&optional _junk)
   "Jumps to the definition of selected Java construct via OO-Browser support.
 Optional JUNK is ignored.  Does nothing if the OO-Browser is not available.
 
@@ -512,7 +539,7 @@ in the current directory or any of its ancestor directories."
 	  (smart-tags-display tag next)
 	  (message "Found definition for `%s'" tag))
       (error
-       (message "`%s' not found in tag tables" tag)
+       (message "`%s' definition not found in identifier lookup/tag tables" tag)
        (beep)))))
 
 (defconst smart-javascript-keywords
@@ -560,23 +587,28 @@ Excludes character matching square brackets, so may be used with skip-characters
 
 (defun smart-lisp (&optional show-doc)
   "Jump to the definition of any selected Lisp identifier or optionally SHOW-DOC.
+
+Assume caller has checked that (smart-emacs-lisp-mode-p) is true.
+
 If on an Emacs Lisp require, load, or autoload clause and the
 `find-library' function from the \"load-library\" package has
 been loaded, this jumps to the library source whenever possible.
 
 Otherwise, if a definition for the identifier is found within a TAGS
 file in the current directory or any of its ancestor directories, this
-jumps to the definition.
+jumps to the definition.  Supports Hyperbole implicit button types and
+action types.
 
-Optional SHOW-DOC flag means show documentation for the tag at point
-rather than displaying its source code definition.
+With optional SHOW-DOC flag, show documentation for the tag at point
+rather than displaying its source code definition.  In this case, tag
+must be a bound or fbound symbol or it is ignored.
 
 This command assumes that its caller has already checked that the key was
 pressed in an appropriate buffer and has moved the cursor to the selected
 buffer."
 
   (interactive)
-  (unless (and (smart-emacs-lisp-mode-p) (fboundp 'find-library)
+  (unless (and (fboundp 'find-library)
 	       ;; Handle Emacs Lisp `require', `load', and `autoload' clauses.
 	       (let ((opoint (point))
 		     type
@@ -606,30 +638,54 @@ buffer."
 			 (goto-char (match-beginning 0))
 		       (error "(smart-lisp): Found autoload library but no definition for `%s'" name)))
 		   t)))
-    (let* ((elisp-p (smart-emacs-lisp-mode-p))
-	   (tag (smart-lisp-at-tag-p t))
-	   (tag-sym (intern-soft tag)))
-      (cond ((and show-doc elisp-p)
-	     ;; Emacs Lisp function, variable and face documentation display.
-	     (cond ((fboundp tag-sym) (describe-function tag-sym))
-		   ((and tag-sym (boundp tag-sym)) (describe-variable tag-sym))
-		   ((facep tag-sym) (describe-face tag-sym))
-		   (t (error "(smart-lisp): `%s' unbound symbol definition not found" tag))))
-	    ((and elisp-p (fboundp 'find-function-noselect)
-		  (let ((result (smart-lisp-bound-symbol-def tag-sym)))
-		    (when (cdr result)
-		      (hpath:display-buffer (car result))
-		      (goto-char (cdr result))
-		      t))))
-	    (t (condition-case ()
-		   ;; Tag of any language
-		   (and (featurep 'etags) (smart-tags-display tag show-doc))
-		 (error (unless (and elisp-p (stringp smart-emacs-tags-file)
-				     (condition-case ()
-					 (smart-tags-display
-					  tag show-doc (list smart-emacs-tags-file))
-				       (error nil)))
-			  (error "(smart-lisp): `%s' definition not found in any tag table" tag)))))))))
+    (smart-lisp-find-tag nil show-doc)))
+
+(defun smart-lisp-find-tag (&optional tag show-doc)
+  "Find the definition of optional Lisp TAG (or identifier at point when TAG is nil) or show its documentation with optional prefix arg SHOW-DOC non-nil.
+Use `hpath:display-buffer' to show definition or documentation."
+  (interactive
+   (list (read-string (format "%s Lisp identifier: "
+			      (if current-prefix-arg
+				  "Show doc for" "Find")))
+	 current-prefix-arg))
+  (unless (stringp tag)
+    (setq tag (smart-lisp-at-tag-p t)))
+  (let* ((elisp-flag (smart-emacs-lisp-mode-p))
+	 (tag-sym (intern-soft tag)))
+    (cond ((and show-doc elisp-flag)
+	   ;; Emacs Lisp function, variable and face documentation display.
+	   (cond ((fboundp tag-sym) (describe-function tag-sym))
+		 ((and tag-sym (boundp tag-sym)) (describe-variable tag-sym))
+		 ((facep tag-sym) (describe-face tag-sym))
+		 ;; (t (error "(smart-lisp): `%s' unbound symbol definition not found" tag))
+		 ;; Ignore unbound symbols if displaying doc.
+		 (t nil)))
+	  ((and elisp-flag (fboundp 'find-function-noselect)
+		(let ((result (smart-lisp-bound-symbol-def tag-sym)))
+		  (when (cdr result)
+		    (hpath:display-buffer (car result))
+		    (goto-char (cdr result))
+		    t))))
+	  ;; If elisp-flag is true, then make xref use tags tables to
+	  ;; find symbols not yet loaded into Emacs; otherwise, use
+	  ;; standard xref backends for the current language.
+	  (t (let ((etags-mode (and elisp-flag (boundp 'xref-etags-mode) xref-etags-mode)))
+	       (unwind-protect
+		   (progn
+		     (and (not etags-mode) elisp-flag (fboundp 'xref-etags-mode)
+			  (xref-etags-mode 1))
+		     (condition-case ()
+			 ;; Tag of any language
+			 (when (featurep 'etags)
+			   (smart-tags-display tag show-doc))
+		       (error (unless (and elisp-flag (stringp smart-emacs-tags-file)
+					   (condition-case ()
+					       (smart-tags-display
+						tag show-doc (list smart-emacs-tags-file))
+					     (error nil)))
+				(error "(smart-lisp): No definition found for `%s'" tag)))))
+		 (and (not etags-mode) elisp-flag (fboundp 'xref-etags-mode)
+		      (xref-etags-mode 0))))))))
 
 (defun smart-lisp-at-definition-p ()
     "Return t when point is in a non-help buffer on the first line of a non-alias Lisp definition, else nil."
@@ -650,7 +706,7 @@ buffer."
       (looking-at "\\(;*[ \t]*\\)?(\\(autoload\\|load\\|require\\)")))
 
 (defun smart-lisp-at-change-log-tag-p ()
-  "When in a change-log mode, match to only bound Elisp identifiers and those with a '-' somewhere in the middle.
+  "When in a change-log mode, match to bound Elisp identifiers only and those with a '-' somewhere in the middle.
 These tight tests help eliminate undesired matches.
 Returns matching ELisp tag name that point is within, else nil."
   (when (derived-mode-p 'change-log-mode)
@@ -658,31 +714,63 @@ Returns matching ELisp tag name that point is within, else nil."
       (and identifier (intern-soft identifier)
 	   (string-match "[^-]-[^-]" identifier)))))
 
+(defun smart-lisp-htype-tag (tag)
+  "Given TAG at point, if it is a Hyperbole type, return the full symbol name, else return the TAG unchanged."
+  (setq tag (cond ((and tag (string-match-p "::" tag))
+		   tag)
+		  ((and tag
+			(save-excursion
+			  (skip-chars-backward "^ \t\n\r\f")
+			  (skip-chars-backward " \t\n\r\f")
+			  (equal (smart-lisp-at-non-htype-tag-p t) "hact")))
+		   ;; If tag is preceded by an 'hact' call, then treat as a Hyperbole actype.
+		   (or (symtable:actype-p tag) tag))
+		  (tag
+		   (if (intern-soft tag)
+		       tag
+		     (or (symtable:ibtype-p tag) (symtable:actype-p tag) tag)))))
+  (cond ((or (null tag) (stringp tag))
+	 tag)
+	((symbolp tag)
+	 (symbol-name tag))))
+
 (defun smart-lisp-at-tag-p (&optional no-flash)
-  "Return Lisp tag name that point is within, else nil.
+  "Return possibly non-existent Lisp tag name that point is within, else nil.
+Return nil when point is on the first line of a non-alias Lisp definition.
+
+Resolve Hyperbole implicit button type and action type references."
+  (smart-lisp-htype-tag 
+   (smart-lisp-at-non-htype-tag-p no-flash)))
+
+(defun smart-lisp-at-non-htype-tag-p (&optional no-flash)
+  "Return possibly non-existent Lisp tag name that point is within, else nil.
 Return nil when point is on the first line of a non-alias Lisp definition."
   (unless (smart-lisp-at-definition-p)
     (save-excursion
       (skip-chars-backward smart-lisp-identifier-chars)
-      (when (and (looking-at smart-lisp-identifier)
-		 ;; Ignore any punctuation matches.
-		 (save-match-data
-		   (not (string-match "\\`[-<>*]+\\'" (match-string 0)))))
-	;; Ignore any leading '<' character from action buttons or
-	;; other links, i.e. only when followed by an alphabetic character.
-	(when (and (eq (following-char) ?\<) (eq ?w (char-syntax (1+ (point)))))
-	  (goto-char (1+ (point))))
-	(if no-flash
+      ;; Ignore double quoted strings and environment variables: $PATH, ${PATH}, $(PATH), {$PATH}, ($PATH)
+      (unless (save-excursion
+		(skip-chars-backward "[\"${(]")
+		(looking-at "\"\\|\\$[{(]\\|[{]\\$]"))
+	(when (and (looking-at smart-lisp-identifier)
+		   ;; Ignore any punctuation matches.
+		   (save-match-data
+		     (not (string-match "\\`[-<>*]+\\'" (match-string 0)))))
+	  ;; Ignore any leading '<' character from action buttons or
+	  ;; other links, i.e. only when followed by an alphabetic character.
+	  (when (and (eq (following-char) ?\<) (eq ?w (char-syntax (1+ (point)))))
+	    (goto-char (1+ (point))))
+	  (if no-flash
+	      (if (eq (char-after (1- (match-end 0))) ?:)
+		  (buffer-substring-no-properties (point) (1- (match-end 0)))
+		(buffer-substring-no-properties (point) (match-end 0)))
 	    (if (eq (char-after (1- (match-end 0))) ?:)
-		(buffer-substring-no-properties (point) (1- (match-end 0)))
-	      (buffer-substring-no-properties (point) (match-end 0)))
-	  (if (eq (char-after (1- (match-end 0))) ?:)
+		(smart-flash-tag
+		 (buffer-substring-no-properties (point) (1- (match-end 0)))
+		 (point) (1- (match-end 0)))
 	      (smart-flash-tag
-	       (buffer-substring-no-properties (point) (1- (match-end 0)))
-	       (point) (1- (match-end 0)))
-	    (smart-flash-tag
-	     (buffer-substring-no-properties (point) (match-end 0))
-	     (point) (match-end 0))))))))
+	       (buffer-substring-no-properties (point) (match-end 0))
+	       (point) (match-end 0)))))))))
 
 ;;;###autoload
 (defun smart-lisp-mode-p ()
@@ -730,7 +818,7 @@ Otherwise:
       (error
        (if (or (not smart-c-use-lib-man)
 	       (not (file-readable-p "~/.CLIBS-LIST")))
-	   (progn (message "`%s' not found in tag tables" tag)
+	   (progn (message "`%s' definition not found in identifier lookup/tag tables" tag)
 		  (beep))
 	 (message
 	  "Checking if `%s' is an Objective-C library function..." tag)
@@ -739,12 +827,12 @@ Otherwise:
 	       (message
 		"Displaying Objective-C library man page for `%s'" tag)
 	       (manual-entry tag))
-	   (message "`%s' not found in tag tables or Objective-C libraries"
+	   (message "`%s' definition not found in identifier lookup/tag tables or Objective-C libraries"
 		    tag)
 	   (beep)))))))
 
 ;;; The following should be called only if the OO-Browser is available.
-(defun smart-objc-oo-browser (&optional junk)
+(defun smart-objc-oo-browser (&optional _junk)
   "Jump to the definition of selected Objective-C construct via OO-Browser support.
 Optional JUNK is ignored.  Does nothing if the OO-Browser is not available.
 
@@ -848,11 +936,11 @@ in the current directory or any of its ancestor directories."
 	(progn
 	  (smart-tags-display tag next)
 	  (message "Found definition for `%s'" tag))
-      (error (progn (message "`%s' not found in tag tables" tag)
+      (error (progn (message "`%s' definition not found in identifier lookup/tag tables" tag)
 		    (beep))))))
 
 ;;; The following should be called only if the OO-Browser is available.
-(defun smart-python-oo-browser (&optional junk)
+(defun smart-python-oo-browser (&optional _junk)
   "Jumps to the definition of selected Python construct via OO-Browser support.
 Optional JUNK is ignored.  Does nothing if the OO-Browser is not available.
 
@@ -1275,15 +1363,16 @@ See the \"${hyperb:dir}/smart-clib-sym\" script for more information."
       found)))
 
 (defun smart-tags-display (tag next &optional list-of-tags-tables)
-  (if next (setq tag nil))
+  (when  next (setq tag nil))
   (let* ((tags-table-list (or list-of-tags-tables
 			      (and (boundp 'tags-table-list)
 				   (nconc (smart-tags-file-list) tags-table-list))
 			      (smart-tags-file-list)))
+	 ;; Identifier searches should almost always be case-sensitive today
+	 (tags-case-fold-search nil)
 	 (func (smart-tags-noselect-function))
-	 (tags-file-name (if tags-table-list
-			     nil
-			   (and (boundp 'tags-file-name) tags-file-name)))
+	 (tags-file-name (unless tags-table-list
+			   (when (boundp 'tags-file-name) tags-file-name)))
 	 find-tag-result
 	 ;; For InfoDock and XEmacs
 	 (tags-always-exact t)
@@ -1295,29 +1384,33 @@ See the \"${hyperb:dir}/smart-clib-sym\" script for more information."
     ;; For InfoDock (XEmacs may also take this branch), force exact match
     ;; when `next' is false (otherwise tag would = nil and the following
     ;; stringp test would fail).
-    (if (featurep 'infodock)
-	(if (stringp tag)
-	    (setq tag (list tag))))
-    (if (and func (setq find-tag-result (funcall func tag)))
-	(cond ((eq func 'find-tag-internal)
-	       ;; InfoDock and XEmacs
-	       (hpath:display-buffer (car find-tag-result))
-	       (goto-char (cdr find-tag-result)))
-	      ((vectorp find-tag-result)
-	       ;; Newer GNU Emacs with xref.el
-	       (hpath:display-buffer (xref-item-buffer find-tag-result))
-	       (goto-char (xref-item-position find-tag-result)))
-	      ((bufferp find-tag-result)
-	       ;; Older GNU Emacs
-	       (hpath:display-buffer find-tag-result))
-	      (t
-	       ;; Emacs with some unknown version of tags.
-	       ;; Signals an error if tag is not found which is caught by
-	       ;; many callers of this function.
-	       (with-no-warnings (find-tag tag))))
-      ;; Signals an error if tag is not found which is caught by
-      ;; many callers of this function.
-      (with-no-warnings (find-tag tag)))))
+    (and (featurep 'infodock) (stringp tag) (setq tag (list tag)))
+    (cond ((and func (or tags-table-list tags-file-name) (setq find-tag-result (funcall func tag)))
+	   (cond ((eq func 'find-tag-internal)
+		  ;; InfoDock and XEmacs
+		  (hpath:display-buffer (car find-tag-result))
+		  (goto-char (cdr find-tag-result)))
+		 ((or (eq (type-of find-tag-result) 'xref-item)
+		      (vectorp find-tag-result))
+		  ;; Newer GNU Emacs with xref.el
+		  (hpath:display-buffer (xref-item-buffer find-tag-result))
+		  (goto-char (xref-item-position find-tag-result)))
+		 ((bufferp find-tag-result)
+		  ;; Older GNU Emacs
+		  (hpath:display-buffer find-tag-result))
+		 (t
+		  ;; Emacs with some unknown version of tags.
+		  ;; Signals an error if tag is not found which is caught by
+		  ;; many callers of this function.
+		  ;; Find exact identifier matches only.
+		  (with-no-warnings (find-tag (concat "\\`" (regexp-quote tag) "\\'") nil t)))))
+	  ((or tags-table-list tags-file-name)
+	   ;; Signals an error if tag is not found which is caught by
+	   ;; many callers of this function.
+	   ;; Find exact identifier matches only.
+	   (with-no-warnings (find-tag (concat "\\`" (regexp-quote tag) "\\'") nil t)))
+	  (t
+	   (error "No existing tag tables in which to find `%s'" tag)))))
 
 ;;;###autoload
 (defun smart-tags-file-path (file)
@@ -1357,11 +1450,12 @@ to look.  If no tags file is found, an error is signaled."
 	     smart-emacs-tags-file
 	     (smart-emacs-lisp-mode-p)
 	     (let ((path (file-name-directory curr-dir-or-filename)))
-	       (and path (delq nil (mapcar
-				    (lambda (p)
-				      (and p (equal (file-name-as-directory p)
-						    path)))
-				    load-path)))))
+	       (when path
+		 (delq nil (mapcar
+			    (lambda (p)
+			      (when p
+				(equal (file-name-as-directory p) path)))
+			    load-path)))))
 	(setq tags-table-list (list smart-emacs-tags-file)))
     ;; Return the appropriate tags file list.
     (cond (tags-table-list
@@ -1374,10 +1468,10 @@ to look.  If no tags file is found, an error is signaled."
 	  ((fboundp 'buffer-tag-table-list)
 	   ;; InfoDock and XEmacs
 	   (buffer-tag-table-list))
-	  ((and (boundp 'buffer-tag-table) buffer-tag-table)
+	  ((when (boundp 'buffer-tag-table) buffer-tag-table)
 	   ;; InfoDock and XEmacs
 	   (list buffer-tag-table))
-	  ((and (boundp 'tags-file-name) tags-file-name)
+	  ((when (boundp 'tags-file-name) tags-file-name)
 	   (list tags-file-name))
 	  (t (error "Needed tags file not found; see `man etags' for how to build one")))))
 
