@@ -1,10 +1,11 @@
-;;; htz.el --- Timezone-based time and date support for GNU Hyperbole
+;;; htz.el --- Timezone-based time and date support for GNU Hyperbole  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Masanobu Umeda             / Bob Weiner
 ;;
 ;; Orig-Date:    14-Oct-91 at 07:22:08
+;; Last-Mod:     24-Jan-22 at 00:18:47 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; Derived from "timezone.el" in GNU Emacs.
@@ -270,8 +271,7 @@ Optional argument TIMEZONE specifies a time zone."
 	  (capitalize (car (rassq month htz:months-assoc)))
 	  (- year (* (/ year 100) 100))	;1990 -> 90
 	  time
-	  (if timezone (concat " " timezone) "")
-	  ))
+	  (if timezone (concat " " timezone) "")))
 
 (defun htz:date-make-unix (year month day time &optional timezone)
   "Approximate Unix date format from YEAR, MONTH, DAY, and TIME.
