@@ -1,10 +1,11 @@
-;;; hui-jmenu.el --- Popup menus for jumping to and managing buffers, frames, and windows
+;;; hui-jmenu.el --- Popup menus for jumping to and managing buffers, frames, and windows  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     9-Mar-94 at 23:37:28
+;; Last-Mod:     24-Jan-22 at 00:18:47 by Bob Weiner
 ;;
-;; Copyright (C) 1994-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1994-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -133,8 +134,7 @@
     "----"
     ["Pop-from-Ring"      hywconfig-delete-pop      (not (hywconfig-ring-empty-p))]
     ["Save-to-Ring"       hywconfig-ring-save       t]
-    ["Yank-from-Ring"     hywconfig-yank-pop        (not (hywconfig-ring-empty-p))]
-    ))
+    ["Yank-from-Ring"     hywconfig-yank-pop        (not (hywconfig-ring-empty-p))]))
 
 ;;; ************************************************************************
 ;;; Private functions
@@ -144,8 +144,9 @@
   (let ((mname (buffer-local-value 'mode-name buffer)))
     (if mname
 	;; Next line needed to ensure mode name is always formatted as
-	;; a string.
-	(format-mode-line mname)
+	;; a string and spaces are replaced with dashes.
+	(subst-char-in-string ?\ ?-
+			      (format-mode-line (or (car-safe mname) mname)))
       (capitalize (symbol-name (buffer-local-value 'major-mode buffer))))))
 
 (defun hui-menu-frame-name (frame)
@@ -197,27 +198,26 @@
 
 (defun hui-menu-sort-buffers (buffer-and-mode-name-list)
   "Reverse sort and return list of (`buffer-name' . `mode-name') elements by `mode-name' and then by `buffer-name'."
-  (let ((buf (get-buffer-create " tmp-sort")))
-    (with-current-buffer buf
-      (setq buffer-read-only nil)
-      (erase-buffer)
-      (let ((standard-output (current-buffer)))
-	(mapc #'print buffer-and-mode-name-list))
-     (while (search-forward "\n\n" nil t)
-	(replace-match "\n"))
-      (if (hui-menu-program-path "sort")
-	  (call-process-region (point-min) (point-max)
-			       "sort" t t nil "-r" "-k3,3" "-k1,1")
-	;; This fallback of sort-fields can only sort on one field, so
-	;; sort by major-mode and leave buffers within each mode
-	;; unsorted when no UNIX sort program is available.
-	(sort-fields 3 (point-min) (point-max))
-	(reverse-region (point-min) (point-max)))
-      (insert "\)\n")
-      (goto-char (point-min))
-      (insert "\(")
-      (goto-char (point-min))
-      (read (current-buffer)))))
+  (with-temp-buffer
+    (setq buffer-read-only nil)
+    (erase-buffer)
+    (let ((standard-output (current-buffer)))
+      (mapc #'print buffer-and-mode-name-list))
+    (while (search-forward "\n\n" nil t)
+      (replace-match "\n"))
+    (if (hui-menu-program-path "sort")
+	(call-process-region (point-min) (point-max)
+			     "sort" t t nil "-r" "-k3,3" "-k1,1")
+      ;; This fallback of sort-fields can only sort on one field, so
+      ;; sort by major-mode and leave buffers within each mode
+      ;; unsorted when no UNIX sort program is available.
+      (sort-fields 3 (point-min) (point-max))
+      (reverse-region (point-min) (point-max)))
+    (insert "\)\n")
+    (goto-char (point-min))
+    (insert "\(")
+    (goto-char (point-min))
+    (read (current-buffer))))
 
 (defun hui-menu-of-frames ()
   (let ((frames (copy-sequence (frame-list))))
