@@ -1,10 +1,11 @@
-;;; hui-menu.el --- Menubar menu of GNU Hyperbole commands
+;;; hui-menu.el --- Menubar menu of GNU Hyperbole commands -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    28-Oct-94 at 10:59:44
+;; Last-Mod:     26-Feb-22 at 17:03:44 by Bob Weiner
 ;;
-;; Copyright (C) 1994-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1994-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -63,8 +64,7 @@
 	  ["Toggle-URLs-in-New-Window"
 	   (setq browse-url-new-window-flag (not browse-url-new-window-flag))
 	   :style toggle
-	   :selected browse-url-new-window-flag]
-	  )))
+	   :selected browse-url-new-window-flag])))
 
 ;; List explicit buttons in the current buffer for menu activation.
 (defun hui-menu-explicit-buttons (rest-of-menu)
@@ -89,7 +89,7 @@
 			  "Activate:")
 			(mapcar (lambda (label) (vector label `(ebut:act ,label) t))
 				(if hui-menu-order-explicit-buttons
-				    (sort labels 'string-lessp)
+				    (sort labels #'string-lessp)
 				  labels))
 			(if cutoff '(". . ."))
 			'("----" "----"))))))
@@ -135,9 +135,9 @@ Return t if cutoff, else nil."
     (vector (hui-menu-key-binding-item "Drag-Emulation-Key  \t\t"   'hkey-operate)                       '(hui:bind-key #'hkey-operate) t)                   ;; {M-o}
     (vector (hui-menu-key-binding-item "Find-Web-Key        \t\t"   'hui-search-web)                     '(hui:bind-key #'hui-search-web) t)                 ;; {C-c /}
     (vector (hui-menu-key-binding-item "Grid-of-Windows-Key \t"     'hycontrol-windows-grid)             '(hui:bind-key #'hycontrol-windows-grid) t)         ;; {C-c @}
-    (vector (hui-menu-key-binding-item "Hyperbole-Menu-Key  \t"     'hyperbole)                          '(hui:bind-key #'hyperbole) t)                      ;; {C-h h}
+    (vector (hui-menu-key-binding-item "Hyperbole-Menu-Key  \t"     'hyperbole)                          '(hui:global-bind-key #'hyperbole) t)                      ;; {C-h h}
     (vector (hui-menu-key-binding-item "Jump-Thing-Key      \t\t"   'hui-select-goto-matching-delimiter) '(hui:bind-key #'hui-select-thing) t)               ;; {C-c .}
-    (vector (hui-menu-key-binding-item "Mark-Thing-Key      \t\t"   'hui-select-thing)                   '(hui:bind-key #'hui-select-thing) t)               ;; {C-c C-m}
+    (vector (hui-menu-key-binding-item "Mark-Thing-Key      \t\t"   'hui-select-thing)                   '(hui:bind-key #'hui-select-thing) t)               ;; {C-c RET}
     (vector (hui-menu-key-binding-item "Smart-Help-Key      \t\t"   'hkey-help)                          '(hui:bind-key #'hkey-help) t)                      ;; {C-h A}
     (vector (hui-menu-key-binding-item "Windows-Control-Key\t"      'hycontrol-enable-windows-mode)      '(hui:bind-key #'hycontrol-enable-windows-mode) t)) ;; {C-C \}
    rest-of-menu))
@@ -156,11 +156,20 @@ Return t if cutoff, else nil."
 
 (defun hui-menu-web-search ()
   ;; Pulldown menu
-  (mapcar (lambda (service)
-	    (vector service
-		    (list #'hyperbole-web-search service nil)
-		    t))
-	  (mapcar 'car hyperbole-web-search-alist)))
+  (let* (service
+	 action)
+    (mapcar (lambda (service-and-action)
+	      (setq service (car service-and-action)
+		    action  (cdr service-and-action))
+	      (if (stringp action)
+		  (vector service
+			  (list #'hyperbole-web-search service nil)
+			  t)
+		(vector service
+			;; a command symbol
+			action
+			t)))
+	    hyperbole-web-search-alist)))
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -171,8 +180,7 @@ Return t if cutoff, else nil."
 		  (if (= (aref hyperb:version 0) ?0)
 		      (substring hyperb:version 1)
 		    hyperb:version))
-	  #'(hypb:display-file-with-logo
-	     (expand-file-name "HY-ABOUT" hyperb:dir))
+	  #'(hypb:display-file-with-logo "HY-ABOUT")
 	  t))
 
 (defconst hui-menu-options
@@ -212,12 +220,28 @@ Return t if cutoff, else nil."
 				 `(setq hpath:display-where ',sym)
 				 :style 'radio
 				 :selected `(eq hpath:display-where ',sym)))
-			      (mapcar 'car hpath:display-where-alist))))
+			      (mapcar #'car hpath:display-where-alist))))
 	  '("----")
 	  (hui-menu-browser "Display-URLs-in" browse-url-browser-function)
 	  '("----")
 	  (hui-menu-browser "Display-Web-Searches-in" hyperbole-web-search-browser-function)
 	  '("----")
+	  '(("Org-M-RETURN"
+	     "----"
+	     "----"
+	     ["All-Programmed-Contexts"
+	      (customize-save-variable 'hsys-org-enable-smart-keys t)
+	      :style radio :selected (when (boundp 'hsys-org-enable-smart-keys)
+					(eq hsys-org-enable-smart-keys t))]
+	     ["Hypb-Buttons-Only"
+	      (customize-save-variable 'hsys-org-enable-smart-keys 'buttons)
+	      :style radio :selected (when (boundp 'hsys-org-enable-smart-keys)
+					(eq hsys-org-enable-smart-keys 'buttons))]
+	     ["Ignore"
+	      (customize-save-variable 'hsys-org-enable-smart-keys nil)
+	      :style radio :selected (when (boundp 'hsys-org-enable-smart-keys)
+					(eq hsys-org-enable-smart-keys nil))]))
+	  '("----")
 	  '(("Smart-Key-Press-at-Eol"
 	     "----"
 	     "----"
@@ -226,29 +250,25 @@ Return t if cutoff, else nil."
 	     ;; with a conditional.
 	     ["Scrolls-a-Windowful"
 	      (setq smart-scroll-proportional nil)
-	      :style radio :selected (if (boundp 'smart-scroll-proportional)
-					 (null smart-scroll-proportional))]
+	      :style radio :selected (when (boundp 'smart-scroll-proportional)
+				       (null smart-scroll-proportional))]
 	     ["Scrolls-Proportionally"
 	      (setq smart-scroll-proportional t)
-	      :style radio :selected (if (boundp 'smart-scroll-proportional)
-					 smart-scroll-proportional)]
-	     ))
+	      :style radio :selected (when (boundp 'smart-scroll-proportional)
+				       smart-scroll-proportional)]))
 	  '("----"
 	    ["Toggle-Isearch-Invisible-Text" hypb:toggle-isearch-invisible
 	     :visible (boundp 'isearch-invisible)
-	     :style toggle :selected (and (boundp 'isearch-invisible)
-					  isearch-invisible)]
+	     :style toggle :selected (when (boundp 'isearch-invisible)
+				       isearch-invisible)]
 	    ["Toggle-Messaging-Explicit-Buttons" hyperbole-toggle-messaging
 	     :style toggle :selected (not inhibit-hyperbole-messaging)]
-	    ["Toggle-Override-Local-Keys" hkey-toggle-override-local-bindings
-	     :style toggle :selected hkey-init-override-local-keys]
 	    ["Toggle-Rolo-Dates" hyrolo-toggle-datestamps
 	     :style toggle :selected (and (boundp 'hyrolo-add-hook)
 					  (listp hyrolo-add-hook)
 					  (memq 'hyrolo-set-date hyrolo-add-hook))]
 	    ["Toggle-Smart-Key-Debug (HyDebug)" hkey-toggle-debug
-	     :style toggle :selected hkey-debug]
-	    ))
+	     :style toggle :selected hkey-debug]))
   "Untitled menu of Hyperbole options.")
 
 (defvar infodock-hyperbole-menu nil)
@@ -277,7 +297,7 @@ Return t if cutoff, else nil."
 				    "Koutline")
 				   ((global-key-binding [menu-bar OO-Browser])
 				    "OO-Browser"))))
-	     (add-submenu nil (infodock-hyperbole-menu t) add-before))))
+             (easy-menu-add-item (current-global-map) '("menu-bar") (infodock-hyperbole-menu t) add-before))))
   ;; Force a menu-bar update.
   (force-mode-line-update))
 
@@ -299,16 +319,14 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		 "Hyperbole"
 		 :config 'Hyperbole
 		 hui-menu-about
-		 ["Demonstration"  (hypb:display-file-with-logo
-				    (expand-file-name "DEMO" hyperb:dir)) t]
+		 ["Demonstration"  hyperbole-demo t]
 		 ;; Comment InfoDock manual reference out until
 		 ;; InfoDock is modernized for Emacs 25.
 		 ;; (if (and (boundp 'infodock-version) infodock-version)
 		 ;;     ["Manual"      (id-info "(infodock)Hyperbole Menu") t]
 		 ;;   ["Manual"      (id-info "(hyperbole)Top") t])
 		 ["Manual"         (id-info "(hyperbole)Top") t]
-		 ["What-is-New?"   (hypb:display-file-with-logo
-				    (expand-file-name "HY-NEWS" hyperb:dir)) t]
+		 ["What-is-New?"   (hypb:display-file-with-logo "HY-NEWS") t]
 		 ["Why-Use?"       (find-file
 				    (expand-file-name "HY-WHY.kotl" hyperb:dir)) t]
 		 "----"
@@ -318,7 +336,7 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		    (hui-menu-remove Hyperbole)
 		    ;;
 		    ;; Remove Hyperbole button comment from future outgoing mail.
-		    (if (boundp 'smail:comment) (setq smail:comment nil)))
+		    (when (boundp 'smail:comment) (setq smail:comment nil)))
 		  t]
 		 "----"
 		 ["Activate-Button-in-Buffer" hui:hbut-act t]
@@ -330,17 +348,14 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		   ["Edit-Per-Directory-File" (find-file hbmap:filename) t]
 		   ["Edit-Personal-File" (find-file
 					  (expand-file-name
-					   hbmap:filename hbmap:dir-user)) t]
-		   )
+					   hbmap:filename hbmap:dir-user)) t])
 		 (cons "Customize" hui-menu-options)
 		 '("Documentation"
 		   ["Manual"      (id-info "(hyperbole)Top") t]
 		   "----"
-		   ["Demonstration"  (hypb:display-file-with-logo
-				      (expand-file-name "DEMO" hyperb:dir)) t]
+		   ["Demonstration"  hyperbole-demo t]
 		   ["Glossary"    (id-info "(hyperbole)Glossary") t]
-		   ["Manifest"    (hypb:display-file-with-logo
-				   (expand-file-name "MANIFEST" hyperb:dir)) t]
+		   ["Manifest"    (hypb:display-file-with-logo "MANIFEST") t]
 		   ["Smart-Key-Summary" (id-browse-file (hypb:hkey-help-file)) t]
 		   ("Types"
 		    ["Action-Types-Manual"
@@ -351,27 +366,23 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		    ["Show-Action-Types"
 		     (hui:htype-help-current-window 'actypes) t]
 		    ["Show-Implicit-Button-Types"
-		     (hui:htype-help-current-window 'ibtypes 'no-sort) t]
-		    ))
+		     (hui:htype-help-current-window 'ibtypes 'no-sort) t]))
 		 '("Explicit-Button"
 		   :filter hui-menu-explicit-buttons
 		   ["Activate" hui:ebut-act t]
 		   ["Create" hui:ebut-create t]
 		   ["Delete" hui:ebut-delete t]
-		   ["Edit"   hui:ebut-modify t]
+		   ["Edit"   hui:ebut-edit t]
 		   ("Help"
 		    ["Manual"   (id-info "(hyperbole)Location") t]
 		    "----"
 		    ["Buffer-Buttons"   (hui:hbut-report -1) t]
 		    ["Current-Button"   (hui:hbut-report)    t]
-		    ["Ordered-Buttons"  (hui:hbut-report 1)  t]
-		    )
-		   ["Modify" hui:ebut-modify t]
+		    ["Ordered-Buttons"  (hui:hbut-report 1)  t])
 		   ["Rename" hui:ebut-rename t]
 		   ["Search" hui:ebut-search t]
 		   ["Types"
-		    (hui:htype-help-current-window 'actypes) t]
-		   )
+		    (hui:htype-help-current-window 'actypes) t])
 		 (append
 		  '("Find"
 		    ["Manual"   (id-info-item "menu, Find") t]
@@ -390,26 +401,24 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 		    ["Save-Lines-Here"      hypb:save-lines t]
 		    "----"
 		    "Web-Search:")
-		  (hui-menu-web-search)
-		  )
+		  (hui-menu-web-search))
 		 '("Global-Button"
 		   :filter hui-menu-global-buttons
 		   ["Create" hui:gbut-create t]
-		   ["Edit"   hui:gbut-modify t]
+		   ["Delete" hui:gbut-delete t]
+		   ["Edit"   hui:gbut-edit t]
 		   ["Help"   gbut:help t]
-		   ["Modify" hui:gbut-modify t]
-                   ["Rename" hui:gbut-rename t]
-		   )
+                   ["Rename" hui:gbut-rename t])
 		 '("Implicit-Button"
 		   ["Manual"   (id-info "(hyperbole)Implicit Buttons") t]
 		   "----"
 		   ["Activate" hui:ibut-act t]
 		   ["Delete-Type" (hui:htype-delete 'ibtypes) t]
+		   ["Edit"   hui:ibut-edit t]
 		   ["Help"   hui:hbut-help t]
 		   ["Label"  hui:ibut-label-create t]
 		   ["Rename" hui:ibut-rename t]
-		   ["Types"  (hui:htype-help 'ibtypes 'no-sort) t]
-		   )
+		   ["Types"  (hui:htype-help 'ibtypes 'no-sort) t])
 		 '("Koutliner"
 		   ["Manual" (id-info "(hyperbole)Koutliner") t]
 		   ["Example"   kotl-mode:example t]
@@ -430,8 +439,7 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 					  (kcell-view:label)))
 		    (eq major-mode 'kotl-mode)]
 		   ["Show-Top-Level-Only" kotl-mode:hide-body
-		    (eq major-mode 'kotl-mode)]
-		   )
+		    (eq major-mode 'kotl-mode)])
 		 '("Mail-Lists"
 		   ["Manual" (id-info "(hyperbole)Suggestion or Bug Reporting")
 		    t]
@@ -453,8 +461,7 @@ REBUILD-FLAG is non-nil, in which case the menu is rebuilt."
 				   "Just send the message; subject and body are ignored.") t]
 		   ["Leave-Hyperbole-Bug-Report-List"
 		    (hmail:compose "bug-hyperbole-leave@gnu.org" nil
-				   "Just send the message; subject and body are ignored.") t]
-		   )
+				   "Just send the message; subject and body are ignored.") t])
 		 infodock-hyrolo-menu
 		 '("Screen (HyControl)" :filter hui-menu-screen)
 		 hui-menu-hywconfig)))))
