@@ -1,10 +1,11 @@
-;;; hui-mouse.el --- Use key or mouse key for many functions, e.g. GNU Hyperbole menus
+;;; hui-mouse.el --- Use key or mouse key for many functions, e.g. GNU Hyperbole menus  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    04-Feb-89
+;; Last-Mod:     17-Apr-22 at 13:46:49 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2018  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -37,11 +38,15 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
+(eval-when-compile (require 'hsys-org))
 (require 'hbut)
 (unless (fboundp 'smart-info)
   (require 'hmouse-info))
 (unless (fboundp 'smart-c-at-tag-p)
   (require 'hmouse-tag))
+(require 'hsys-org)
+
+(eval-when-compile (require 'tar-mode))
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -51,9 +56,18 @@
   "*Command that sets point to the mouse cursor position.")
 
 (defun action-key-error ()
-  (hypb:error "(Hyperbole Action Key): No action defined for this context; try another location"))
+  "If in Org mode and Hyperbole shares {M-RET}, run org-meta-return, else signal an error."
+  (if (and (funcall hsys-org-mode-function)
+	   (hsys-org-meta-return-shared-p))
+      (hact 'org-meta-return current-prefix-arg)
+    (hypb:error "(Hyperbole Action Key): No action defined for this context; try another location")))
+
 (defun assist-key-error ()
-  (hypb:error "(Hyperbole Assist Key): No action defined for this context; try another location"))
+  "If in Org mode and Hyperbole shares {M-RET}, run org-meta-return, else signal an error."
+  (if (and (funcall hsys-org-mode-function)
+	   (hsys-org-meta-return-shared-p))
+      (hact 'org-meta-return current-prefix-arg)
+    (hypb:error "(Hyperbole Assist Key): No action defined for this context; try another location")))
 
 (defcustom action-key-default-function #'action-key-error
   "*Function run by the Action Key in an unspecified context.
@@ -76,16 +90,78 @@ to use the Treemacs file manager package instead."
 
 (defcustom action-key-eol-function #'smart-scroll-up
   "*Function run by the Action Key at the end of a line.
-Its default value is #'smart-scroll-up."
+Its default value is #'smart-scroll-up.  To disable it, set it to
+#'ignore."
   :type 'function
   :group 'hyperbole-keys)
 
 (defcustom assist-key-eol-function #'smart-scroll-down
   "*Function run by the Assist Key at the end of a line.
-Its default value is #'smart-scroll-down."
+Its default value is #'smart-scroll-down.  To disable it, set it to
+#'ignore."
   :type 'function
   :group 'hyperbole-keys)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function todotxt-archive "ext:todotxt")
+(declare-function todotxt-bury "ext:todotxt")
+(declare-function todotxt-complete-toggle "ext:todotxt")
+(declare-function todotxt-edit-item "ext:todotxt")
+
+(declare-function magit-current-section "magit-selection")
+(declare-function magit-diff-visit-file "etx:magit-diff")
+(declare-function magit-diff-visit-file--internal "etx:magit-diff")
+(declare-function magit-file-at-point "etx:magit-git")
+(declare-function magit-section-cycle-diffs "etx:magit-diff")
+(declare-function magit-section-cycle-global "etx:magit-selection")
+(declare-function magit-section-hide "etx:magit-selection")
+(declare-function magit-section-show "etx:magit-selection")
+
+(declare-function -flatten "ext:dash")
+
+(declare-function imenu--make-index-alist "imenu")
+
+(declare-function image-dired-thumbnail-display-external "image-dired")
+(declare-function image-dired-display-thumbnail-original-image "image-dired")
+(declare-function image-dired-mouse-select-thumbnail "image-dired")
+
+(declare-function helm-action-window "ext:helm-lib")
+(declare-function helm-buffer-get "ext:helm-lib")
+;; (declare-function helm-get-current-action "ext:helm-?")
+(declare-function helm-get-selection "ext:helm")
+(declare-function helm-mark-current-line "ext:helm")
+(declare-function helm-next-line "ext:helm")
+(declare-function helm-pos-candidate-separator-p "ext:helm")
+(declare-function helm-pos-header-line-p "ext:helm")
+(declare-function helm-resume "ext:helm")
+(declare-function helm-window "ext:helm-lib")
+(declare-function with-helm-buffer "ext:helm-lib")
+(defvar helm-action-buffer)
+(defvar helm-alive-p)
+(defvar helm-buffer)
+
+(declare-function ibuffer-mark-for-delete "ibuffer")
+(declare-function ibuffer-unmark-forward "ibuffer")
+(declare-function ibuffer-unmark-all "ibuffer")
+(declare-function ibuffer-do-view "ibuffer")
+(declare-function ibuffer-mark-forward "ibuffer")
+(declare-function ibuffer-do-kill-on-deletion-marks "ibuffer")
+(declare-function ibuffer-get-marked-buffers "ibuffer")
+(declare-function ibuffer-current-buffer "ibuffer")
+
+(declare-function br-buffer-menu-select "ext:br")
+
+(declare-function gnus-topic-read-group "gnus-topic")
+
+(declare-function company-show-doc-buffer "ext:company")
+;; (declare-function company-quick-help-manual-begin "ext:company?")
+(declare-function company-show-location "ext:company")
+(declare-function company-select-mouse "ext:company")
+
+
 ;;; ************************************************************************
 ;;; Hyperbole context-sensitive keys dispatch table
 ;;; ************************************************************************
@@ -99,11 +175,23 @@ Its default value is #'smart-scroll-down."
     ((and (boundp 'company-active-map)
 	  (memq company-active-map (current-minor-mode-maps))) .
 	  ((smart-company-to-definition) . (smart-company-help)))
+    ;; Handle any Org mode-specific contexts
+    ((and (not (hyperb:stack-frame '(smart-org)))
+	  (let ((hrule:action #'actype:identity))
+	    (smart-org current-prefix-arg))) .
+     ((smart-org current-prefix-arg) . (smart-org current-prefix-arg)))
+    ;; Ivy minibuffer completion mode
+    ((and (boundp 'ivy-mode) ivy-mode (minibuffer-window-active-p (selected-window))) .
+     ((ivy-done) . (ivy-dispatching-done)))
     ;;
     ;; Treemacs hierarchical file manager
     ((eq major-mode 'treemacs-mode) .
      ((smart-treemacs) . (smart-treemacs)))
     ;;
+    ;; dired-sidebar-mode
+    ((eq major-mode 'dired-sidebar-mode) .
+     ((smart-dired-sidebar) . (smart-dired-sidebar)))
+    ;;
     ;; Handle Emacs push buttons in buffers
     ((and (fboundp 'button-at) (button-at (point))) .
      ((push-button nil (mouse-event-p last-command-event))
@@ -113,7 +201,7 @@ Its default value is #'smart-scroll-down."
     ;; accept argument or give completion help.
     ((and (> (minibuffer-depth) 0)
 	  (eq (selected-window) (minibuffer-window))
-	  (not (eq hargs:reading-p 'hmenu))
+	  (not (eq hargs:reading-type 'hmenu))
 	  (not (smart-helm-alive-p))) .
 	  ((funcall (key-binding (kbd "RET"))) . (smart-completion-help)))
     ;;
@@ -128,7 +216,7 @@ Its default value is #'smart-scroll-down."
     ;; session and activate the selected item.
     ((and (> (minibuffer-depth) 0)
 	  (eq (selected-window) (minibuffer-window))
-	  (or (eq hargs:reading-p 'hmenu)
+	  (or (eq hargs:reading-type 'hmenu)
 	      (smart-helm-alive-p))) .
 	  ((funcall (key-binding (kbd "RET"))) . (funcall (key-binding (kbd "RET")))))
     ;;
@@ -145,14 +233,15 @@ Its default value is #'smart-scroll-down."
     ;;
     ;; If at the end of a line (eol), invoke the associated Smart Key handler EOL handler.
     ((if (eq major-mode 'kotl-mode)
-	 (and (not (kotl-mode:eobp)) (kotl-mode:eolp))
+	 (and (not (kotl-mode:eobp)) (kotl-mode:eolp t))
        (smart-eolp)) .
        ((funcall action-key-eol-function) . (funcall assist-key-eol-function)))
     ;;
     ;; The Smart Menu system is an attractive in-buffer menu system
     ;; that works on any display system that supports Emacs.  It
     ;; predates Emacs' menu systems; it is a part of InfoDock.
-    ;; It is not included with Hyperbole.
+    ;; It is not included with Hyperbole but is compatible with the
+    ;; Smart Keys.
     ;;
     ;; This selects or gives help for a menu item.
     ((eq major-mode 'smart-menu-mode) .
@@ -161,6 +250,9 @@ Its default value is #'smart-scroll-down."
     ((derived-mode-p 'dired-mode) .
      ((smart-dired) . (smart-dired-assist)))
     ;;
+    ((string-prefix-p "magit-" (symbol-name major-mode)) .
+     ((smart-magit) . (smart-magit-assist)))
+    ;;
     ;; If on a Hyperbole button, perform action or give help.
     ((hbut:at-p) .
      ((hui:hbut-act 'hbut:current) . (hui:hbut-help 'hbut:current)))
@@ -224,6 +316,14 @@ Its default value is #'smart-scroll-down."
      ((hui-select-mark-delimited-sexp) .
       (progn (hui-select-mark-delimited-sexp) (hmouse-kill-region))))
     ;;
+    ((eq major-mode 'occur-mode) .
+     ((occur-mode-goto-occurrence) . (occur-mode-goto-occurrence)))
+    ;;
+    ((eq major-mode 'moccur-mode) .
+     ((moccur-mode-goto-occurrence) . (moccur-mode-goto-occurrence)))
+    ((eq major-mode 'amoccur-mode) .
+     ((amoccur-mode-goto-occurrence) . (amoccur-mode-goto-occurrence)))
+    ;;
     ((eq major-mode 'kotl-mode) .
      ((kotl-mode:action-key) . (kotl-mode:assist-key)))
     ;;
@@ -237,6 +337,10 @@ Its default value is #'smart-scroll-down."
 	 (string-match "^\\*Help\\|Help\\*$" (buffer-name))) .
 	 ((hkey-help-hide) . (hkey-help-hide)))
     ;;
+    ;; Handle widgets in Custom-mode
+    ((eq major-mode 'Custom-mode) .
+     ((smart-custom) . (smart-custom-assist)))
+    ;;
     ;; Emacs bookmarks menu (bookmark.el)
     ((eq major-mode 'bookmark-bmenu-mode) .
      ((bookmark-jump (bookmark-bmenu-bookmark) (hpath:display-buffer-function)) .
@@ -260,10 +364,6 @@ Its default value is #'smart-scroll-down."
      . ((smart-imenu-display-item-where (car hkey-value) (cdr hkey-value)) .
 	(imenu-choose-buffer-index)))
     ;;
-    ;; Function menu listing mode in XEmacs
-    ((eq major-mode 'fume-list-mode) .
-     ((fume-list-mouse-select current-mouse-event) . (fume-prompt-function-goto)))
-    ;;
     ((and (eq major-mode 'c-mode)
 	  buffer-file-name (smart-c-at-tag-p)) .
 	  ((smart-c) . (smart-c nil 'next-tag)))
@@ -306,14 +406,6 @@ Its default value is #'smart-scroll-down."
 	  buffer-file-name (smart-fortran-at-tag-p)) .
 	  ((smart-fortran) . (smart-fortran nil 'next-tag)))
     ;;
-    ((eq major-mode 'occur-mode) .
-     ((occur-mode-goto-occurrence) . (occur-mode-goto-occurrence)))
-    ;;
-    ((eq major-mode 'moccur-mode) .
-     ((moccur-mode-goto-occurrence) . (moccur-mode-goto-occurrence)))
-    ((eq major-mode 'amoccur-mode) .
-     ((amoccur-mode-goto-occurrence) . (amoccur-mode-goto-occurrence)))
-    ;;
     ((eq major-mode 'calendar-mode) .
      ((smart-calendar) . (smart-calendar-assist)))
     ;;
@@ -375,9 +467,12 @@ Its default value is #'smart-scroll-down."
     ;; Outline minor mode is on and usable.
     ((and (boundp 'outline-minor-mode) outline-minor-mode) .
      ((smart-outline) . (smart-outline-assist)))
-    )
+    ;;
+    ;; Todotxt
+    ((eq major-mode 'todotxt-mode) .
+     ((smart-todotxt) . (smart-todotxt-assist))))
   "Alist of predicates and form-conses for the Action and Assist Keyboard Keys.
-Each element is: (predicate-form . (action-key-form . assist-key-form)).
+Each element is: (PREDICATE-FORM . (ACTION-KEY-FORM . ASSIST-KEY-FORM)).
 When the Action or Assist Key is pressed, the first or second form,
 respectively, associated with the first non-nil predicate is evaluated.
 
@@ -586,6 +681,55 @@ If assist-key is pressed:
 	 (ibuffer-unmark-forward nil nil 1))
 	(t (ibuffer-mark-for-delete nil nil 1))))
 
+;;; ************************************************************************
+;;; smart-custom and widget functions
+;;; ************************************************************************
+
+(defun smart-custom ()
+  "Use a single key or mouse key to manipulate customizable settings.
+
+Invoked via a key press when in Custom-mode.  It assumes that
+its caller has already checked that the key was pressed in an
+appropriate buffer and has moved the cursor there.
+
+If key is pressed:
+ (1) on the last line of the buffer, exit Custom mode, potentially
+     prompting to save any changes;
+ (2) at the end of any other line, scroll the window down down a windowful;
+ (3) if a mouse event on a widget, activate the widget or display a menu;
+ (4) anywhere else, execute the command bound to {RETURN}."
+  (interactive)
+  (cond ((last-line-p) (Custom-buffer-done))
+	((eolp) (smart-scroll-up))
+	((mouse-event-p last-command-event)
+	 (widget-button-click action-key-release-args))
+	 ;; Handle widgets in Custom-mode
+	(t (Custom-newline (point)))))
+
+(defun smart-custom-assist ()
+  "Use an assist-key or mouse assist-kkey to manipulate customizable settings.
+
+Invoked via a key press when in Custom-mode.  It assumes that
+its caller has already checked that the key was pressed in an
+appropriate buffer and has moved the cursor there.
+
+If key is pressed:
+ (1) on the last line of the buffer, exit Custom mode, potentially
+     prompting to save any changes;
+ (2) at the end of any other line, scroll the window down down a windowful;
+ (3) if a mouse event on a widget, activate the widget or display a menu;
+ (4) anywhere else, execute the command bound to {RETURN}."
+  (interactive)
+  (cond ((last-line-p) (Custom-buffer-done))
+	((eolp) (smart-scroll-down))
+	((mouse-event-p last-command-event)
+	 (widget-button-click action-key-release-args))
+	 ;; Handle widgets in Custom-mode
+	(t (Custom-newline (point)))))
+
+
+
+
 ;;; ************************************************************************
 ;;; smart-calendar functions
 ;;; ************************************************************************
@@ -605,9 +749,9 @@ If key is pressed:
 
   (interactive)
   (cond ((eobp) (calendar-cursor-to-nearest-date)
-	 (scroll-calendar-left-three-months 1))
+	 (calendar-scroll-left-three-months 1))
 	((< (current-column) 5) (calendar-cursor-to-nearest-date)
-	 (scroll-calendar-right-three-months 1))
+	 (calendar-scroll-right-three-months 1))
 	(t (calendar-cursor-to-nearest-date)
 	   (diary-view-entries 1))))
 
@@ -627,10 +771,10 @@ If assist-key is pressed:
 
   (interactive)
   (cond ((eobp) (calendar-cursor-to-nearest-date)
-	 (scroll-calendar-right-three-months 1))
+	 (calendar-scroll-right-three-months 1))
 	((< (current-column) 5) (calendar-cursor-to-nearest-date)
-	 (scroll-calendar-left-three-months 1))
-	(t (mark-diary-entries))))
+	 (calendar-scroll-left-three-months 1))
+	(t (diary-mark-entries))))
 
 ;;; ************************************************************************
 ;;; smart-company mode functions
@@ -749,9 +893,7 @@ If assist-key is pressed:
 	 ;; Prevent any region selection from causing multiple files
 	 ;; to be marked for deletion; we want to mark only one.
 	 (deactivate-mark t)
-	 (if (fboundp 'dired-flag-file-deletion)
-	     (dired-flag-file-deletion 1)
-	   (dired-flag-file-deleted 1)))))
+	 (dired-flag-file-deletion 1))))
 
 ;;; ************************************************************************
 ;;; smart-gnus functions
@@ -870,8 +1012,7 @@ If key is pressed within:
 	     (progn (set-buffer gnus-summary-buffer)
 		    (setq this-command 'gnus-summary-next-unread-article)
 		    (gnus-summary-next-unread-article)
-		    (gnus-summary-goto-subject gnus-current-article)
-		    )
+		    (gnus-summary-goto-subject gnus-current-article))
 	   (let ((artic (get-buffer-window gnus-article-buffer)))
 	     (if artic (select-window artic)))))
 	((and (not (eolp)) (Info-handle-in-note)))
@@ -894,8 +1035,7 @@ If assist-key is pressed within:
 	     (progn (set-buffer gnus-summary-buffer)
 		    (setq this-command 'gnus-summary-prev-article)
 		    (gnus-summary-prev-article nil)
-		    (gnus-summary-goto-subject gnus-current-article)
-		    )
+		    (gnus-summary-goto-subject gnus-current-article))
 	   (let ((artic (get-buffer-window gnus-summary-buffer)))
 	     (if artic (select-window artic)))))
 	((and (not (eolp)) (Info-handle-in-note)))
@@ -914,7 +1054,7 @@ If assist-key is pressed within:
 
 (defun smart-helm-line-has-action ()
   "Mark and return the actions for the helm selection item at the point of Action Key depress, or nil if line lacks any action.
-Assumes Hyperbole has already checked that helm is active."
+Assume Hyperbole has already checked that helm is active."
   (let ((helm-buffer (if (equal helm-action-buffer (buffer-name)) helm-buffer (buffer-name))))
     (save-excursion
       (with-helm-buffer
@@ -949,7 +1089,7 @@ Assumes Hyperbole has already checked that helm is active."
   "Return non-nil iff Smart Mouse DEPRESS-EVENT was on a helm section header, candidate separator or at eob or eol.
 If non-nil, returns a property list of the form: (section-header <bool> separator <bool> eob <bool> or eol <bool>).
 If a section-header or separator, selects the first following candidate line.
-Assumes Hyperbole has already checked that helm is active."
+Assume Hyperbole has already checked that helm is active."
   (and (eventp depress-event)
        ;; Nil means in the buffer text area
        (not (posn-area (event-start depress-event)))
@@ -1163,7 +1303,7 @@ buffer and has moved the cursor to the selected buffer."
   (interactive)
   (hyrolo-edit-entry))
 
-(defalias 'smart-hyrolo-assist 'smart-hyrolo)
+(defalias 'smart-hyrolo-assist #'smart-hyrolo)
 
 ;;; ************************************************************************
 ;;; smart-image-dired functions
@@ -1184,7 +1324,7 @@ buffer and has moved the cursor to the selected buffer."
 ;;; ************************************************************************
 
 (defun smart-imenu-display-item-where (item-name item-pos)
-  "Displays an ITEM-NAME defined at ITEM-POS within the current buffer.
+  "Display an ITEM-NAME defined at ITEM-POS within the current buffer.
 Uses the imenu library and the value of `hpath:display-where' to display."
   (hpath:display-buffer (current-buffer))
   (funcall imenu-default-goto-function item-name item-pos))
@@ -1246,6 +1386,99 @@ NO-RECURSE-FLAG non-nil prevents infinite recursions."
 ;;; ************************************************************************
 ;;; In "hmouse-info.el".
 
+
+;;; ************************************************************************
+;;; smart-magit functions
+;;; ************************************************************************
+
+(defun smart-magit-display-file (return-command)
+  "Execute Magit command bound to return, possibly displaying utilizing hpath:display-buffer setting."
+  (cond ((eq return-command #'magit-diff-visit-file)
+	 ;; Use Hyperbole display variable to determine where
+	 ;; to display the file of the diff.
+	 (magit-diff-visit-file--internal
+	  (magit-file-at-point t t) nil #'hpath:display-buffer))
+	(t (call-interactively return-command))))
+
+(defun smart-magit ()
+  "Use a key or mouse key to jump to source and to hide/show changes.
+
+Invoked via a key press when in a magit mode and not on a button.
+It assumes that its caller has already checked that the key was
+pressed in an appropriate buffer and has moved the cursor to the
+selected buffer.
+
+If key is pressed:
+ (1) on the last line, quit from the magit mode (\"q\" key binding);
+ (2) at the end of a line, scroll up a windowful;
+ (3) on an initial read-only header line, cycle visibility of diff sections;
+ (4) anywhere else, hide/show the thing at point (\"TAB\" key binding)
+     unless that does nothing in the mode, then jump to the thing at
+     point (\"RET\" key binding) but display based on the value of
+     `hpath:display-where'."
+  (interactive)
+  (cond ((last-line-p)
+	 (call-interactively (key-binding "q")))
+	((eolp)
+	 (smart-scroll-up))
+	(t
+	 (let ((magit-display-buffer-function #'hpath:display-buffer))
+	   (call-interactively #'smart-magit-tab)))))
+
+(defun smart-magit-assist ()
+  "Use an assist key or mouse key to jump to source and to hide/show changes.
+
+Invoked via an assist key press when in a magit mode and not on a
+button.  It assumes that its caller has already checked that the
+assist-key was pressed in an appropriate buffer and has moved the
+cursor to the selected buffer.
+
+If assist-key is pressed:
+ (1) on the last line, quit from the magit mode (\"q\" key binding);
+ (2) at the end of a line, scroll down a windowful;
+ (3) on an initial read-only header line, cycle visibility of all sections;
+ (4) anywhere else, jump to the thing at point (\"RET\" key binding)
+     but display based on the value of `hpath:display-where'."
+  (interactive)
+  (cond ((last-line-p)
+	 (call-interactively (key-binding "q")))
+	((eolp)
+	 (smart-scroll-down))
+	(t
+	 (let ((magit-display-buffer-function #'hpath:display-buffer)
+	       (current-prefix-arg)
+	       (non-text-area-p (and (eventp action-key-depress-args)
+				     (posn-area (event-start action-key-depress-args)))))
+	   (if non-text-area-p
+	       (magit-section-cycle-global)
+	     (smart-magit-display-file (key-binding (kbd "RET"))))))))
+
+;; Thanks to Jonas Bernoulli <tarsius>, magit author, for most of this
+;; next function.
+;; Usage: (define-key magit-section-mode-map "TAB" 'smart-magit-tab)
+(defun smart-magit-tab (section)
+  "Toggle visibility of the body of the current section."
+  (interactive (list (magit-current-section)))
+  (require 'magit)
+  (let* ((magit-display-buffer-function #'hpath:display-buffer)
+	 (non-text-area-p (and (eventp action-key-depress-args)
+			       (posn-area (event-start action-key-depress-args)))))
+    (cond (non-text-area-p (magit-section-cycle-diffs))
+	  ((eq section magit-root-section)
+	   (user-error "Cannot hide root section"))
+          ((or (oref section content)
+	       (oref section washer))
+	   (goto-char (oref section start))
+	   (if (oref section hidden)
+	       (magit-section-show section)
+	     (magit-section-hide section)))
+          (t
+	   (if-let ((command (key-binding (kbd "RET"))))
+	       (progn (setq last-command-event ?\()
+		      (setq this-command command)
+		      (smart-magit-display-file command))
+	     (user-error "Nothing to visit either"))))))
+
 ;;; ************************************************************************
 ;;; smart-man functions
 ;;; ************************************************************************
@@ -1334,8 +1567,7 @@ local variable containing its pathname."
 			(if ref
 			    (if (string-match "(\\(.\\)\\(.+\\))" ref)
 				(setq ref (concat (substring ref 0 (match-end 1))
-						  "\)"))))
-			)))))))
+						  "\)")))))))))))
     (cond ((equal ref "") nil)
 	  ((stringp ref) (list 'manual-entry ref))
 	  (t ref))))
@@ -1356,8 +1588,7 @@ locate the definition."
 		  (if (or (looking-at "\\([_~<>:a-zA-Z0-9]+\\)[ \t\n\r]*(")
 			  (looking-at "\\([_~<:A-Z][_<>:A-Z0-9]+\\)"))
 		      (setq ref (buffer-substring
-				 (match-beginning 1) (match-end 1))
-			    )))))
+				 (match-beginning 1) (match-end 1)))))))
     (if ref
 	(list 'smart-tags-display ref nil
 	      (smart-tags-file-list (and (boundp 'man-path) man-path))))))
@@ -1375,13 +1606,119 @@ If not on a file name, returns nil."
 		    (skip-chars-backward "^ \t")
 		    (if (looking-at "/[^ \t\n\r]+")
 			(setq ref (buffer-substring
-				   (match-beginning 0) (match-end 0))
-			      )))))
+				   (match-beginning 0) (match-end 0)))))))
     (if ref
 	(list (if (br-in-browser)
 		  'find-file 'find-file-other-window)
 	      ref))))
 
+;;; ************************************************************************
+;;; smart-org functions
+;;; ************************************************************************
+
+(defun smart-org (&optional _assist-flag)
+  "If `hsys-org-enable-smart-keys' is non-nil, follow Org mode references, cycles outline visibility and executes code blocks.
+
+When the Action Key is pressed:
+
+  First, this follows internal links in Org mode files.  When pressed on a
+  link referent/target, the link definition is displayed, allowing two-way
+  navigation between definitions and targets.
+
+  Second, this follows Org mode external links.
+
+  Third, within a radio target definition, this jumps to the first
+  occurrence of an associated radio target.
+
+  Fourth, when point is on an outline heading in Org mode, this
+  cycles the view of the subtree at point.
+
+  Fifth, with point on the first line of a code block definition, this
+  executes the code block via the Org mode standard binding of {C-c C-c},
+  (org-ctrl-c-ctrl-c).
+
+  In any other context besides the end of a line, the Action Key invokes the
+  Org mode standard binding of {M-RET}, (org-meta-return).
+
+When the Assist Key is pressed:
+
+  First, on an Org mode heading, this cycles through views of the whole buffer outline.
+
+  Second, on an Org mode link or agenda item, this displays standard Hyperbole help.
+
+To disable ALL Hyperbole support within Org major and minor modes, set the
+custom option `hsys-org-enable-smart-keys' to nil.  Then in Org modes, this
+will simply invoke `org-meta-return'.
+
+Org links may be used outside of Org mode buffers.  Such links are
+handled by the separate implicit button type, `org-link-outside-org-mode'."
+  (when (funcall hsys-org-mode-function)
+    (let (start-end)
+      (cond ((not hsys-org-enable-smart-keys)
+	     (when (hsys-org-meta-return-shared-p)
+	       (hact 'org-meta-return current-prefix-arg))
+	     ;; Ignore any further Smart Key non-Org contexts
+	     t)
+	    ((eq hsys-org-enable-smart-keys t)
+	     (cond ((hsys-org-agenda-item-at-p)
+		    (if (not assist-flag)
+			(progn (hsys-org-set-ibut-label (cons (line-beginning-position) (line-end-position)))
+			       (hact 'org-agenda-show-and-scroll-up current-prefix-arg))
+		      (hact 'hkey-help))
+		    ;; Ignore any further Smart Key non-Org contexts
+		    t)
+		   ((setq start-end (hsys-org-internal-link-target-at-p))
+		    (hsys-org-set-ibut-label start-end)
+		    (hact 'org-internal-link-target)
+		    t)
+		   ((hsys-org-radio-target-def-at-p)
+		    (hact 'org-radio-target)
+		    t)
+		   ((setq start-end (hsys-org-link-at-p))
+		    (if (not assist-flag)
+			(progn (hsys-org-set-ibut-label start-end)
+			       (hact 'org-link))
+		      (hact 'hkey-help))
+		    t)
+		   ((hsys-org-block-start-at-p)
+		    (hact 'org-ctrl-c-ctrl-c)
+		    t)
+		   ((hbut:at-p)
+		    ;; Fall through until Hyperbole button context and
+		    ;; activate normally.
+		    nil)
+		   ((hsys-org-heading-at-p)
+		    (if (not assist-flag)
+			(hact 'hsys-org-cycle)
+		      (hact 'hsys-org-global-cycle))
+		    t)
+		   (t
+		    ;; Continue with any further Smart Key non-Org contexts
+		    nil)))
+	    ((eq hsys-org-enable-smart-keys 'buttons)
+	     (cond ((hsys-org-radio-target-def-at-p)
+		    (hact 'org-radio-target)
+		    t)
+		   ((setq start-end (hsys-org-link-at-p))
+		    (if (not assist-flag)
+			(progn (hsys-org-set-ibut-label start-end)
+			       (hact 'org-link))
+		      (hact 'hkey-help))
+		    t)
+ 		   ((hbut:at-p)
+		    ;; Fall through until Hyperbole button context and
+		    ;; activate normally.
+		    nil)
+		   (t
+		    (when (hsys-org-meta-return-shared-p)
+		      (hact 'org-meta-return current-prefix-arg))
+		    ;; Ignore any further Smart Key non-Org contexts
+		    t)))
+	    (t
+	     ;; hsys-org-enable-smart-keys is set to t, so try other Smart
+	     ;; contexts
+	     nil)))))
+
 ;;; ************************************************************************
 ;;; smart-outline functions
 ;;; ************************************************************************
@@ -1448,7 +1785,7 @@ If key is pressed:
 
 
 (defun smart-outline-assist ()
-  "Collaps, expand, and move outline entries.
+  "Collapse, expand, and move outline entries.
 Invoked via an assist-key press when in `outline-mode'.  It assumes that
 its caller has already checked that the assist-key was pressed in an appropriate
 buffer and has moved the cursor to the selected buffer.
@@ -1515,6 +1852,31 @@ CURR-ENTRY-LEVEL is not needed."
 	 (and selective-display (eq (following-char) ?\r)))
     t))
 
+;;; ************************************************************************
+;;; smart-todotxt functions
+;;; ************************************************************************
+
+(defun smart-todotxt ()
+  "Use a single key or mouse key to manipulate `todotxt' items.
+
+If key is pressed:
+ (1) at the end of buffer, bury buffer
+ (2) on a todo item, toggle the completion"
+  (interactive)
+  (cond ((eobp) (todotxt-bury))
+	(t (todotxt-complete-toggle))))
+
+(defun smart-todotxt-assist ()
+  "Use a single assist-key or mouse assist-key to manipulate `todotxt' items.
+
+If key is pressed:
+ (1) at the end of buffer, archive completed items
+ (2) on a todo item, edit it"
+
+  (interactive)
+  (cond ((eobp) (todotxt-archive))
+	(t (todotxt-edit-item))))
+
 (defun smart-eolp ()
   "Return t if point is at the end of a visible line but not the end of the buffer."
   ;; smart-helm handles eol for helm buffers
