@@ -1,10 +1,11 @@
-;;; hui-window.el --- Smart Mouse Key window and modeline depress/release actions.
+;;; hui-window.el --- Smart Mouse Key window and modeline depress/release actions.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    21-Sep-92
+;; Last-Mod:     17-Apr-22 at 11:39:16 by Bob Weiner
 ;;
-;; Copyright (C) 1992-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1992-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -46,7 +47,7 @@
 ;;   buffer/file menu item         Display buffer/file in      Swap window buffers
 ;;                                   window of button release
 ;;   buffer/file menu 1st line     Move buffer/file menu       Swap window buffers
-;;   anywhere else                 Create/modify a link button Swap window buffers
+;;   anywhere else                 Create/edit a link button   Swap window buffers
 ;;
 ;; Drag outside of Emacs from:
 ;;   buffer/file menu item         Display buffer/file in      Move window to a new frame
@@ -78,6 +79,7 @@
 (require 'hycontrol)
 ;; If installed, use pulse library for momentary highlighting of buffer/file item lines.
 (require 'pulse nil t)
+(require 'hui-select)
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -182,21 +184,16 @@ drag release window.")
     (setq hmouse-alist
 	  (append
 	   '(
-	     ;; If click in the minibuffer when it is not active (blank),
-	     ;; display the Hyperbole minibuffer menu or popup the jump menu.
-	     ((hmouse-inactive-minibuffer-p) .
-	      ((funcall action-key-minibuffer-function) .
-	       (funcall assist-key-minibuffer-function)))
 	     ((hmouse-drag-thing) .
 	      ((hmouse-yank-region) . (hmouse-kill-and-yank-region)))
 	     ((hmouse-drag-window-side) .
 	      ((hmouse-resize-window-side) . (hmouse-resize-window-side)))
 	     ;;
-	     ;; Although Hyperbole can distinguish whether
-	     ;; inter-window drags are between frames or not,
-	     ;; having different behavior for those 2 cases could
-	     ;; be confusing, so treat all modeline drags between
-	     ;; windows the same and comment out this next clause.
+	     ;; Although Hyperbole can distinguish whether inter-window
+	     ;; drags are between frames or not, having different behavior
+	     ;; for those 2 cases could be confusing, so treat all
+	     ;; modeline drags between windows the same and comment out
+	     ;; this next clause.
 	     ;;   Modeline drag between frames
 	     ;;   ((and (hmouse-modeline-depress) (hmouse-drag-between-frames)) .
 	     ;;    ((hmouse-clone-window-to-frame) . (hmouse-move-window-to-frame)))
@@ -234,6 +231,16 @@ drag release window.")
 	      ((or (hmouse-drag-item-to-display)
 		   (hycontrol-clone-window-to-new-frame)) .
 		   (hycontrol-window-to-new-frame)))
+	     ;; If click in the minibuffer when it is not active (blank),
+	     ;; display the Hyperbole minibuffer menu or popup the jump menu.
+	     ((hmouse-inactive-minibuffer-p) .
+	      ((funcall action-key-minibuffer-function) .
+	       (funcall assist-key-minibuffer-function)))
+	     ((and (boundp 'ivy-mode) ivy-mode (minibuffer-window-active-p (selected-window))) .
+	      ((ivy-mouse-done action-key-release-args) . (ivy-mouse-dispatching-done assist-key-release-args)))
+	     ;; Handle widgets in Custom-mode
+	     ((eq major-mode 'Custom-mode) .
+	      ((smart-custom) . (smart-custom-assist)))
 	     ;;
 	     ;; Now since this is not a drag and if there was an active
 	     ;; region prior to when the Action or Assist Key was
@@ -304,14 +311,15 @@ part of InfoDock and not a part of Hyperbole)."
 
 (defun hmouse-prior-active-region ()
   "Return t iff there is a non-empty active region in buffer of the last Smart Mouse Key release."
-  (when (setq hkey-value (if assist-flag assist-key-depress-prev-point action-key-depress-prev-point))
+  (when (and (setq hkey-value (if assist-flag assist-key-depress-prev-point action-key-depress-prev-point))
+	     (buffer-live-p (marker-buffer hkey-value)))
     (save-excursion
-      (set-buffer (marker-buffer hkey-value))
-      ;; Store and goto any prior value of point from the region
-      ;; prior to the Smart Key depress, so we can return to it later.
-      (and (goto-char hkey-value)
-	   (hmouse-save-region)
-	   t))))
+      (with-current-buffer (marker-buffer hkey-value)
+	;; Store and goto any prior value of point from the region
+	;; prior to the Smart Key depress, so we can return to it later.
+	(and (goto-char hkey-value)
+	     (hmouse-save-region)
+	     t)))))
 
 (defun hmouse-dired-readin-hook ()
   "Remove local `hpath:display-where' setting whenever re-read a dired directory.
@@ -329,9 +337,7 @@ displayed in all windows on screen, including the dired window.
 
 If the directory is re-read into the dired buffer with {g}, then Action
 Key behavior reverts to as though no items have been dragged."
-  nil
-  " DisplayHere"
-  nil
+  :lighter " DisplayHere"
   (if hmouse-dired-display-here-mode
       (progn (set (make-local-variable 'hpath:display-where) 'this-window)
 	     (add-hook 'dired-after-readin-hook 'hmouse-dired-readin-hook nil t))
@@ -460,7 +466,7 @@ Signals an error if the buffer is read-only."
 	;; In this case, we want an error that will terminate execution so that
 	;; hkey-region is not reset to nil.  This allows the user to fix the
 	;; problem and then to try yanking again.
-	(error "(hmouse-yank-region): Use {%s} to enable yanking into this buffer"
+	(error "(hmouse-yank-region): Buffer is read-only; use {%s} to enable yanking"
 	       (hmouse-read-only-toggle-key))
       ;; Permanently return to release point
       (let ((release-window (if assist-flag assist-key-release-window action-key-release-window)))
@@ -496,9 +502,9 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 	 (window-live-p action-key-release-window)
 	 (not (eq action-key-depress-window action-key-release-window)))))
 
-(defun hmouse-drag-same-window ()
+(defun hmouse-press-release-same-window ()
   "Return non-nil if last Action Key depress and release were in the same window.
-If free variable `assist-flag' is non-nil, uses Assist Key."
+If free variable `assist-flag' is non-nil, use Assist Key."
   (if assist-flag
       (and (window-live-p assist-key-depress-window)
 	   (window-live-p assist-key-release-window)
@@ -533,12 +539,17 @@ items in other modes."
     (hmouse-item-to-window new-window)
     t))
 
+(defun hmouse-drag-same-window ()
+  "Return non-nil if last Action Key depress and release were in the same window and minimum drag distance was exceeded.
+If free variable `assist-flag' is non-nil, uses Assist Key."
+  (or (hmouse-drag-horizontally) (hmouse-drag-vertically) (hmouse-drag-diagonally)))
+
 (defun hmouse-drag-diagonally ()
   "Return non-nil iff last Action Key use was a diagonal drag within a single window.
-If free variable `assist-flag' is non-nil, uses Assist Key.
+If free variable `assist-flag' is non-nil, use Assist Key.
 Value returned is nil if not a diagonal drag, or one of the following symbols
 depending on the direction of the drag: southeast, southwest, northwest, northeast."
-  (when (hmouse-drag-same-window)
+  (when (hmouse-press-release-same-window)
     (let ((last-depress-x) (last-release-x)
 	  (last-depress-y) (last-release-y))
       (if assist-flag
@@ -566,10 +577,10 @@ depending on the direction of the drag: southeast, southwest, northwest, northea
 
 (defun hmouse-drag-horizontally ()
   "Return non-nil iff last Action Key use was a horizontal drag within a single window.
-If free variable `assist-flag' is non-nil, uses Assist Key.
+If free variable `assist-flag' is non-nil, use Assist Key.
 Value returned is nil if not a horizontal drag, 'left if drag moved left or
 'right otherwise."
-  (when (hmouse-drag-same-window)
+  (when (hmouse-press-release-same-window)
     (let ((last-depress-x) (last-release-x)
 	  (last-depress-y) (last-release-y))
       (if assist-flag
@@ -594,7 +605,7 @@ Value returned is nil if not a horizontal drag, 'left if drag moved left or
 
 (defun hmouse-drag-vertically-within-emacs ()
   "Return non-nil iff last Action Key use was a vertical drag that started and ended within the same Emacs frame.
-If free variable `assist-flag' is non-nil, uses Assist Key.
+If free variable `assist-flag' is non-nil, use Assist Key.
 Value returned is nil if not a vertical line drag, 'up if drag moved up or
 'down otherwise."
   (unless (or (hmouse-drag-between-frames) (hmouse-drag-outside-all-windows))
@@ -626,7 +637,7 @@ Value returned is nil if not a vertical line drag, 'up if drag moved up or
 If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a vertical line drag, 'up if drag moved up or
 'down otherwise."
-  (when (hmouse-drag-same-window)
+  (when (hmouse-press-release-same-window)
     (hmouse-drag-vertically-within-emacs)))
 
 (defun hmouse-drag-window-side ()
@@ -649,10 +660,8 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
 		    hmouse-side-sensitivity))))))
 
 (defun hmouse-read-only-toggle-key ()
-  "Return the first key binding that toggle read-only mode, or nil if none."
-  (key-description (or (where-is-internal #'read-only-mode nil t)
-		       (where-is-internal #'vc-toggle-read-only nil t)
-		       (where-is-internal #'toggle-read-only nil t))))
+  "Return the first key binding that toggles read-only mode, or nil if none."
+  (key-description (where-is-internal #'read-only-mode nil t)))
 
 (defun hmouse-vertical-action-drag ()
   "Handle an Action Key vertical drag within a window: add a window to the right of this one.
@@ -695,19 +704,22 @@ Beep and print message if the window cannot be split further."
   (cond ((null coords) nil)
 	((eventp coords)
 	 (let ((w-or-f (posn-window (event-start coords))))
-	   (if (framep w-or-f) (setq w-or-f (frame-selected-window w-or-f)))
-	   (eq w-or-f window)))
+	   (when (framep w-or-f)
+	     (setq w-or-f (frame-selected-window w-or-f)))
+	   (when (and (eq w-or-f window) (window-valid-p window))
+	     window)))
 	((fboundp 'window-edges)
 	 (let* ((edges (window-edges window))
-		  (w-xmin (nth 0 edges))
-		  (w-ymin (nth 1 edges))
-		  (w-xmax (nth 2 edges))
-		  (w-ymax (nth 3 edges))
-		  (x  (hmouse-x-coord coords))
-		  (y  (hmouse-y-coord coords)))
-	     (and (<= w-xmin x) (<= x w-xmax)
-		  (<= w-ymin y) (<= y w-ymax)
-		  window)))))
+		(w-xmin (nth 0 edges))
+		(w-ymin (nth 1 edges))
+		(w-xmax (nth 2 edges))
+		(w-ymax (nth 3 edges))
+		(x  (hmouse-x-coord coords))
+		(y  (hmouse-y-coord coords)))
+	   (and (<= w-xmin x) (<= x w-xmax)
+		(<= w-ymin y) (<= y w-ymax)
+		(window-valid-p window)
+		window)))))
 
 (defun smart-point-of-coords (coords)
   "Return point within window in which COORDS fall or nil if none.
@@ -725,7 +737,7 @@ Ignores minibuffer window."
 	 (get-buffer-window (marker-buffer coords)))
 	((eventp coords)
 	 (let ((w-or-f (posn-window (event-start coords))))
-	   (if (framep w-or-f) (setq w-or-f (frame-selected-window w-or-f)))
+	   (when (framep w-or-f) (setq w-or-f (frame-selected-window w-or-f)))
 	   w-or-f))
 	(t (let ((window-list (hypb:window-list 'no-minibuf))
 		 (window)
@@ -791,8 +803,9 @@ With optional boolean NEW-WINDOW non-nil, sensibly split the release window befo
 (defun hmouse-inactive-minibuffer-p ()
   "Return t if the last command event was a mouse press or release within an inactive minibuffer, else nil."
   (let ((window (posn-window (event-start last-command-event))))
-    (if (framep window) (setq window (frame-selected-window window)))
-    (and (window-minibuffer-p window)
+    (when (framep window) (setq window (frame-selected-window window)))
+    (and (window-live-p window)
+	 (window-minibuffer-p window)
 	 (not (minibuffer-window-active-p window)))))
 
 (defun hmouse-insert-region ()
@@ -838,7 +851,6 @@ item, this moves the menu buffer itself to the release location."
 	 ;; Release may be outside of an Emacs window in which case,
 	 ;; create a new frame and window.
 	 (w2 (or action-key-release-window (frame-selected-window (hycontrol-make-frame))))
-	 (buf-name)
 	 (w1-ref)
 	 (pos))
     (when (and (window-live-p w1) (window-live-p w2))
@@ -867,7 +879,7 @@ item, this moves the menu buffer itself to the release location."
 		 (if (not (window-live-p w1))
 		     (error "(hmouse-item-to-window): Action Mouse Key item drag must start in a live window")
 		   (error "(hmouse-item-to-window): No item to display at start of Action Mouse Key drag")))
-		((buffer-live-p w1-ref)
+		((buffer-live-p w1-ref) ;; Must be a buffer, not a buffer name
 		 (set-window-buffer w2 w1-ref)
 		 (set-buffer w1-ref))
 		((and (stringp w1-ref) (file-readable-p w1-ref))
@@ -912,7 +924,7 @@ If the Action Key is:
 	    ((hmouse-release-left-edge) (bury-buffer))
 	    ((hmouse-release-right-edge)
 	     (if (eq major-mode 'Info-mode)
-		 (Info-exit)
+		 (quit-window)
 	       (info)))
 	    (t (funcall action-key-modeline-function))))))
 
@@ -959,18 +971,14 @@ If the Assist Key is:
   "Return non-nil if last Smart Key depress and release locations differ.
 Even if the mouse pointer stays within the same position within a
 window, its frame may have been moved by a bottommost modeline drag."
-  (not (and (eq (if assist-flag
-		    assist-key-depress-window
-		  action-key-depress-window)
-		(if assist-flag
-		    assist-key-release-window
-		  action-key-release-window))
-	    (equal (if assist-flag
-		       assist-key-depress-position
-		     action-key-depress-position)
-		   (if assist-flag
-		       assist-key-release-position
-		     action-key-release-position)))))
+  (or (not (eq (if assist-flag
+		   assist-key-depress-window
+		 action-key-depress-window)
+	       (if assist-flag
+		   assist-key-release-window
+		 action-key-release-window)))
+      ;; Depress and release were in the same window; test if there is a drag.
+      (hmouse-drag-same-window)))
 
 (defun hmouse-modeline-click ()
   "Return non-nil if last Smart Key depress and release were at a single point (less than drag tolerance apart) in a modeline."
@@ -987,8 +995,8 @@ window, its frame may have been moved by a bottommost modeline drag."
       ;; the mode-line area, so manually compute if that was the location.
       (let* ((w (smart-window-of-coords event))
 	     ;; Do all calculations in pixels if possible.
-	     (line-height (if w (frame-char-height (window-frame w))))
-	     (mode-ln (if w (nth 3 (window-edges w nil t t))))
+	     (line-height (when (and w (window-live-p w)) (frame-char-height (window-frame w))))
+	     (mode-ln (when (and w (window-live-p w)) (nth 3 (window-edges w nil t t))))
 	     (last-press-y (cdr (posn-x-y (event-start event)))))
 	(and (not (eq w (minibuffer-window)))
 	     last-press-y mode-ln (< (- mode-ln last-press-y) line-height)))))
@@ -1042,32 +1050,22 @@ If free variable `assist-flag' is non-nil, uses Assist Key."
       (eq (lookup-key keymap [mode-line mouse-1]) 'mode-line-previous-buffer))))
 
 (defun hmouse-modeline-resize-window ()
-  "Resizes window whose mode line was depressed on by the last Smart Key.
+  "Resize window whose mode line was depressed upon with the last Smart Key.
 Resize amount depends upon the vertical difference between press and release
 of the Smart Key."
   (cond ((not (hyperb:window-system)) nil)
-	(t (let* ((owind (selected-window))
-		  (window (smart-window-of-coords
-			   (if assist-flag assist-key-depress-args
-			     action-key-depress-args)))
-		  (mode-ln (and window (1- (nth 3 (window-edges window)))))
-		  (last-release-y
-		   (hmouse-y-coord
-		    (if assist-flag assist-key-release-args
-		      action-key-release-args)))
-		  (shrink-amount (- mode-ln last-release-y)))
-	     ;; Restore position of point prior to Action Key release.
-	     (if action-key-release-prev-point
-		 (let ((obuf (current-buffer)))
-		   (unwind-protect
-		       (progn
-			 (set-buffer
-			  (marker-buffer action-key-release-prev-point))
-			 (goto-char
-			  (marker-position action-key-release-prev-point)))
-		     (set-buffer obuf))))))))
+	(t ;; Restore position of point prior to Action Key release.
+	 (when action-key-release-prev-point
+	   (let ((obuf (current-buffer)))
+	     (unwind-protect
+		 (progn
+		   (set-buffer
+		    (marker-buffer action-key-release-prev-point))
+		   (goto-char
+		    (marker-position action-key-release-prev-point)))
+	       (set-buffer obuf)))))))
 
-(defun hmouse-clone-window-to-frame (&optional always-delete-flag)
+(defun hmouse-clone-window-to-frame (&optional _always-delete-flag)
   (let ((hycontrol-keep-window-flag t))
     (hmouse-move-window-to-frame)))
 
@@ -1150,8 +1148,7 @@ of the Smart Key."
 		 (hmouse-x-coord
 		  (if assist-flag assist-key-release-args
 		    action-key-release-args)))
-		(shrink-amount (- right-side-ln last-release-x))
-		)
+		(shrink-amount (- right-side-ln last-release-x)))
 	   ;; Restore position of point prior to Action Key release.
 	   (if action-key-release-prev-point
 	       (let ((obuf (current-buffer)))
@@ -1180,13 +1177,10 @@ of the Smart Key."
 	 (w2 (if assist-flag assist-key-release-window
 	       action-key-release-window))
 	 (w1-buf (and w1 (window-buffer w1)))
-	 (w2-buf (and w2 (window-buffer w2)))
-	 )
+	 (w2-buf (and w2 (window-buffer w2))))
     (cond ((not (and w1 w2))
 	   (error "(hmouse-swap-buffers): Last depress or release was not within a window"))
-	  ((eq w1 w2)
-	   ;; Do nothing silently.
-	   )
+	  ((eq w1 w2))  ;; Do nothing silently.
 	  (t ;; Swap window buffers.
 	   (set-window-buffer w1 w2-buf)
 	   (set-window-buffer w2 w1-buf)))))
@@ -1200,27 +1194,24 @@ of the Smart Key."
 	 (w1-width  (and w1 (window-width w1)))
 	 (w1-height (and w1 (window-height w1)))
 	 (w2-width  (and w2 (window-width w2)))
-	 (w2-height (and w2 (window-height w2)))
-	 )
+	 (w2-height (and w2 (window-height w2))))
     (or (and w1 w2)
 	(error "(hmouse-swap-windows): Last depress or release was not within a window"))
     (unwind-protect
 	(progn
 	  (select-window w1)
-	  (if (not (= w1-height (frame-height)))
-	      (shrink-window (- w1-height w2-height)))
-	  (if (not (= w1-width (frame-width)))
-	      (shrink-window-horizontally (- w1-width w2-width)))
+	  (unless (= w1-height (frame-height))
+	    (shrink-window (- w1-height w2-height)))
+	  (unless (= w1-width (frame-width))
+	    (shrink-window-horizontally (- w1-width w2-width)))
 	  (select-window w2)
 	  (setq w2-width (window-width w2)
 		w2-height (window-height w2))
-	  (if (not (= w2-height (frame-height)))
-	      (shrink-window (- w2-height w1-height)))
-	  (if (not (= w2-width (frame-width)))
-	      (shrink-window-horizontally (- w2-width w1-width)))
-	  )
-      (select-window w2)
-      )))
+	  (unless (= w2-height (frame-height))
+	    (shrink-window (- w2-height w1-height)))
+	  (unless (= w2-width (frame-width))
+	    (shrink-window-horizontally (- w2-width w1-width))))
+      (select-window w2))))
 
 (defun hmouse-x-coord (args)
   "Return x coordinate in characters from window system dependent ARGS or nil."
@@ -1228,34 +1219,42 @@ of the Smart Key."
 	       (save-excursion
 		 (hypb:goto-marker args)
 		 (current-column))
-	     (eval (cdr (assoc (hyperb:window-system)
-			       '(("emacs" . (progn (if (eventp args) (setq args (event-start args)))
+	     (funcall (cdr (assoc (hyperb:window-system)
+				  '(("emacs" . (lambda (args)
+						 (when (eventp args) (setq args (event-start args)))
 						   (cond
 						    ((posnp args)
 						     (let ((w-or-f (posn-window args)))
-						       (if (framep w-or-f)
-							   (setq w-or-f (frame-selected-window w-or-f)))
-						       (+ (car (posn-col-row args))
+						       (when (framep w-or-f)
+							 (setq w-or-f (frame-selected-window w-or-f)))
+						       (+ (condition-case ()
+							      (car (posn-col-row args))
+							    (error 0))
 							  (nth 0 (window-edges w-or-f)))))
 						    (t (car args)))))
-				 ("next"   .  (nth 1 args))
-				 )))))))
-    (if (integerp x) x)))
+				 ("next"   .  (lambda (args) (nth 1 args))))))
+		      args))))
+    (when (integerp x)
+      x)))
 
 (defun hmouse-y-coord (args)
   "Return y coordinate in frame lines from window system dependent ARGS or nil."
-  (let ((y (eval (cdr (assoc (hyperb:window-system)
-			     '(("emacs" . (progn (if (eventp args) (setq args (event-start args)))
-						 (cond ((posnp args)
-							(let ((w-or-f (posn-window args)))
-							  (if (framep w-or-f)
-							      (setq w-or-f (frame-selected-window w-or-f)))
-							  (+ (cdr (posn-col-row args))
-							     (nth 1 (window-edges w-or-f)))))
-						       (t (cdr args)))))
-			       ("next"   .  (nth 2 args))
-			       ))))))
-    (if (integerp y) y)))
+  (let ((y (funcall (cdr (assoc (hyperb:window-system)
+				'(("emacs" . (lambda (args)
+					       (when (eventp args) (setq args (event-start args)))
+						    (cond ((posnp args)
+							   (let ((w-or-f (posn-window args)))
+							     (when (framep w-or-f)
+							       (setq w-or-f (frame-selected-window w-or-f)))
+							     (+ (condition-case ()
+								    (cdr (posn-col-row args))
+								  (error 0))
+								(nth 1 (window-edges w-or-f)))))
+							  (t (cdr args)))))
+				  ("next"   .  (lambda (args) (nth 2 args))))))
+		    args)))
+    (when (integerp y)
+      y)))
 
 ;;; ************************************************************************
 ;;; Private variables
