@@ -1,10 +1,11 @@
-;;; hvar.el --- Variable manipulation routines for GNU Hyperbole
+;;; hvar.el --- Variable manipulation routines for GNU Hyperbole  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     1-Oct-91 at 14:00:24
+;; Last-Mod:     24-Jan-22 at 00:18:52 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -18,6 +19,19 @@
 
 (require 'set)
 
+;;; ************************************************************************
+;;; Forward declarations
+;;; ************************************************************************
+
+(defvar inhibit-hyperbole-messaging) ; ; Defined in `hsettings' required below
+
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defvar var::append-list nil
+  "List of (VAR-SYMBOL . APPENDED-LIST) elements saved from this Emacs session.")
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
@@ -27,15 +41,15 @@
   "Add to HOOK (a symbol ending with -hook) HOOK-FUNCTION and then call HOOK-FUNCTION in every buffer with the matching major mode based on HOOK's name."
   (add-hook hook hook-function)
   (let* ((hook-name (symbol-name hook))
-	     (mode (when (string-match "-hooks?\\'" hook-name)
-		         (intern (substring hook-name 0 (match-beginning 0))))))
+	 (mode (when (string-match "-hooks?\\'" hook-name)
+		 (intern (substring hook-name 0 (match-beginning 0))))))
     (when mode (var:run-hook-in-matching-buffers mode hook-function))))
 
 (defun var:append-all ()
   "Add back all hook values previously added by var:append in this Emacs session.
 The ones that were removed by var:remove-all at some point."
   (mapc (lambda (elt) (var:append (car elt) (cdr elt)))
-	    var::append-list)
+	var::append-list)
   var::append-list)
 
 ;;;###autoload
@@ -51,15 +65,15 @@ Do nothing when `inhibit-hyperbole-messaging' is non-nil."
   (unless inhibit-hyperbole-messaging
     (let ((val) result)
       (setq result
-	        (if (and (boundp var-symbol)
-		             (setq val (symbol-value var-symbol))
-		             (or (when (symbolp val)
-			               (setq val (cons val nil)))
-			             (listp val)))
-		        (progn (when (eq (car val) 'lambda)
-			             (setq val (list val)))
-		               (set var-symbol (set:union val list-to-add)))
-	          (set var-symbol list-to-add)))
+	    (if (and (boundp var-symbol)
+		     (setq val (symbol-value var-symbol))
+		     (or (when (symbolp val)
+			   (setq val (cons val nil)))
+			 (listp val)))
+		(progn (when (functionp val)
+			 (setq val (list val)))
+		       (set var-symbol (set:union val list-to-add)))
+	      (set var-symbol list-to-add)))
       (add-to-list 'var::append-list (cons var-symbol result))
       (symbol-value var-symbol))))
 
@@ -73,7 +87,7 @@ Use to remove from hook variables."
   (when (eq (car list-to-remove) 'lambda)
     (setq list-to-remove (list list-to-remove)))
   (mapc (lambda (func) (remove-hook var-symbol func))
-	    list-to-remove)
+	list-to-remove)
   (setq var::append-list (delete (cons var-symbol list-to-remove) var::append-list))
   (symbol-value var-symbol))
 
@@ -81,16 +95,16 @@ Use to remove from hook variables."
   "Remove all hook values added by var:append in this Emacs session from their associated hook variables.
 Keeps a copy of these values for future re-use."
   (mapc (lambda (elt) (var:remove (car elt) (cdr elt)))
-	    var::append-list)
+	var::append-list)
   var::append-list)
 
 (defun var:run-hook-in-matching-buffers (mode hook-function)
   "For a given major MODE (a symbol) call HOOK-FUNCTION in all existing buffers with that major mode.
 This is used after a hook is changed to affect buffers that existed before the change was made."
   (mapc (lambda (buf) (with-current-buffer buf (funcall hook-function)))
-	    (delq nil (mapcar (lambda (buf) (when (eq (buffer-local-value 'major-mode buf) mode)
-					                      buf))
-			              (buffer-list)))))
+	(delq nil (mapcar (lambda (buf) (when (eq (buffer-local-value 'major-mode buf) mode)
+					  buf))
+			  (buffer-list)))))
 
 ;;; ************************************************************************
 ;;; Private variables
@@ -102,4 +116,8 @@ This is used after a hook is changed to affect buffers that existed before the c
 (provide 'hvar)
 
 
+;; `hsettings' and `hvar' have a cyclic dependency; require this after providing 'hvar
+;; to avoid an infinite loop.
+(require 'hsettings)
+
 ;;; hvar.el ends here
