@@ -1,11 +1,12 @@
-;;; hversion.el --- GNU Hyperbole version and system information setup
+;;; hversion.el --- GNU Hyperbole version and system information setup  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;; Maintainer:   Bob Weiner, Mats Lidell
 ;;
 ;; Orig-Date:     1-Jan-94
+;; Last-Mod:      1-May-22 at 10:25:28 by Bob Weiner
 ;;
-;; Copyright (C) 1994-2020  Free Software Foundation, Inc.
+;; Copyright (C) 1994-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -23,7 +24,7 @@
 ;;; Public variables
 ;;; ************************************************************************
 
-(defconst hyperb:version "7.1.1" "GNU Hyperbole revision number.")
+(defconst hyperb:version "8.0.0" "GNU Hyperbole revision number.")
 
 ;;;###autoload
 (defvar hyperb:microsoft-os-p
@@ -50,6 +51,11 @@ Override this if the system-computed default is incorrect for your specific mous
     "^/tmp_mnt/"
     "Regexp to match any automounter prefix in a pathname."))
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(declare-function br-to-view-window "ext:br")
+
 ;;; ************************************************************************
 ;;; Support functions
 ;;; ************************************************************************
@@ -138,8 +144,7 @@ Where a part in the term-type is delimited by a `-' or  an `_'."
 			  (equal (getenv "TERM") "NeXT")
 			  (equal (getenv "TERM") "eterm"))
 		      ;; NEXTSTEP add-on support to Emacs
-		      "next")
-		     )))
+		      "next"))))
     (set-frame-parameter frame 'hyperb:window-system
 			 (and term (setq term (substring term 0 (string-match "[-_]" term)))))
     term))
@@ -204,7 +209,7 @@ If nil after system initialization, no window system or mouse support is availab
     (error "(id-info-item): Info index item must be a string: `%s'" index-item))))
 
 (if (not (fboundp 'id-tool-quit))
-(defalias 'id-tool-quit 'eval))
+(defalias 'id-tool-quit #'eval))
 
 (if (not (fboundp 'id-tool-invoke))
 (defun id-tool-invoke (sexp)
