@@ -1,10 +1,11 @@
-;;; hycontrol.el --- Interactive sizing, moving, replicating and deleting of windows and frames
+;;; hycontrol.el --- Interactive sizing, moving, replicating and deleting of windows and frames  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     1-Jun-16 at 15:35:36
+;; Last-Mod:     27-Feb-22 at 01:11:05 by Bob Weiner
 ;;
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -34,7 +35,7 @@
 ;;   zooming; if called interactively, it stores the current window
 ;;   configuration for restoration via a press of the `)' key.
 ;;   `hycontrol-enable-windows-mode' is typically bound by Hyperbole
-;;   to {C-c \} or just use {C-h h s w}.  Then press {t} if you want
+;;   to {C-c \ } or just use {C-h h s w}.  Then press {t} if you want
 ;;   to switch to frame control.
 ;;
 ;;   With a HyControl minor mode active, a multi-line help summary of
@@ -121,6 +122,8 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
+(require 'hhist)     ; To store frame-config when hycontrol-windows-grid is used
+(require 'hypb)
 (require 'set)
 ;; Frame face enlarging/shrinking (zooming) requires this separately available library.
 ;; Everything else works fine without it, so don't make it a required dependency.
@@ -228,6 +231,11 @@ The final predicate should always be t, for default values, typically of zero.")
   "HyControl copy of `prefix-arg' that it changes within key bindings.
 `pre-command-hook' synchronizes this value to `prefix-arg'.")
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(defvar ibuffer-mode-map)
+
 ;;; Frame Keys
 
 (defvar hycontrol-frames-mode-map
@@ -251,7 +259,7 @@ The final predicate should always be t, for default values, typically of zero.")
     (define-key map [kp-9]  (lambda () (interactive) (hycontrol-numeric-keypad 'kp-9 hycontrol-arg)))
 
     ;; Clear hycontrol-arg
-    (define-key map "."     (lambda () (interactive) (setq hycontrol-arg 0) (hycontrol-frame-to-screen-edges 0)))
+    (define-key map "."     'hycontrol-reset-prefix-arg)
     (define-key map "@"     'hycontrol-windows-grid)
     (define-key map "?"     'hycontrol-toggle-help)
     (define-key map "a"     'hycontrol-frame-adjust-widths)
@@ -296,7 +304,7 @@ The final predicate should always be t, for default values, typically of zero.")
     (define-key map "~"     (lambda () (interactive)
 			      (unless (hycontrol-frame-swap-buffers)
 				(hycontrol-user-error hycontrol-debug "(HyControl): There must be at least two frames and the frame to swap to must have only a single window."))))
-    (define-key map "-"     'hycontrol-frame-minimize-lines)
+    (define-key map "-"     'hycontrol-minus-key)
     (define-key map "+"     'toggle-frame-maximized)
     (define-key map "="     (lambda () (interactive)
 			      (and (> (length (visible-frame-list)) 1)
@@ -306,8 +314,7 @@ The final predicate should always be t, for default values, typically of zero.")
 					    f (frame-pixel-width) (frame-pixel-height) t))
 					 (visible-frame-list)))))
 
-    (define-key map "\C-u"  (lambda () (interactive) (setq hycontrol-arg (* hycontrol-arg 4))
-			      (if (> hycontrol-arg hycontrol-maximum-units) (setq hycontrol-arg 4))))
+    (define-key map "\C-u"  'hycontrol-multiply-universal-arg)
     (define-key map "0"     (lambda () (interactive) (hycontrol-universal-arg-digit 0)))
     (define-key map "1"     (lambda () (interactive) (hycontrol-universal-arg-digit 1)))
     (define-key map "2"     (lambda () (interactive) (hycontrol-universal-arg-digit 2)))
@@ -352,7 +359,7 @@ The final predicate should always be t, for default values, typically of zero.")
     (define-key map [kp-9]  (lambda () (interactive) (hycontrol-numeric-keypad 'kp-9 hycontrol-arg)))
 
     ;; Clear hycontrol-arg
-    (define-key map "."     (lambda () (interactive) (setq hycontrol-arg 0) (hycontrol-frame-to-screen-edges 0)))
+    (define-key map "."     'hycontrol-reset-prefix-arg)
     (define-key map "@"     'hycontrol-windows-grid)
     (define-key map "?"     'hycontrol-toggle-help)
     (define-key map "a"     'hycontrol-frame-adjust-widths)
@@ -402,14 +409,13 @@ The final predicate should always be t, for default values, typically of zero.")
     (define-key map "~"     (lambda () (interactive)
 			      (unless (hycontrol-window-swap-buffers)
 				(hycontrol-user-error hycontrol-debug "(HyControl): There must be precisely two windows within the selected frame to swap buffers."))))
-    (define-key map "-"     'hycontrol-window-minimize-lines)
+    (define-key map "-"     'hycontrol-minus-key)
     (define-key map "+"     'hycontrol-window-maximize-lines)
     (define-key map "="     (lambda () (interactive) (and (> (length (window-list)) 1)
 							  (y-or-n-p "Resize windows evenly across this frame?")
 							  (balance-windows))))
 
-    (define-key map "\C-u"  (lambda () (interactive) (setq hycontrol-arg (* hycontrol-arg 4))
-			      (if (> hycontrol-arg hycontrol-maximum-units) (setq hycontrol-arg 4))))
+    (define-key map "\C-u"  'hycontrol-multiply-universal-arg)
     (define-key map "0"     (lambda () (interactive) (hycontrol-universal-arg-digit 0)))
     (define-key map "1"     (lambda () (interactive) (hycontrol-universal-arg-digit 1)))
     (define-key map "2"     (lambda () (interactive) (hycontrol-universal-arg-digit 2)))
@@ -424,6 +430,11 @@ The final predicate should always be t, for default values, typically of zero.")
     map)
   "Keymap to use when in Hyperbole HyControl window mode.")
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(declare-function ibuffer-get-marked-buffers "ibuffer")
+(declare-function which-key--hide-popup-ignore-command "ext:which-key")
 
 ;;; ************************************************************************
 ;;; Private variables
@@ -477,6 +488,7 @@ associated key: quit {q}, abort {C-g}, or toggle {t}.")
 
 (defvar hycontrol--frame-widths-pointer nil)
 (defvar hycontrol--frame-heights-pointer nil)
+(defvar hycontrol--buffer-list nil)
 (defvar hycontrol--buffer-list-pointer nil)
 
 (defvar hycontrol--initial-which-key-inhibit nil
@@ -530,9 +542,55 @@ associated key: quit {q}, abort {C-g}, or toggle {t}.")
   "Return non-nil if HyControl mode should remain active."
   (null hycontrol--exit-status))
 
+
+(defun hycontrol-invert-prefix-arg ()
+  "Invert the sign of the current Hycontrol prefix argument.
+If the argument is 0, set it to -1."
+  (interactive)
+  (setq this-command 'hycontrol-invert-prefix-arg
+	hycontrol-arg (if (zerop hycontrol-arg) -1 (- hycontrol-arg))))
+
+(defun hycontrol-minus-key ()
+  "Conditional command to execute when the minus key is pressed in a HyControl mode."
+  (interactive)
+  (cond ((and (symbolp last-command)
+	      (string-match-p "\\`\\(hui:menu-enter\\|hycontrol.*-\\(frames\\|windows\\|prefix\\|universal\\)-\\(mode\\|arg\\)\\)" (symbol-name last-command)))
+	 (hycontrol-invert-prefix-arg))
+	(hycontrol-frames-mode
+	 (hycontrol-frame-minimize-lines))
+	(t
+	 (hycontrol-window-minimize-lines))))
+
+(defun hycontrol-multiply-universal-arg ()
+  "Return the new prefix argument based on existing `hycontrol-arg' and one press of the universal arg, C-u."
+  (interactive)
+  (setq this-command 'hycontrol-multiply-universal-arg
+	hycontrol-arg (* hycontrol-arg 4))
+  (when (> (abs hycontrol-arg) hycontrol-maximum-units)
+    (setq hycontrol-arg (if (< hycontrol-arg 0) -4 4)))
+  hycontrol-arg)
+
+(defun hycontrol-reset-prefix-arg ()
+  "Reset HyControl prefix arg to 0."
+  (interactive)
+  (setq hycontrol-arg 0)
+  (hycontrol-frame-to-screen-edges 0))
+
 (defun hycontrol-universal-arg-digit (digit)
   "Return the new prefix argument based on existing `hycontrol-arg' and new DIGIT."
-  (setq hycontrol-arg (+ (* hycontrol-arg 10) digit)) (if (> hycontrol-arg hycontrol-maximum-units) (setq hycontrol-arg digit))
+  (setq this-command 'hycontrol-universal-arg-digit
+	hycontrol-arg
+	(if (< hycontrol-arg 0)
+	    (if (and (= hycontrol-arg -1)
+		     (not (zerop digit)))
+		(- digit)
+	       (- (* hycontrol-arg 10) digit))
+	  (+ (* hycontrol-arg 10) digit)))
+  (when (> (abs hycontrol-arg) hycontrol-maximum-units)
+    (setq hycontrol-arg
+	  (if (< hycontrol-arg 0)
+	      (- digit)
+	    digit)))
   hycontrol-arg)
 
 
@@ -772,9 +830,40 @@ multiple of the default frame font width."
 	      hycontrol-display-buffer-predicate-list)
     (error "(HyDebug): Invalid expression in `hycontrol-display-buffer-predicate-list' - %s" err)))
 
+(defvar hycontrol--blank-buffer (get-buffer-create "BLANK")
+      "Blank buffer to display in extra window grid windows after selected buffer list is exhausted.")
+
 (defun hycontrol-window-display-buffer (window)
-  "Given a WINDOW, choose the next appropriate buffer to display therein using `hycontrol-display-buffer-predicate-list'.
-Also uses the value of free variable `buffer-list' as the list of
+  "Given a WINDOW, choose the next appropriate buffer to display from `hycontrol--buffer-list'.
+
+When `hycontrol--invert-display-buffer-predicates' is non-nil and not 'ignore,
+the list of buffers used is further filtered using the functions and sexpressions
+in `hycontrol-display-buffer-predicate-list', which by default filters
+a frame's buffer-list to just those buffers with attached files.
+
+Filtering is disabled if a specific list of buffers is sent to the
+`hycontrol-make-windows-grid' function that calls this."
+  (let ((buf (car hycontrol--buffer-list-pointer)))
+    (setq hycontrol--buffer-list-pointer (cdr hycontrol--buffer-list-pointer))
+    (while (and buf (or (= (aref (buffer-name buf) 0) ?\ )
+			(and (not hycontrol--invert-display-buffer-predicates)
+			     (not (eval (cons 'or (hycontrol-display-buffer-predicate-results buf)))))
+			(and hycontrol--invert-display-buffer-predicates
+			     (not (eq hycontrol--invert-display-buffer-predicates 'ignore))
+			     (eval (cons 'or (hycontrol-display-buffer-predicate-results buf))))))
+      ;; Buffer is not one to display, get the next one and test again.
+      (setq buf (car hycontrol--buffer-list-pointer)
+	    hycontrol--buffer-list-pointer (cdr hycontrol--buffer-list-pointer)))
+    (set-window-buffer window
+		       (or buf
+			   ;; Out of buffers to display, display a blank one
+			   hycontrol--blank-buffer))))
+
+(defun hycontrol-window-display-buffer-with-repeats (window)
+  "This is no longer used since Hyperbole V8.  Left here for reference.
+
+Given a WINDOW, choose the next appropriate buffer to display therein using `hycontrol-display-buffer-predicate-list'.
+Also uses the value of`hycontrol--buffer-list' as the list of
 buffers to distribute among the windows."
   (let ((buf (car hycontrol--buffer-list-pointer)))
     (setq hycontrol--buffer-list-pointer (cdr hycontrol--buffer-list-pointer))
@@ -782,7 +871,7 @@ buffers to distribute among the windows."
       ;; Now on each new pass through buffer-list, the buffer predicate tests will
       ;; be inverted to expand the list of allowed buffers because the
       ;; 1st pass did not produce a buffer for this window.
-      (setq hycontrol--buffer-list-pointer buffer-list
+      (setq hycontrol--buffer-list-pointer hycontrol--buffer-list
 	    buf (car hycontrol--buffer-list-pointer)
 	    hycontrol--buffer-list-pointer (cdr hycontrol--buffer-list-pointer))
       (unless (eq hycontrol--invert-display-buffer-predicates 'ignore)
@@ -849,8 +938,8 @@ is set to 1.  If it is > `hycontrol-maximum-units', it is set to
 (defun hycontrol-disable-modes ()
   "Disable HyControl Frames and Windows modes when active."
   (interactive)
-  (if (or hycontrol-frames-mode hycontrol-windows-mode)
-      (hycontrol-invert-mode-line))
+  (when (or hycontrol-frames-mode hycontrol-windows-mode)
+    (hycontrol-invert-mode-line))
   (hycontrol-frames-mode -1)
   (hycontrol-windows-mode -1))
 
@@ -870,14 +959,15 @@ instead of quitting HyControl."
 	   (eq (local-key-binding "q") #'quit-window))
       (call-interactively #'quit-window)
     (when hycontrol-windows-mode
-      (message "Finished controlling windows"))
+      (message "Hyperbole finished controlling windows"))
     (when hycontrol-frames-mode
-      (message "Finished controlling frames"))
+      (message "Hyperbole finished controlling frames"))
     (hycontrol-disable-modes)))
 
 ;;;###autoload
 (define-global-minor-mode hycontrol-frames-mode hycontrol-local-frames-mode
-  (lambda () (hycontrol-local-frames-mode 1)))
+  (lambda () (hycontrol-local-frames-mode 1))
+  :group 'hyperbole-screen)
 
 ;; These hooks run by the generated `hycontrol-frames-mode' function
 ;; do the global work of turning on and off the mode.
@@ -889,15 +979,17 @@ instead of quitting HyControl."
 ;; This just sets the keymap locally and shows the minor mode
 ;; indicator in the buffer's mode-line; the separate global minor mode
 ;; turns things on and off.
+;;;###autoload
 (define-minor-mode hycontrol-local-frames-mode
   "Toggle Hyperbole Frames control minor mode in the current buffer."
-  nil " HyFrm" nil
+  :lighter " HyFrm"
   :group 'hyperbole-screen
   :global t)
 
 ;;;###autoload
 (define-global-minor-mode hycontrol-windows-mode hycontrol-local-windows-mode
-  (lambda () (hycontrol-local-windows-mode 1)))
+  (lambda () (hycontrol-local-windows-mode 1))
+  :group 'hyperbole-screen)
 
 ;; These hooks run by the generated `hycontrol-windows-mode' function
 ;; do the global work of turning on and off the mode.
@@ -909,9 +1001,10 @@ instead of quitting HyControl."
 ;; This just sets the keymap locally and shows the minor mode
 ;; indicator in the buffer's mode-line; the separate global minor mode
 ;; turns things on and off.
+;;;###autoload
 (define-minor-mode hycontrol-local-windows-mode
   "Toggle Hyperbole Windows control minor mode in the current buffer."
-  nil " HyWin" nil
+  :lighter " HyWin"
   :group 'hyperbole-screen
   :global t)
 
@@ -1186,8 +1279,7 @@ If already at the top, adjust its height to ARG percent of the screen (50% by de
 if ARG is 1 or nil) but keep it at the top of the screen."
   (interactive "p")
   (setq arg (hycontrol-frame-resize-percentage arg))
-  (let ((frame-resize-pixelwise t)
-	(top (nth 1 (hycontrol-frame-edges))))
+  (let ((frame-resize-pixelwise t))
     (if (hycontrol-frame-at-top-p)
 	;; Reduce frame height to ARG percent, keeping top side fixed.
 	(set-frame-height nil (floor (* (frame-pixel-height) arg)) nil t)
@@ -1378,64 +1470,133 @@ Heights are given in screen percentages by the list
       (set-face-background 'mode-line fg))
     (force-mode-line-update t)))
 
+(defun hycontrol-windows-grid-marked-items ()
+  "Return any marked items in Dired, Buffer Menu or IBuffer mode."
+  ;; Get the list of marked items if in an item list buffer and
+  ;; convert items to buffers.
+  (let ((items (cond ((and (derived-mode-p 'dired-mode)
+			   (mapcar #'find-file-noselect (dired-get-marked-files))))
+		     ((and (eq major-mode 'Buffer-menu-mode)
+			   (Buffer-menu-marked-buffers)))
+		     ((and (eq major-mode 'ibuffer-mode)
+			   (ibuffer-get-marked-buffers))))))
+    ;; Ignore buffer list predicate filters when items exist so the
+    ;; items are not filtered out.
+    (setq hycontrol--invert-display-buffer-predicates
+	  (when items 'ignore))
+    ;; Return either non-nil items or frame's full buffer list.
+    items))
+
 (defun hycontrol-windows-grid-buffer-list ()
-  "Return the existing frame's buffer list with any marked items prepended.
-Marked items are included when the current buffer is in Dired, Buffer
-Menu or IBuffer mode."
-  ;; If selecting buffers by major-mode, then ignore any marked items.
+  "Return either any marked items in Dired, Buffer Menu or IBuffer mode or the existing frame's buffer list.
+If selecting buffers by major-mode, then ignore any marked items."
   (if (and (boundp 'mode) (symbolp mode))
       (buffer-list (selected-frame))
     ;; Get the list of marked items if in an item list buffer and
     ;; convert items to buffers.
-    (let ((items (cond ((and (derived-mode-p 'dired-mode)
-			     (mapcar #'find-file-noselect (dired-get-marked-files))))
-		       ((and (eq major-mode 'Buffer-menu-mode)
-			     (Buffer-menu-marked-buffers)))
-		       ((and (eq major-mode 'ibuffer-mode)
-			     (ibuffer-get-marked-buffers))))))
-      ;; Ignore buffer list predicate filters when items exist so the
-      ;; items are not filtered out.
-      (setq hycontrol--invert-display-buffer-predicates
-	    (when items 'ignore))
-      ;; Prepend items to buffer list.
-      (apply 'set:create (nconc items (buffer-list (selected-frame)))))))
+    ;; Return either non-nil items or frame's full buffer list.
+    (or (hycontrol-windows-grid-marked-items)
+	(buffer-list (selected-frame)))))
 
+;;; Split selected frame into a grid of windows given by row and
+;;; column count, displaying different buffers in each window.
 ;;;###autoload
 (defun hycontrol-windows-grid (arg)
   "Display a grid of windows in the selected frame, sized according to prefix ARG.
 Left digit of ARG is the number of grid rows and the right digit is
-the number of grid columns.
+the number of grid columns.  Use {C-h h h} to restore the prior frame
+configuration after a grid is displayed.
 
 If ARG is 0, prompt for a major mode whose buffers should be
-displayed first in the grid windows, then prompt for the grid size.
+displayed in the grid windows, then prompt for the grid size.
 
-Otherwise, prompt for the grid size if ARG is an invalid size.
+If ARG is < 0, prompt for a glob-type file pattern and display
+files that match the pattern in an auto-sized windows grid.
+
+Otherwise, prompt for the grid size if ARG is an invalid size
+(positive and more than two digits).
 
 With a current buffer in Dired, Buffer Menu or IBuffer mode that
 contains marked items, the buffers associated with those items
-are displayed first in the grid (unless ARG is 0).
+are displayed in the grid (unless ARG is 0).
 
-Then the most recently used buffers are displayed in each window,
+By default, the most recently used buffers are displayed in each window,
 first selecting only those buffers which match any of the
 predicate expressions in `hycontrol-display-buffer-predicate-list'.
 \(The default predicate list chooses buffers with attached files).
-
 Then, if there are not enough buffers for all windows, the buffers
 that failed to match to any predicate are used.  In all cases, buffers
 whose names start with a space are ignored.
 
-When done, this resets the persistent prefix argument to 1 to
-prevent following commands from using the often large grid size
+When done, this resets the persistent HyControl prefix argument to 1
+to prevent following commands from using the often large grid size
 argument."
   (interactive "p")
-  (setq arg (abs (prefix-numeric-value (or arg current-prefix-arg))))
-  (if (/= arg 0)
-      (hycontrol-make-windows-grid arg)
-    (setq current-prefix-arg 0)
-    (call-interactively #'hycontrol-windows-grid-by-major-mode)))
+  (let* ((key (hypb:cmd-key-vector #'hycontrol-windows-grid hyperbole-mode-map))
+	 (this-key-flag (and (called-interactively-p 'interactive)
+			     (equal (this-single-command-keys) key))))
+    (cond ((and this-key-flag (derived-mode-p 'org-mode)
+		(lookup-key org-mode-map key))
+	   ;; Prevent a conflict with binding in Org mode
+	   (call-interactively (lookup-key org-mode-map key)))
+	  ((and this-key-flag (derived-mode-p 'outline-mode)
+		(lookup-key outline-mode-map key))
+	   ;; Prevent a conflict with binding in Outline mode
+	   (call-interactively (lookup-key outline-mode-map key)))
+	  ((and this-key-flag (boundp 'outline-minor-mode)
+		outline-minor-mode (lookup-key outline-minor-mode-map key))
+	   ;; Prevent a conflict with binding in Outline minor mode
+	   (call-interactively (lookup-key outline-minor-mode-map key)))
+	  ;;
+	  ;; No key conflicts, display window grid
+	  (t (let ((hist-elt (hhist:element)))
+	       (setq arg (prefix-numeric-value (or arg current-prefix-arg)))
+	       (cond ((> arg 0)
+		      (hycontrol-make-windows-grid arg))
+		     ((< arg 0)
+		      (setq current-prefix-arg nil)
+		      (call-interactively #'hycontrol-windows-grid-by-file-pattern))
+		     (t
+		      (setq current-prefix-arg 0)
+		      (call-interactively #'hycontrol-windows-grid-by-major-mode)))
+	       (hhist:add hist-elt)))))) ;; Save prior frame configuration for easy return
+
+(defun hycontrol-windows-grid-minimum-size (num-buffers)
+  "Return the minimum integer window grid size to display NUM-BUFFERS.  Minimize number of rows rather than columns.
+Size is a 2 digit whole number with the first digit number of rows and
+the second, number of columns of windows."
+  (let* ((num-cols (ceiling (sqrt num-buffers)))
+	 (num-rows (1- num-cols))
+	 (grid-size (+ (* num-rows 10) num-cols)))
+    (when (< (* num-rows num-cols) num-buffers)
+      (setq grid-size (+ 10 grid-size)))
+    grid-size))
+
+(defun hycontrol-windows-grid-by-buffer-list (buffers)
+  "Display an automatically sized window grid showing list of BUFFERS."
+    (if (null buffers)
+	(error "(hycontrol-windows-grid-by-buffer-list): No matching buffers")
+      (let ((grid-size (hycontrol-windows-grid-minimum-size (length buffers))))
+	(mapc #'switch-to-buffer (reverse buffers))
+	(hycontrol-make-windows-grid grid-size buffers))))
+
+(defun hycontrol-windows-grid-by-file-list (files)
+  "Display an automatically sized window grid showing list of FILES."
+    (if (null files)
+	(error "(hycontrol-windows-grid-by-file-list): No matching files")
+      (let ((grid-size (hycontrol-windows-grid-minimum-size (length files)))
+	    (buffers (nreverse (mapcar #'find-file (reverse files)))))
+      (hycontrol-make-windows-grid grid-size buffers))))
+
+;;;###autoload
+(defun hycontrol-windows-grid-by-file-pattern (pattern &optional full)
+  "Display an automatically sized window grid showing files found from glob PATTERN.
+Use absolute file paths if optional FULL is non-nil."
+  (interactive "FPattern of files to display in windows grid: \nP")
+  (let* ((find-file-wildcards t)
+	 (files (file-expand-wildcards pattern full)))
+    (hycontrol-windows-grid-by-file-list files)))
 
-;;; Split selected frame into a grid of windows given by row and
-;;; column count, displaying different buffers in each window.
 ;;;###autoload
 (defun hycontrol-windows-grid-by-major-mode (arg mode)
   "Display a grid of windows in the selected frame, sized according to prefix ARG, with buffers of major MODE.
@@ -1446,8 +1607,8 @@ See documentation of `hycontrol-windows-grid' for further details."
   (interactive
    (list (prefix-numeric-value current-prefix-arg)
 	 (let* ((set:equal-op 'eq)
-		(mode-strings (mapcar 'symbol-name (apply #'set:create (mapcar (lambda (buf) (buffer-local-value 'major-mode buf))
-									       (hycontrol-windows-grid-buffer-list))))))
+		(mode-strings (mapcar #'symbol-name (apply #'set:create (mapcar (lambda (buf) (buffer-local-value 'major-mode buf))
+										(hycontrol-windows-grid-buffer-list))))))
 	   (intern-soft (completing-read "(HyControl Grid Windows): Major mode of buffers to display: "
 					 mode-strings nil t nil nil (symbol-name major-mode))))))
   (let ((hycontrol-display-buffer-predicate-list `((eq major-mode ',mode))))
@@ -1474,14 +1635,23 @@ See documentation of `hycontrol-windows-grid' for further details."
 	(hycontrol-windows-grid arg)
 	(setq arg nil)))))
 
-(defun hycontrol-make-windows-grid (arg)
+(defun hycontrol-make-windows-grid (arg &optional buffer-list)
   "Display a grid of windows in the selected frame, sized according to prefix ARG.
 Left digit of ARG is the number of grid rows and the right digit is
-the number of grid columns. 
+the number of grid columns.  If ARG is invalid, prompt for grid size.
+
+If optional BUFFER-LIST is provided, use its length to determine
+minimum grid size, ignoring ARG.
 
 See documentation of `hycontrol-windows-grid' for further details."
   (interactive "p")
 
+  (unless buffer-list
+    (setq buffer-list (hycontrol-windows-grid-marked-items)))
+
+  (when buffer-list
+    (setq arg (hycontrol-windows-grid-minimum-size (length buffer-list))))
+
   ;; Check ARG, must be 2 digits of [1-9], else read a new ARG or
   ;; signal an error when in a HyControl mode and help is displayed.
   (if (and (and hycontrol-help-flag (or hycontrol-frames-mode hycontrol-windows-mode))
@@ -1542,9 +1712,10 @@ See documentation of `hycontrol-windows-grid' for further details."
 	       ;; the predicate tests.  Always ignore buffers that
 	       ;; start with a space.  With each succeeding pass, the
 	       ;; predicate list is inverted again.
-	       (let ((buffer-list (hycontrol-windows-grid-buffer-list)))
-		 (setq hycontrol--buffer-list-pointer buffer-list)
-  		 (walk-windows #'hycontrol-window-display-buffer 'no-minibuf))
+	       (setq hycontrol--buffer-list (or buffer-list
+						(hycontrol-windows-grid-buffer-list))
+		     hycontrol--buffer-list-pointer hycontrol--buffer-list)
+  	       (walk-windows #'hycontrol-window-display-buffer 'no-minibuf)
 
 	       ;; Prevent user from mistakenly using the typically
 	       ;; large argument that invoked this function; reset it
@@ -1553,7 +1724,7 @@ See documentation of `hycontrol-windows-grid' for further details."
       (error (set-window-configuration wconfig)
 	     (if (and hycontrol-help-flag (or hycontrol-frames-mode hycontrol-windows-mode))
 		 (pop-to-buffer "*Messages*"))
-	     (error "(HyDebug): %s" err)))))
+	     (error "(HyDebug): Grid Size: %d; %s" arg err)))))
 
 
 (defun hycontrol-delete-other-windows ()
@@ -1606,15 +1777,13 @@ otherwise, delete the original window."
   (interactive)
   (let ((w (selected-window))
 	(frame-resize-pixelwise t)
-	(only-one-window (one-window-p))
-	buf)
+	(only-one-window (one-window-p)))
     (cond ((window-minibuffer-p w)
 	   (beep)
 	   (minibuffer-message "(Hyperbole): Select a non-minibuffer window"))
 	  (t
 	   ;; Give temporary modes such as isearch a chance to turn off.
 	   (run-hooks 'mouse-leave-buffer-hook)
-	   (setq buf (window-buffer w))
 	   (select-frame (make-frame (frame-parameters)))
 	   (unless only-one-window
 	     (hycontrol-set-frame-size nil (window-size w t t) (window-size w nil t) t))
