@@ -1,10 +1,11 @@
-;;; hypb.el --- Miscellaneous GNU Hyperbole support features
+;;; hypb.el --- Miscellaneous GNU Hyperbole support features  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     6-Oct-91 at 03:42:38
+;; Last-Mod:      1-May-22 at 10:25:55 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2019  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -16,7 +17,7 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hversion hact locate)))
+(eval-and-compile (mapc #'require '(compile hversion hact locate)))
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -37,10 +38,34 @@ It must end with a space."
   :type 'string
   :group 'hyperbole-commands)
 
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(defvar mh-e-RCS-id)
+(defvar pm-version)
+(defvar vm-version)
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
 
+(defmacro hypb:assert-same-start-and-end-buffer (&rest body)
+  "Trigger an error with traceback if the buffer is not live or its name differs at the start and end of BODY."
+  (declare (indent 0) (debug t))
+  `(let ((debug-on-error t)
+	 (start-buffer (current-buffer)))
+     (unless (buffer-live-p start-buffer)
+       (error "Start buffer, '%s', is not live" (current-buffer)))
+     ;; `kill-buffer' can change current-buffer in some odd cases.
+     (unwind-protect
+	 (progn ,@body)
+       (unless  (eq start-buffer (current-buffer))
+	 (error "Start buffer, '%s', differs from end buffer, '%s'" start-buffer (current-buffer)))
+       (unless (buffer-live-p start-buffer)
+	 (error "End buffer, '%s', is not live" (current-buffer))))))
+
 (defun hypb:call-process-p (program &optional infile predicate &rest args)
   "Call an external PROGRAM with INFILE for input.
 If PREDICATE is given, it is evaluated in a buffer with the PROGRAM's
@@ -63,12 +88,11 @@ Rest of ARGS are passed as arguments to PROGRAM."
       (kill-buffer buf))
     found))
 
-
 (defun hypb:char-count (char array)
   "Return count of occurrences of CHAR in ARRAY."
   (let ((i 0) (c 0) (l (length array)))
     (while (< i l)
-      (if (= char (aref array i)) (setq c (1+ c)))
+      (when (= char (aref array i)) (setq c (1+ c)))
       (setq i (1+ i)))
     c))
 
@@ -78,25 +102,73 @@ OP may be +, -, xor, or default =."
   (let ((func (cond ((eq op '+)   #'logior)
 		    ((eq op '-)   (lambda (p1 p2) (logand (lognot p1) p2)))
 		    ((eq op 'xor) #'logxor)
-		    (t            (lambda (p1 p2) p1)))))
+		    (t            (lambda (p1 p2) p2 p1)))))
     (set-file-modes file (funcall func (hypb:oct-to-int octal-permissions)
 				  (file-modes file)))))
 
-(defun hypb:cmd-key-string (cmd-sym &optional keymap)
-  "Return a single pretty printed key sequence string bound to CMD-SYM.
-Global keymap is used unless optional KEYMAP is given."
+(defun hypb:cmd-key-series (cmd-sym &optional keymap)
+  "Return a single, brace-delimited, human readable key sequence string bound to CMD-SYM.
+Global keymap is used unless optional KEYMAP is given.
+
+Trigger an error if CMD-SYM is not bound."
   (if (and cmd-sym (symbolp cmd-sym) (fboundp cmd-sym))
-  (let* ((get-keys (lambda (cmd-sym keymap)
-		     (key-description (where-is-internal
-				       cmd-sym keymap 'first))))
-	 (keys (funcall get-keys cmd-sym keymap)))
-    (concat "{"
-	    (if (string= keys "")
-		(concat (funcall get-keys 'execute-extended-command nil)
-			" " (symbol-name cmd-sym) " RET")
-	      keys)
-	    "}"))
-  (error "(hypb:cmd-key-string): Invalid cmd-sym arg: %s" cmd-sym)))
+      (let* ((get-keys (lambda (cmd-sym keymap)
+		         (key-description (where-is-internal
+				           cmd-sym keymap 'first))))
+	     (keys (funcall get-keys cmd-sym keymap)))
+        (concat "{"
+	        (if (string= keys "")
+		    (concat (funcall get-keys 'execute-extended-command nil)
+			    " " (symbol-name cmd-sym) " RET")
+	          keys)
+	        "}"))
+    (error "(hypb:cmd-key-series): Invalid cmd-sym arg: %s" cmd-sym)))
+
+(defun hypb:cmd-key-vector (cmd-sym &optional keymap)
+  "Return as a vector the first key sequence bound to CMD-SYM from global keymap or optional KEYMAP.
+Return nil if no valid key binding is found.
+
+The returned value may be compared with `equal' to `this-single-command-keys'.
+Use `key-description' to make it human readable."
+  (where-is-internal cmd-sym keymap t))
+
+(defun hypb:installation-type ()
+  "Return a list of (hyperbole-installation-type-string hyperbole-install-version-number-string).
+If no matching installation type is found, return a list of (\"unknown\" hyperb:dir)."
+  (let ((hypb-dir-name (file-name-nondirectory (directory-file-name hyperb:dir)))
+	(dir-sep-string (substring (file-name-as-directory ".") -1)))
+    (cond
+     ;; straight.el package install -- hyperbole gnu-elpa-mirror master 56cd3d8 2022-02-05
+     ((string-match (concat dir-sep-string "straight" dir-sep-string
+			    "build" dir-sep-string "hyperbole") hyperb:dir)
+      (let* ((plist (hypb:straight-package-plist "hyperbole"))
+	     (pkg-version (plist-get plist :version))
+	     (git-commit (when (string-match " \\([a-f0-9]+\\) " pkg-version)
+			   (match-string 1 pkg-version))))
+	(list "straight" git-commit)))
+     ;; elpa-devel package install -- hyperbole-7.0.0pre0.20220126.1138
+     ((string-match "hyperbole-\\([.[:digit:]]+pre[.[:digit:]]+\\)" hypb-dir-name)
+      (list "elpa-devel" (match-string 1 hypb-dir-name)))
+     ;; melpa/quelpa package install -- hyperbole-20220205.1429
+     ((string-match "hyperbole-\\([1-9][0-9][0-9][0-9][0-1][0-9][0-3][0-9]\\.[0-9]+\\)"
+		    hypb-dir-name)
+      (list "melpa" (match-string 1 hypb-dir-name)))
+     ;; git install -- hyperbole d27f4c5197
+     ((file-exists-p (expand-file-name ".git" hyperb:dir))
+      (ignore-errors
+        (let ((default-directory hyperb:dir))
+          (list
+           "git"
+           (substring (shell-command-to-string "git rev-parse HEAD") 0 10)))))
+     ;; elpa package install -- /elpa/hyperbole-8.0.0"
+     ((and (string-match-p (concat dir-sep-string "elpa" dir-sep-string) hyperb:dir)
+	   (string-match "hyperbole-\\([.[:digit:]]+\\)" hypb-dir-name))
+      (list "elpa" (match-string 1 hypb-dir-name)))
+     ;; tarball archive install -- hyperbole-8.0.0
+     ((string-match "hyperbole-\\([.[:digit:]]+\\)" hypb-dir-name)
+      (list "archive" (match-string 1 hypb-dir-name)))
+     ;; unknown -- hyperbole
+     (t (list "unknown" hyperb:dir)))))
 
 ;;;###autoload
 (defun hypb:configuration (&optional out-buf)
@@ -109,42 +181,40 @@ Global keymap is used unless optional KEYMAP is given."
       (goto-char (point-max)))
     (delete-blank-lines) (delete-blank-lines)
     (let ((start (point)))
-      (insert (format "I use:\tEditor:      %s\n\tHyperbole:   %s\n"
-		      (cond ((boundp 'infodock-version)
-			     infodock-version)
-			    (t (hypb:replace-match-string
-				" of .+" (emacs-version) "" t)))
-                      hyperb:version))
-      (if (and (boundp 'br-version) (stringp br-version))
-	  (insert (format "\tOO-Browser:  %s\n" br-version)))
-      (if (and (boundp 'system-configuration) (stringp system-configuration))
-	  (insert (format "\tSys Type:    %s\n" system-configuration)))
+      (insert (format "I use:\tEditor:      GNU Emacs %s\n\tHyperbole:   %s\n"
+		      emacs-version hyperb:version))
+      (when (and (boundp 'br-version) (stringp br-version))
+	(insert (format "\tOO-Browser:  %s\n" br-version)))
+      (when (and (boundp 'system-configuration) (stringp system-configuration))
+	(insert (format "\tSys Type:    %s\n" system-configuration)))
       (insert (format "\tOS Type:     %s\n\tWindow Sys:  %s\n"
                       system-type (or window-system (hyperb:window-system)
 				      "None")))
-      (if (and (boundp 'hmail:reader) hmail:reader)
-          (insert (format "\tMail Reader: %s\n"
-                          (cond ((eq hmail:reader 'rmail-mode) "RMAIL")
-                                ((eq hmail:reader 'vm-mode)
-                                 (concat "VM " vm-version))
-                                ((and (eq hmail:reader 'mh-show-mode)
-                                      (string-match "v ?\\([0-9]+.[0-9]+\\)"
-                                          mh-e-RCS-id))
-                                 (concat "MH-e "
-                                         (substring mh-e-RCS-id
-                                                    (match-beginning 1)
-                                                    (match-end 1))))
-                                ((eq hmail:reader 'pm-fdr-mode)
-                                 (concat "PIEmail " pm-version))
-                                ))))
-      (if (and (boundp 'hnews:reader) (boundp 'gnus-version) hnews:reader)
-          (insert (format "\tNews Reader: %s\n" gnus-version)))
+      (when (and (boundp 'hmail:reader) hmail:reader)
+        (insert (format "\tMail Reader: %s\n"
+                        (cond ((eq hmail:reader 'rmail-mode) "RMAIL")
+                              ((eq hmail:reader 'vm-mode)
+                               (concat "VM " vm-version))
+                              ((and (eq hmail:reader 'mh-show-mode)
+                                    (string-match "v ?\\([0-9]+.[0-9]+\\)"
+                                                  mh-e-RCS-id))
+                               (concat "MH-e "
+                                       (substring mh-e-RCS-id
+                                                  (match-beginning 1)
+                                                  (match-end 1))))
+                              ((eq hmail:reader 'pm-fdr-mode)
+                               (concat "PIEmail " pm-version))))))
+      (when (and (boundp 'hnews:reader) (boundp 'gnus-version) hnews:reader)
+        (insert (format "\tNews Reader: %s\n" gnus-version)))
+      (let ((install-type (hypb:installation-type)))
+        (when install-type
+          (insert (format "\tInstall:     %s, %s" (car install-type) (cadr install-type)))))
       (insert "\n")
       ;; Insert recent Hyperbole debugging messages if any.
-      (if (get-buffer "*Messages*")
-	  (let ((opoint (point)))
-	    (insert-buffer-substring "*Messages*")
-	    (keep-lines "^(HyDebug)" opoint (point))))
+      (when (get-buffer "*Messages*")
+	(let ((opoint (point)))
+	  (insert-buffer-substring "*Messages*")
+	  (keep-lines "^(HyDebug)" opoint (point))))
       (untabify start (point)))))
 
 (defun hypb:debug ()
@@ -185,6 +255,22 @@ Global keymap is used unless optional KEYMAP is given."
       ;; string.
       string)))
 
+;; Similar keyboard macro to next function, but less flexible: {C-x 1 M-o F M-o a C-x b *scratch* RET M-< M-o s C-@ C-M-h M-o t a C-u C-@ C-u C-@ M-o a C-M-p}
+
+;;;###autoload
+(defun hypb:def-to-buffer (&optional arg buffer)
+  "Copy next optional ARG (default = 1) code definitions to the start of BUFFER (default = *scratch*) and leave point at the start of the inserted text."
+  (interactive "p\nbDef insertion buffer (default *scratch*): ")
+  (let ((def (save-excursion
+	       (mark-defun arg)
+	       (deactivate-mark)
+	       (buffer-substring (region-beginning) (region-end)))))
+    (pop-to-buffer (or buffer "*scratch*"))
+    (goto-char (point-min))
+    (insert def)
+    (goto-char (point-min))
+    (forward-line 1)))
+
 (defun hypb:domain-name ()
   "Return current Internet domain name with '@' prepended or nil if none."
   (let* ((dname-cmd (or (file-exists-p "/usr/bin/domainname")
@@ -193,29 +279,27 @@ Global keymap is used unless optional KEYMAP is given."
 			 (string-match "\\." message-user-fqdn)
 			 message-user-fqdn)
 		    (getenv "DOMAINNAME")
-		    (if dname-cmd
-			(hypb:call-process-p
-			 "domainname" nil
-			 '(substring (buffer-string) 0 -1)))))
+		    (when dname-cmd
+		      (hypb:call-process-p
+		       "domainname" nil
+		       '(substring (buffer-string) 0 -1)))))
 	 host-and-domain)
-    (if (or (and dname (string-match "\\." dname))
-	    (and (setq host-and-domain (hypb:call-process-p
-					"hostname" nil '(substring (buffer-string) 0 -1) "-f"))
-		 (setq dname (if (string-match "\\`[^.]+\\." host-and-domain)
+    (when (or (and dname (string-match "\\." dname))
+	      (and (setq host-and-domain (hypb:call-process-p
+					  "hostname" nil '(substring (buffer-string) 0 -1) "-f"))
+		   (setq dname (when (string-match "\\`[^.]+\\." host-and-domain)
 				 (substring host-and-domain (match-end 0)))))
-	    (let* ((src "/etc/resolv.conf")
-		   (src-buf-exists-p (get-file-buffer src)))
-	      (and (file-exists-p src) (file-readable-p src)
-		   (with-temp-buffer
-		     (insert-file-contents-literally src)
-		     (goto-char (point-min))
-		     (if (re-search-forward  "^domain[ \t]+\\([^ \t\n\r]+\\)"
-					     nil t)
-			 (setq dname (buffer-substring (match-beginning 1)
-						       (match-end 1))))
-		     (or src-buf-exists-p (kill-buffer nil))
-		     dname))))
-	(concat "@" dname))))
+	      (let* ((src "/etc/resolv.conf")
+		     (src-buf-exists-p (get-file-buffer src)))
+	        (and (file-exists-p src) (file-readable-p src)
+		     (with-temp-buffer
+		       (insert-file-contents-literally src)
+		       (goto-char (point-min))
+		       (when (re-search-forward  "^domain[ \t]+\\([^ \t\n\r]+\\)" nil t)
+			 (setq dname (match-string 1)))
+		       (or src-buf-exists-p (kill-buffer nil))
+		       dname))))
+      (concat "@" dname))))
 
 (defun hypb:emacs-byte-code-p (obj)
   "Return non-nil iff OBJ is an Emacs byte compiled object."
@@ -231,18 +315,37 @@ Global keymap is used unless optional KEYMAP is given."
     (put 'error 'error-message msg)
     (error msg)))
 
+(defun hypb:fgrep-git-log (string)
+  "Asynchronously list git log entries whose changesets include `string' for selection and display.
+A press of RET, the Action Key or the Assist Key on any log line will display its committed changes."
+  (interactive "sFgrep git commits containing: ")
+  (compile (format "git log -S'%s' --line-prefix='commit ' --oneline" string)
+	   #'hypb:fgrep-git-log-mode))
+
+(defun hypb:fgrep-git-log-activate (_ignore1 &optional _ignore2)
+  "Display git commit for the current line when `compile-goto-error' {RET} is used.
+Does not support use of next and previous error; simply displays the current one."
+  (interactive '(nil))
+  (hkey-either nil))
+
+(define-derived-mode hypb:fgrep-git-log-mode compilation-mode "Fgrep-Git-Log"
+  "Major mode (derived from `compilation-mode' for listing a matching set of git commits for selection and display.
+Turning on Fgrep-Git-Log mode runs the normal hook `compilation-mode-hook'."
+  (setq-local next-error-function #'hypb:fgrep-git-log-activate))
+
 (defun hypb:file-major-mode (file)
   "Return the major mode used by FILE.
 FILE is temporarily read into a buffer to determine the major mode if necessary."
   (let ((existing-flag (get-file-buffer file))
 	(buf (find-file-noselect file)))
-    (prog1 (when buf (save-excursion (set-buffer buf) major-mode))
+    (prog1 (when buf (save-excursion (with-current-buffer buf
+				       major-mode)))
       (unless (or existing-flag (null buf))
 	(kill-buffer buf)))))
 
 (defun hypb:format-quote (arg)
-  "Replace all single % with %% in any string ARG so that a call to `format' or `message' ignore them.
-Return either the modified string or the original ARG."
+  "Replace all single % with %% in any string ARG so that a call to `format' or `message' ignores them.
+Return either the modified string or the original ARG when not modified."
   (if (stringp arg)
       (replace-regexp-in-string
        "@@@" "%%" (replace-regexp-in-string
@@ -252,28 +355,15 @@ Return either the modified string or the original ARG."
     arg))
 
 (defun hypb:function-copy (func-symbol)
-  "Copy FUNC-SYMBOL's body for overloading.  Return copy of body."
+  "Copy FUNC-SYMBOL's body for overloading.  Return a copy of the body."
   (if (fboundp func-symbol)
       (let ((func (hypb:indirect-function func-symbol)))
 	(cond ((listp func) (copy-sequence func))
 	      ((subrp func) (error "(hypb:function-copy): `%s' is a primitive; can't copy body"
 				   func-symbol))
 	      ((and (hypb:emacs-byte-code-p func) (fboundp 'make-byte-code))
-	       (if (not (fboundp 'compiled-function-arglist))
-		   (let ((new-code (append func nil))) ; turn it into a list
-		     (apply 'make-byte-code new-code))
-		 ;; Can't reference bytecode objects as vectors in modern
-		 ;; XEmacs.
-		 (let ((new-code (nconc
-				  (list (compiled-function-arglist func)
-					(compiled-function-instructions func)
-					(compiled-function-constants func)
-					(compiled-function-stack-depth func)
-					(compiled-function-doc-string func))))
-		       spec)
-		   (if (setq spec (compiled-function-interactive func))
-		       (setq new-code (nconc new-code (list (nth 1 spec)))))
-		   (apply 'make-byte-code new-code))))
+	       (let ((new-code (append func nil))) ; turn it into a list
+		 (apply 'make-byte-code new-code)))
 	      (t (error "(hypb:function-copy): Can't copy function body: %s" func))))
     (error "(hypb:function-copy): `%s' symbol is not bound to a function"
 	   func-symbol)))
@@ -310,39 +400,70 @@ FUNC-SYM may be a function symbol or its body.  All occurrences within lists
 are replaced.  Returns body of modified FUNC-SYM."
   (let ((body (hypb:indirect-function func-sym))
 	(constant-vector) (constant))
-    (if (listp body)
-	;; assume V18 byte compiler
-	(setq constant-vector
-	      (car (delq nil (mapcar
-			      (lambda (elt)
-				(and (listp elt)
-				     (vectorp (setq constant-vector (nth 2 elt)))
-				     constant-vector))
-			      body))))
-      ;; assume EMACS byte compiler   (eq (compiled-function-p body) t)
-      (setq constant (if (fboundp 'compiled-function-constants)
-			 (compiled-function-constants body)
-		       (aref body 2))
-	    constant-vector (if (vectorp constant) constant)))
-    (if constant-vector
-	;; Code is byte-compiled.
-	(hypb:constant-vector-symbol-replace
-	 constant-vector sym-to-replace replace-with-sym)
-      ;;
-      ;; Code is not byte-compiled.
-      ;; Replaces occurrence of symbol within lists only.
-      (hypb:map-sublists
-       (lambda (atom list)
-	 ;; The ' in the next line *is* required for proper substitution.
-	 (if (eq atom 'sym-to-replace)
+    (if (subrp body)
+        ;; Non-Lisp code, can't do any replacement
+        body
+      (if (listp body)
+	  ;; assume V18 byte compiler
+	  (setq constant-vector
+	        (car (delq nil (mapcar
+			        (lambda (elt)
+				  (and (listp elt)
+				       (vectorp (setq constant-vector (nth 2 elt)))
+				       constant-vector))
+			        body))))
+        ;; assume EMACS byte compiler   (eq (compiled-function-p body) t)
+        (setq constant (if (fboundp 'compiled-function-constants)
+			   (compiled-function-constants body)
+		         (aref body 2))
+	      constant-vector (when (vectorp constant) constant)))
+      (if constant-vector
+	  ;; Code is byte-compiled.
+	  (hypb:constant-vector-symbol-replace
+	   constant-vector sym-to-replace replace-with-sym)
+        ;;
+        ;; Code is not byte-compiled.
+        ;; Replaces occurrence of symbol within lists only.
+        (hypb:map-sublists
+         (lambda (atom list)
+	   ;; The ' in the next line *is* required for proper substitution.
+	   (when (eq atom 'sym-to-replace)
 	     (let ((again t))
 	       (while (and again list)
-		 (if (eq (car list) atom)
+	         (if (eq (car list) atom)
 		     (progn (setcar list replace-with-sym)
 			    (setq again nil))
 		   (setq list (cdr list)))))))
-       body))
-    body))
+         body))
+      body)))
+
+;; Extracted from part of `choose-completion' in "simple.el"
+(defun hypb:get-completion (&optional event)
+  "Return the completion at point.
+If EVENT, use EVENT's position to determine the starting position."
+  (interactive (list last-nonmenu-event))
+  ;; In case this is run via the mouse, give temporary modes such as
+  ;; isearch a chance to turn off.
+  (run-hooks 'mouse-leave-buffer-hook)
+  (with-current-buffer (window-buffer (posn-window (event-start event)))
+    (save-excursion
+      (goto-char (posn-point (event-start event)))
+      (let (beg end)
+        (cond
+         ((and (not (eobp)) (get-text-property (point) 'mouse-face))
+          (setq end (point) beg (1+ (point))))
+         ((and (not (bobp))
+               (get-text-property (1- (point)) 'mouse-face))
+          (setq end (1- (point)) beg (point)))
+         (t (error "No completion here")))
+        (setq beg (previous-single-property-change beg 'mouse-face))
+        (setq end (or (next-single-property-change end 'mouse-face)
+                      (point-max)))
+        (buffer-substring-no-properties beg end)))))
+
+(defun hypb:get-raw-syntax-descriptor (char &optional syntax-table)
+  "Return the raw syntax descriptor for CHAR using the current syntax table or optional SYNTAX-TABLE."
+  (aref (or syntax-table (syntax-table)) char))
 
 ;; Derived from pop-global-mark of "simple.el" in GNU Emacs.
 (defun hypb:goto-marker (marker)
@@ -363,6 +484,12 @@ If MARKER is invalid signal an error."
 	     (goto-char position)
 	     (switch-to-buffer buffer)))))
 
+(defun hypb:grep-git-log (regexp)
+  "Asynchronously list git log entries whose changesets include `regexp' for selection and display.
+A press of RET, the Action Key or the Assist Key on any log line will display its committed changes."
+  (interactive "sGrep git commits containing: ")
+  (compile (format "git log -G'%s' --line-prefix='commit ' --oneline" regexp)))
+
 (defun hypb:help-buf-name (&optional suffix)
   "Return a Hyperbole help buffer name for current buffer.
 With optional SUFFIX string, uses it rather than buffer name."
@@ -418,7 +545,7 @@ copied, otherwise, it is omitted."
     (while (< start end)
       (or (kview:char-invisible-p start) (append-to-buffer buffer start (1+ start)))
       (setq start (1+ start)))))
-	
+
 ;;;###autoload
 (defun hypb:locate (search-string &optional filter arg)
   "Find file name match anywhere, calling the value of `locate-command', and putting results in the `*Locate*' buffer.
@@ -451,14 +578,6 @@ then `locate-post-command-hook'."
 		     current-prefix-arg))
   (locate search-string filter arg))
 
-(defalias 'hypb:mark #'mark)
-
-(defun hypb:mark-marker (inactive-p)
-  "Return this buffer's mark as a marker object, or nil if no mark.
-INACTIVE-P is unused, it is for compatibility with XEmacs' version of
-`mark-marker'."
-  (mark-marker))
-
 ;;;###autoload
 (defun hypb:map-plist (func plist)
   "Return result of applying FUNC of two args, key and value, to key-value pairs in PLIST, a property list."
@@ -480,8 +599,8 @@ structure as LIST.  FUNC is therefore normally used just for its side-effects."
 (defun hypb:map-vector (func object)
   "Return list of results of application of FUNC to each element of OBJECT.
 OBJECT should be a vector or `byte-code' object."
-  (if (not (or (vectorp object) (hypb:emacs-byte-code-p object)))
-      (error "(hypb:map-vector): Second argument must be a vector or byte-code object"))
+  (unless (or (vectorp object) (hypb:emacs-byte-code-p object))
+    (error "(hypb:map-vector): Second argument must be a vector or byte-code object"))
   (let ((end (length object))
 	(i 0)
 	(result))
@@ -519,21 +638,33 @@ WINDOW pixelwise."
 	((symbolp object)
 	 (get object 'hyperbole))))
 
-(defun hypb:replace-match-string (regexp str newtext &optional literal)
-  "Replace all matches for REGEXP in STR with NEWTEXT string and return the result.
+(defun hypb:replace-match-string (regexp str new &optional literal fixedcase)
+  "Replace all matches for REGEXP in STR with NEW string and return the result.
+If NEW is nil, return STR unchanged.
+
 Optional LITERAL non-nil means do a literal replacement.
-Otherwise treat \\ in NEWTEXT string as special:
+Otherwise treat \\ in NEW string as special:
   \\& means substitute original matched text,
   \\N means substitute match for \(...\) number N,
   \\\\ means insert one \\.
-NEWTEXT may instead be a function of one argument (the string to replace in)
+
+If optional fifth arg FIXEDCASE is non-nil, do not alter the case of
+the replacement text.  Otherwise, maybe capitalize the whole text, or
+maybe just word initials, based on the replaced text.  If the replaced
+text has only capital letters and has at least one multiletter word,
+convert NEW to all caps.  Otherwise if all words are capitalized
+in the replaced text, capitalize each word in NEW.
+
+NEW may instead be a function of one argument (the string to replace in)
 that returns a replacement string."
-  (unless (stringp str)
-    (error "(hypb:replace-match-string): 2nd arg must be a string: %s" str))
-  (unless (or (stringp newtext) (functionp newtext))
-    (error "(hypb:replace-match-string): 3rd arg must be a string or function: %s"
-	   newtext))
-  (replace-regexp-in-string regexp newtext str nil literal))
+  (if (null new)
+      str
+    (unless (stringp str)
+      (error "(hypb:replace-match-string): 2nd arg must be a string: %s" str))
+    (unless (or (stringp new) (functionp new))
+      (error "(hypb:replace-match-string): 3rd arg must be a string or function: %s"
+	     new))
+    (replace-regexp-in-string regexp new str fixedcase literal)))
 
 (defun hypb:return-process-output (program &optional infile &rest args)
   "Return as a string the output from external PROGRAM with INFILE for input.
@@ -547,7 +678,8 @@ Removes any trailing newline at the end of the output."
       (apply 'call-process program infile buf nil args)
       (setq output (buffer-string))
       ;; Remove trailing newline from output.
-      (if (> (length output) 0) (setq output (substring output 0 -1)))
+      (when (> (length output) 0)
+        (setq output (substring output 0 -1)))
       (set-buffer-modified-p nil)
       (kill-buffer buf))
     output))
@@ -558,13 +690,13 @@ Removes any trailing newline at the end of the output."
     (flush-lines regexp nil nil t))
 
 ;;;###autoload
-(defun hypb:rgrep (pattern &optional prefix-arg)
+(defun hypb:rgrep (pattern &optional prefx-arg)
   "Recursively grep with symbol at point or PATTERN over all non-backup and non-autosave files in the current directory tree.
-If in an Emacs Lisp mode buffer and no PREFIX-ARG is given, limit search to only .el and .el.gz files."
+If in an Emacs Lisp mode buffer and no PREFX-ARG is given, limit search to only .el and .el.gz files."
   (interactive (list (if (and (not current-prefix-arg) (equal (buffer-name) "*Locate*"))
 			 (read-string "Grep files listed here for: ")
 		       (let ((default (symbol-at-point)))
-			 (if default (setq default (symbol-name default)))
+			 (when default (setq default (symbol-name default)))
 			 (read-string (format "Rgrep below current dir for%s: "
 					      (if default
 						  (format " (default %s)" default)
@@ -581,13 +713,13 @@ If in an Emacs Lisp mode buffer and no PREFIX-ARG is given, limit search to only
 	    (format "%s %s -e \%c%s\%c ."
 		    hypb:rgrep-command
 		    (if (and (memq major-mode '(emacs-lisp-mode lisp-interaction-mode))
-			     (not prefix-arg))
+			     (not prefx-arg))
 			(if (string-match "\\`rg " hypb:rgrep-command)
 			    "-g \"*.el\" -g \"*.el.gz\""
 			  "--include=\"*.el\" --include=\"*.el.gz\"")
 		      (if (string-match "\\`rg " hypb:rgrep-command)
 			  "-g \"!*~\" -g \"!#*\" -g \"!TAGS\""
-			"--exclude=\"*~\" --exclude=\"#*\" --exclude=\"TAGS\""))
+			"--exclude=\".git\" --exclude=\"CVS\" --exclude=\"*~\" --exclude=\"#*\" --exclude=\"TAGS\""))
 		    delim pattern delim))))
     (setq this-command `(grep ,grep-cmd))
     (push this-command command-history)
@@ -612,19 +744,68 @@ The value returned is the value of the last form in BODY."
 	     (select-frame-set-input-focus (window-frame window)))
     (error "(hypb:select-window-frame): Argument must be a live window, not '%s'" window)))
 
+(defun hypb:set-raw-syntax-descriptor (char raw-descriptor &optional syntax-table)
+  "Set the syntax of CHAR to RAW-DESCRIPTOR (syntax table value) in the current syntax table or optional SYNTAX-TABLE.
+Return the RAW-DESCRIPTOR.  Use the `syntax-after' function to
+retrieve the raw descriptor for a buffer position.
+
+Similar to modify-syntax-entry but uses a raw descriptor
+previously extracted from a syntax table to set the value rather
+than a string.
+
+Syntax tables are char-tables whose values are encoded as raw
+descriptors."
+  (aset (or syntax-table (syntax-table)) char raw-descriptor))
+
+(defun hypb:straight-package-plist (pkg-string)
+  "Return a property list of package-name, package-download-source and pckage-version for PKG-STRING, else return nil.
+This is for the straight.el package manager."
+  (when (fboundp #'straight-bug-report-package-info)
+    (car (delq nil (mapcar (lambda (pkg-plist)
+			     (when (equal (plist-get pkg-plist :package) pkg-string) pkg-plist))
+			   (straight-bug-report-package-info))))))
+
+(defun hypb:string-count-matches (regexp str &optional start end)
+  "Count occurrences of REGEXP in STR, limited to optional START and END positions.
+
+START is inclusive and indexed from 0; END is exclusive.
+
+This function starts looking for the next match from the end of the
+previous match.  Hence, it ignores matches that overlap a previously
+found match."
+  (let ((str-len (length str))
+	(count 0)
+	substr)
+    (when (and start (or (>= start str-len) (< start 0)))
+      (error "(hypb:string-count-matches): 'start' (%d) must be >= 0 and < str length (%d)"
+	     start str-len))
+    (when (and end (or (> end str-len) (< end 0)))
+      (error "(hypb:string-count-matches): 'end' (%d) must be >= 0 and <= str length (%d)"
+	     end str-len))
+    (setq start (or start 0)
+	  end (or end str-len)
+	  substr (substring str start end)
+	  end (- end start)
+	  start 0)
+    (while (and (< start str-len)
+		(string-match regexp substr start))
+      (setq count (1+ count)
+	    start (match-end 0)))
+    count))
+
 (defun hypb:supercite-p ()
   "Return non-nil iff the Emacs add-on supercite package is in use."
   (let (hook-val)
-    (if (memq t (mapcar
-		 (lambda (hook-var)
-		   (and (boundp hook-var)
-			(progn (setq hook-val (symbol-value hook-var))
-			       (cond ((listp hook-val)
-				      (if (memq 'sc-cite-original hook-val)
+    (when (memq t (mapcar
+		   (lambda (hook-var)
+		     (and (boundp hook-var)
+			  (progn (setq hook-val (symbol-value hook-var))
+			         (cond ((listp hook-val)
+				        (when (memq 'sc-cite-original hook-val)
 					  t))
-				     ((eq hook-val 'sc-cite-original))))))
-		 '(mail-citation-hook mail-yank-hooks)))
-	t)))
+				       ((eq hook-val 'sc-cite-original))))))
+		   '(mail-citation-hook mail-yank-hooks)))
+      t)))
 
 (defun hypb:toggle-isearch-invisible (&optional arg)
   "Toggle interactive invisible searching on or off.
@@ -658,6 +839,40 @@ in the list, even if it is not active.  If MINIBUFFER-FLAG is neither t
 nor nil it means to not count the minibuffer window even if it is active."
   (window-list nil minibuffer-flag))
 
+(defmacro hypb:with-suppressed-warnings (warnings &rest body)
+  "Like `progn', but prevents compiler WARNINGS in BODY.
+
+Defined here for elpa build compatibility which uses Emacs 26 and
+does not include `with-suppressed-warnings'.
+
+WARNINGS is an associative list where the first element of each
+item is a warning type, and the rest of the elements in each item
+are symbols they apply to.  For instance, if you want to suppress
+byte compilation warnings about the two obsolete functions `foo'
+and `bar', as well as the function `zot' being called with the
+wrong number of parameters, say
+
+\(with-suppressed-warnings ((obsolete foo bar)
+                           (callargs zot))
+  (foo (bar))
+  (zot 1 2))
+
+The warnings that can be suppressed are a subset of the warnings
+in `byte-compile-warning-types'; see the variable
+`byte-compile-warnings' for a fuller explanation of the warning
+types.  The types that can be suppressed with this macro are
+`free-vars', `callargs', `redefine', `obsolete',
+`interactive-only', `lexical', `mapcar', `constants' and
+`suspicious'.
+
+For the `mapcar' case, only the `mapcar' function can be used in
+the symbol list.  For `suspicious', only `set-buffer' can be used."
+
+  (declare (debug (sexp &optional body)) (indent 1))
+  (if (fboundp 'with-suppressed-warnings)
+      `(with-suppressed-warnings ,warnings ,@body)
+    `(with-no-warnings ,@body)))
+
 ;;; ************************************************************************
 ;;; About Hyperbole Setup
 ;;; ************************************************************************
@@ -675,12 +890,17 @@ nor nil it means to not count the minibuffer window even if it is active."
 
 ;;;###autoload
 (defun hypb:display-file-with-logo (file)
-  "Display a text FILE in help mode with the Hyperbole banner prepended."
+  "Display a text FILE in help mode with the Hyperbole banner prepended.
+If FILE is not an absolute path, expand it relative to `hyperb:dir'."
+  (unless (stringp file)
+    (error "(hypb:display-file-with-logo): 'file' must be a string, not '%s'" file))
+  (unless (file-name-absolute-p file)
+    (setq file (expand-file-name file hyperb:dir)))
   (let ((existing-buf (when (stringp file) (get-file-buffer file))))
     ;; A stub for this function is defined in hversion.el when not running in InfoDock.
     (id-browse-file file)
     (unless existing-buf
-      (hypb:display-file-with-logo-emacs file)
+      (hypb:insert-hyperbole-banner)
       (goto-char (point-min))
       (skip-syntax-forward "-")
       (set-window-start (selected-window) 1)
@@ -699,7 +919,7 @@ nor nil it means to not count the minibuffer window even if it is active."
 
 (defun hypb:constant-vector-symbol-replace
   (constant-vector sym-to-replace replace-with-sym)
-  ;; Replace symbols within a byte-compiled constant vector.
+  "Replace symbols within a byte-compiled constant vector."
   (let ((i (length constant-vector))
 	constant)
     (while (>= (setq i (1- i)) 0)
@@ -711,15 +931,15 @@ nor nil it means to not count the minibuffer window even if it is active."
 	     (hypb:function-symbol-replace
 	      constant sym-to-replace replace-with-sym))))))
 
-(defun hypb:display-file-with-logo-emacs (&optional file)
+(defun hypb:insert-hyperbole-banner ()
   "Display an optional text FILE with the Hyperbole banner prepended.
 Without file, the banner is prepended to the current buffer."
   (let ((hyperbole-banner-path (expand-file-name "hyperbole-banner.png" hyperb:dir)))
-    (if (not (file-readable-p hyperbole-banner-path))
-	(setq hyperbole-banner-path (if (fboundp 'locate-data-file)
-					(locate-data-file "hyperbole-banner.png")
-				      (expand-file-name "hyperbole-banner.png"
-							data-directory))))
+    (unless (file-readable-p hyperbole-banner-path)
+      (setq hyperbole-banner-path (if (fboundp 'locate-data-file)
+				      (locate-data-file "hyperbole-banner.png")
+				    (expand-file-name "hyperbole-banner.png"
+						      data-directory))))
     (if (or (not (fboundp 'create-image))
 	    (not (display-graphic-p))
 	    (let ((button (next-button (point-min))))
@@ -757,8 +977,8 @@ Without file, the banner is prepended to the current buffer."
 	 (error "(hypb:oct-to-int): Bad octal number: %s" oct-str))
     (mapconcat (lambda (o)
 		 (setq dec-num (+ (* dec-num 8)
-				  (if (and (>= o ?0) (<= o ?7))
-				      (- o ?0)))))
+				  (when (and (>= o ?0) (<= o ?7))
+				    (- o ?0)))))
 	       oct-str "")
     dec-num))
 
