@@ -1,10 +1,11 @@
-;;; hyrolo-logic.el --- Logic functions for GNU Hyperbole Rolo files
+;;; hyrolo-logic.el --- Logic functions for GNU Hyperbole Rolo files  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    13-Jun-89 at 22:57:33
+;; Last-Mod:     10-Apr-22 at 09:53:39 by Bob Weiner
 ;;
-;; Copyright (C) 1989-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1989-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -26,6 +27,9 @@
 ;;       might be:
 ;;        (and (or (not time card) (xor "french balloons" spanish)) teacher pet)
 ;;
+;;       By default, only sub-entries with matches are shown, not entire
+;;       hierarchies of entries, for more intuitive results.  Use a prefix argument
+;;
 ;;       Either double quotes or parentheses may be used to group multiple
 ;;       words as a single argument.
 ;;
@@ -67,8 +71,16 @@
 ;;; ************************************************************************
 
 ;;;###autoload
-(defun hyrolo-fgrep-logical (expr)
-  "Display rolo entries matching EXPR which may contain prefix logical operators.
+(defun hyrolo-fgrep-logical (expr &optional count-only include-sub-entries no-sub-entries-out)
+  "Display rolo entries matching EXPR, a string, which may contain prefix logical operators.
+If optional COUNT-ONLY is non-nil, don't display entries, return
+count of matching entries only.  If optional INCLUDE-SUB-ENTRIES
+flag is non-nil, SEXP will be applied across all sub-entries at
+once.  Default is to apply SEXP to each entry and sub-entry
+separately.  Entries are displayed with all of their sub-entries
+unless INCLUDE-SUB-ENTRIES is nil and optional NO-SUB-ENTRIES-OUT
+flag is non-nil.
+
 A complex example of EXPR might be:
   (and (or (not time card) (xor (and french balloons) spanish)) teacher pet)
 which means:
@@ -80,7 +92,9 @@ which means:
 
 Either double quotes or parentheses may be used to group multiple words as a
 single argument."
-  (interactive "sLogical rolo search: ")
+  (interactive "sLogical rolo search: \nP\nP")
+  (when (called-interactively-p 'any)
+    (setq no-sub-entries-out (not no-sub-entries-out)))
   (let* ((case-fold-search t)
 	 (total-matches))
     (if (not (string-match "\(\\(and\\|or\\|xor\\|not\\)\\>" expr))
@@ -105,11 +119,12 @@ single argument."
 	  (setq saved-expr expr)))
       (setq expr (hypb:replace-match-string
 		  "{\\([^{}]+\\)}" expr "\"\\1\"" nil))
-      (setq expr (hypb:replace-match-string "\(| " expr "\(hyrolo-or " t))
-      (setq expr (hypb:replace-match-string "\(@ " expr "\(hyrolo-xor " t))
-      (setq expr (hypb:replace-match-string "\(! " expr "\(hyrolo-not " t))
-      (setq expr (hypb:replace-match-string "\(& " expr "\(hyrolo-and " t))
-      (setq expr (format "(hyrolo-logic (quote %s) nil nil t)" expr))
+      (setq expr (hypb:replace-match-string "\(| " expr "\(hyrolo-or start end  " t))
+      (setq expr (hypb:replace-match-string "\(@ " expr "\(hyrolo-xor start end " t))
+      (setq expr (hypb:replace-match-string "\(! " expr "\(hyrolo-not start end " t))
+      (setq expr (hypb:replace-match-string "\(& " expr "\(hyrolo-and start end " t))
+      (setq expr (format "(hyrolo-logic (quote %s) nil %s %s %s)"
+			 expr count-only include-sub-entries no-sub-entries-out))
       (setq total-matches (eval (read expr))))
     (if (called-interactively-p 'interactive)
 	(message "%s matching entr%s found in rolo."
@@ -117,48 +132,43 @@ single argument."
 		 (if (= total-matches 1) "y" "ies")))
     total-matches))
 
-(defun hyrolo-logic (sexp &optional in-bufs count-only include-sub-entries
-		     no-sub-entries-out)
+(defun hyrolo-logic (sexp &optional in-bufs count-only include-sub-entries no-sub-entries-out)
   "Apply SEXP to all entries in optional IN-BUFS, display entries where SEXP is non-nil.
 If IN-BUFS is nil, `hyrolo-file-list' is used.  If optional COUNT-ONLY is
 non-nil, don't display entries, return count of matching entries only.  If
-optional INCLUDE-SUB-ENTRIES flag is non-nil, SEXP will be applied across all
-sub-entries at once.  Default is to apply SEXP to each entry and sub-entry
-separately.  Entries are displayed with all of their sub-entries unless
-INCLUDE-SUB-ENTRIES is nil and optional NO-SUB-ENTRIES-OUT flag is non-nil.
-SEXP should use the free variables `start' and `end' which contain the limits
-of the region on which it should operate.  Returns number of evaluations of
-SEXP that matched entries."
-  (let ((obuf (current-buffer))
-	(display-buf (if count-only
-			 nil
+optional INCLUDE-SUB-ENTRIES flag is non-nil, apply SEXP across all sub-entries
+at once.  Default is to apply SEXP to each entry and sub-entry separately.
+Entries are displayed with all of their sub-entries unless INCLUDE-SUB-ENTRIES
+is nil and optional NO-SUB-ENTRIES-OUT flag is non-nil.  SEXP should utilize the
+free variables `start' and `end' as the region on which to operate.
+Return the number of evaluations of SEXP that match entries."
+  (let* ((display-buf (unless count-only
 		       (prog1 (set-buffer (get-buffer-create hyrolo-display-buffer))
 			 (setq buffer-read-only nil)
-			 (erase-buffer)))))
-    (let ((result
-	   (mapcar
-	    (lambda (in-bufs)
-	      (hyrolo-map-logic sexp in-bufs count-only include-sub-entries
-				no-sub-entries-out))
-	    (cond ((null in-bufs) hyrolo-file-list)
-		  ((listp in-bufs) in-bufs)
-		  ((list in-bufs))))))
-      (let ((total-matches (apply '+ result)))
-	(unless (or count-only (= total-matches 0))
-	  (hyrolo-display-matches display-buf))
-	total-matches))))
+			 (erase-buffer))))
+	 (result
+	  (mapcar
+	   (lambda (in-bufs)
+	     (hyrolo-map-logic sexp in-bufs count-only include-sub-entries
+			       no-sub-entries-out))
+	   (cond ((null in-bufs) hyrolo-file-list)
+		 ((listp in-bufs) in-bufs)
+		 ((list in-bufs)))))
+	 (total-matches (apply '+ result)))
+    (unless (or count-only (= total-matches 0))
+      (hyrolo-display-matches display-buf))
+    total-matches))
 
 (defun hyrolo-map-logic (sexp hyrolo-buf &optional count-only
-			    include-sub-entries no-sub-entries-out)
+			 include-sub-entries _no-sub-entries-out)
   "Apply logical SEXP to each entry in HYROLO-BUF and write out matching entries to `hyrolo-display-buffer'.
 If optional COUNT-ONLY is non-nil, don't display entries, return count of
-matching entries only.  If optional INCLUDE-SUB-ENTRIES flag is non-nil, SEXP
-will be applied across all sub-entries at once.  Default is to apply SEXP to
-each entry and sub-entry separately.  Entries are displayed with all of their
-sub-entries unless INCLUDE-SUB-ENTRIES is nil and optional NO-SUB-ENTRIES-OUT
-flag is non-nil.  SEXP should use the free variables `start' and `end' which
-contain the limits of the region on which it should operate.  Returns number
-of applications of SEXP that matched entries."
+matching entries only.  If optional INCLUDE-SUB-ENTRIES flag is non-nil, apply
+SEXP across all sub-entries at once.  Default is to apply SEXP to each entry
+and sub-entry separately.  Entries are displayed with all of their sub-entries
+unless INCLUDE-SUB-ENTRIES is nil and optional NO-SUB-ENTRIES-OUT flag is non-nil.
+SEXP should utilize the free variables `start' and `end' as the region on which
+to operate.  Return the number of evaluations of SEXP that match entries."
   (setq hyrolo-buf (or (get-buffer hyrolo-buf) hyrolo-buf))
   (if (or (bufferp hyrolo-buf)
 	  (if (file-exists-p hyrolo-buf)
@@ -186,35 +196,30 @@ of applications of SEXP that matched entries."
 			next-entry-exists nil
 			curr-entry-level-len (length (match-string-no-properties hyrolo-entry-group-number))
 			end (hyrolo-to-entry-end include-sub-entries curr-entry-level-len))
-		  (let ((result (eval sexp)))
-		    (or count-only
-			(and result (= num-found 0) hdr-pos
-			     (let* ((src (or (buffer-file-name hyrolo-buf)
-					     hyrolo-buf))
-				    (src-line
-				     (format
-				      (concat (if (boundp 'hbut:source-prefix)
-						  hbut:source-prefix
-						"@loc> ")
-					      "%s")
-				      (prin1-to-string src))))
-			       (set-buffer display-buf)
-			       (goto-char (point-max))
-			       (if hdr-pos
-				   (progn
-				     (insert-buffer-substring
-				      hyrolo-buf (car hdr-pos) (cdr hdr-pos))
-				     (insert src-line "\n\n"))
-				 (insert (format hyrolo-hdr-format src-line)))
-			       (set-buffer hyrolo-buf))))
+		  (let ((result (eval sexp `((start . ,start) (end . ,end)))))
+		    (unless count-only
+		      (and result (= num-found 0) hdr-pos
+			   (let* ((src (or (buffer-file-name hyrolo-buf)
+					   hyrolo-buf))
+				  (src-line
+				   (format
+				    (concat (if (boundp 'hbut:source-prefix)
+						hbut:source-prefix
+					      "@loc> ")
+					    "%s")
+				    (prin1-to-string src))))
+			     (set-buffer display-buf)
+			     (goto-char (point-max))
+			     (if hdr-pos
+				 (progn
+				   (insert-buffer-substring
+				    hyrolo-buf (car hdr-pos) (cdr hdr-pos))
+				   (insert src-line "\n\n"))
+			       (insert (format hyrolo-hdr-format src-line)))
+			     (set-buffer hyrolo-buf))))
 		    (if result
 			(progn (goto-char end)
-			       (setq num-found (1+ num-found)
-				     end (if (or include-sub-entries
-						 no-sub-entries-out)
-					     end
-					   (goto-char (hyrolo-to-entry-end
-						       t curr-entry-level-len))))
+			       (setq num-found (1+ num-found))
 			       (or count-only
 				   (append-to-buffer display-buf start end)))
 		      (goto-char end-entry-hdr)))))))
@@ -229,7 +234,7 @@ of applications of SEXP that matched entries."
 ;; Do NOT call the following functions directly.
 ;; Send them as parts of an expression to `hyrolo-logic'.
 
-(defun hyrolo-not (&rest pat-list)
+(defun hyrolo-not (start end &rest pat-list)
   "Logical <not> rolo entry filter.  PAT-LIST is a list of pattern elements.
 Each element may be t, nil, or a string."
   (let ((pat))
@@ -241,7 +246,7 @@ Each element may be t, nil, or a string."
       (setq pat-list (cdr pat-list)))
     (not pat-list)))
 
-(defun hyrolo-or (&rest pat-list)
+(defun hyrolo-or (start end &rest pat-list)
   "Logical <or> rolo entry filter.  PAT-LIST is a list of pattern elements.
 Each element may be t, nil, or a string."
   (if (memq t pat-list)
@@ -255,7 +260,7 @@ Each element may be t, nil, or a string."
 	(setq pat-list (cdr pat-list)))
       (if pat-list t nil))))
 
-(defun hyrolo-xor (&rest pat-list)
+(defun hyrolo-xor (start end &rest pat-list)
   "Logical <xor> rolo entry filter.  PAT-LIST is a list of pattern elements.
 Each element may be t, nil, or a string."
   (let ((pat)
@@ -271,7 +276,7 @@ Each element may be t, nil, or a string."
       (setq pat-list (cdr pat-list)))
     (= matches 1)))
 
-(defun hyrolo-and (&rest pat-list)
+(defun hyrolo-and (start end &rest pat-list)
   "Logical <and> rolo entry filter.  PAT-LIST is a list of pattern elements.
 Each element may be t, nil, or a string."
   (unless (memq nil pat-list)
@@ -286,9 +291,9 @@ Each element may be t, nil, or a string."
 
 ;; Work with regular expression patterns rather than strings
 
-(defun hyrolo-r-not (&rest pat-list)
+(defun hyrolo-r-not (start end &rest pat-list)
   "Logical <not> rolo entry filter.  PAT-LIST is a list of pattern elements.
-Each element may be t, nil, or a string."
+Each element may be t, nil, or a regular expression."
   (let ((pat))
     (while (and pat-list
 		(or (null (setq pat (car pat-list)))
@@ -298,9 +303,9 @@ Each element may be t, nil, or a string."
       (setq pat-list (cdr pat-list)))
     (not pat-list)))
 
-(defun hyrolo-r-or (&rest pat-list)
+(defun hyrolo-r-or (start end &rest pat-list)
   "Logical <or> rolo entry filter.  PAT-LIST is a list of pattern elements.
-Each element may be t, nil, or a string."
+Each element may be t, nil, or a regular expression."
   (if (memq t pat-list)
       t
     (let ((pat))
@@ -312,9 +317,9 @@ Each element may be t, nil, or a string."
 	(setq pat-list (cdr pat-list)))
       (if pat-list t nil))))
 
-(defun hyrolo-r-xor (&rest pat-list)
+(defun hyrolo-r-xor (start end &rest pat-list)
   "Logical <xor> rolo entry filter.  PAT-LIST is a list of pattern elements.
-Each element may be t, nil, or a string."
+Each element may be t, nil, or a regular expression."
   (let ((pat)
 	(matches 0))
     (while (and pat-list
@@ -328,9 +333,9 @@ Each element may be t, nil, or a string."
       (setq pat-list (cdr pat-list)))
     (= matches 1)))
 
-(defun hyrolo-r-and (&rest pat-list)
+(defun hyrolo-r-and (start end &rest pat-list)
   "Logical <and> rolo entry filter.  PAT-LIST is a list of pattern elements.
-Each element may be t, nil, or a string."
+Each element may be t, nil, or a regular expression."
   (unless (memq nil pat-list)
     (let ((pat))
       (while (and pat-list
