@@ -1,10 +1,11 @@
-;;; kfile.el --- Save and restore koutlines from files
+;;; kfile.el --- Save and restore koutlines from files  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    10/31/93
+;; Last-Mod:     17-Apr-22 at 12:17:44 by Bob Weiner
 ;;
-;; Copyright (C) 1993-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1993-2021  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -59,7 +60,8 @@ Return the new kview."
     ;; again.
     (unless (kview:is-p kview)
       (kfile:read buffer existing-file))
-    (or (eq major-mode 'kotl-mode) (kotl-mode))
+    (unless (derived-mode-p 'kotl-mode)
+      (kotl-mode))
     kview))
 
 ;;;###autoload
@@ -101,22 +103,22 @@ Return the new kview."
   "Create a new koutline file attached to BUFFER, with a single empty level 1 kotl cell.
 Return file's kview."
   (or buffer (setq buffer (current-buffer)))
-  (if (not (bufferp buffer))
-      (error "(kfile:create): Invalid buffer argument, %s" buffer))
+  (unless (bufferp buffer)
+    (error "(kfile:create): Invalid buffer argument, %s" buffer))
   (set-buffer buffer)
-  (if buffer-read-only
-      (error "(kfile:create): %s is read-only" buffer))
+  (when buffer-read-only
+    (error "(kfile:create): %s is read-only" buffer))
   (widen)
 
   (let ((empty-p (zerop (buffer-size)))
 	import-from view standard-output)
 
-    (if (not empty-p)
-	;; This is a foreign file whose elements must be converted into
-	;; koutline cells.
-	(progn (setq import-from (kimport:copy-and-set-buffer buffer))
-	       (set-buffer buffer)
-	       (erase-buffer))) ;; We copied the contents to `import-from'.
+    (unless empty-p
+      ;; This is a foreign file whose elements must be converted into
+      ;; koutline cells.
+      (setq import-from (kimport:copy-and-set-buffer buffer))
+      (set-buffer buffer)
+      (erase-buffer)) ;; We copied the contents to `import-from'.
 
     (setq view (kview:create (buffer-name buffer))
 	  standard-output (current-buffer))
@@ -148,8 +150,8 @@ Return file's kview."
       (kimport:file import-from (current-buffer))
       ;; If import buffer name starts with a space, kill it, as it is no
       ;; longer needed.
-      (if (eq ?\ (aref (buffer-name import-from) 0))
-	  (kill-buffer import-from)))
+      (when (eq ?\ (aref (buffer-name import-from) 0))
+	(kill-buffer import-from)))
 
     view))
 
@@ -199,8 +201,8 @@ Return the new view."
     ;; kcell-list is a depth-first list of kcells to be attached to the cell
     ;; contents within the kview down below.
     (setq kcell-list (kfile:build-structure-v2 kotl-structure cell-data)
-	  view (kview:create (buffer-name buffer) cell-count label-type
-				 level-indent label-separator label-min-width))
+	  view (kview:create (buffer-name buffer) cell-count nil label-type
+			     level-indent label-separator label-min-width))
     ;;
     (kfile:narrow-to-kcells)
     (goto-char (point-min))
@@ -219,7 +221,7 @@ Return the new view."
 If V3-FLAG is true, read as a version-3 buffer."
   (let ((standard-input buffer)
 	cell-count label-type label-min-width label-separator
-	level-indent cell-data view)
+	level-indent cell-data top-cell-attributes view)
     (widen)
     (goto-char (point-min))
     ;; Skip past cell contents here.
@@ -235,8 +237,9 @@ If V3-FLAG is true, read as a version-3 buffer."
 	  level-indent (read)
 	  cell-data (read))
     ;;
-    (setq view (kview:create (buffer-name buffer) cell-count label-type
-			     level-indent label-separator label-min-width))
+    (setq top-cell-attributes (aref (aref cell-data 0) 1)
+	  view (kview:create (buffer-name buffer) cell-count top-cell-attributes
+			     label-type level-indent label-separator label-min-width))
     ;;
     (kfile:narrow-to-kcells)
     (goto-char (point-min))
@@ -255,7 +258,7 @@ If V3-FLAG is true, read as a version-3 buffer."
 Leave outline file expanded with structure data showing unless optional
 VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
   (let* ((top (kview:top-cell kview))
-	 (file (kcell:get-attr top 'file))
+	 (file buffer-file-name)
 	 (label-type (kview:label-type kview))
 	 (label-min-width (kview:label-min-width kview))
 	 (label-separator (kview:label-separator kview))
@@ -278,23 +281,23 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
       ;; Prepare cell data for saving.
       (kfile:narrow-to-kcells)
       (kview:map-tree
-        (lambda (view)
+        (lambda (_view)
 	  (setq cell (kcell-view:cell))
 	  (aset kcell-data
 		kcell-num
-		(kcell-data:create cell))
+		(kcell-data:create cell (kcell-view:idstamp-integer)))
 	  (setq kcell-num (1+ kcell-num)))
 	kview t)
       ;; Save top cell, 0, last since above loop may increment the total
       ;; number of cells counter stored in it, if any invalid cells are
       ;; encountered.
-      (aset kcell-data 0 (kcell-data:create top))
+      (aset kcell-data 0 (kcell-data:create top 0))
       (setq id-counter (kcell:get-attr top 'id-counter))
       ;;
       (widen)
       (goto-char (point-min))
-      (if (search-forward "\n\^_\n" nil t)
-	  (delete-region (point-min) (match-end 0)))
+      (when (search-forward "\n\^_\n" nil t)
+	(delete-region (point-min) (match-end 0)))
       (princ ";; -*- Mode: kotl -*- \n")
       (prin1 kfile:version)
       (princ " ;; file-format\n\^_\n")
@@ -322,7 +325,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
       ;; buffer is narrowed here, only the narrowed portion will be saved to
       ;; the file.  Narrow as an option since saving only the portion of the
       ;; file visible in a view may be useful in some situations.
-      (if visible-only-p (kfile:narrow-to-kcells))
+      (when visible-only-p (kfile:narrow-to-kcells))
       ;;
       ;; Return point to its original position as given by the opoint marker.
       (goto-char opoint)
@@ -333,13 +336,12 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
 (defun kfile:write (file)
   "Write current outline to FILE."
   (interactive "FWrite outline file: ")
-  (if (or (null file) (string-equal file ""))
-      (error "(kfile:write): Invalid file name, \"%s\"" file))
+  (when (or (null file) (string-equal file ""))
+    (error "(kfile:write): Invalid file name, \"%s\"" file))
   ;; If arg is just a directory, use same file name, but in that directory.
-  (if (and (file-directory-p file) buffer-file-name)
-      (setq file (concat (file-name-as-directory file)
-			 (file-name-nondirectory buffer-file-name))))
-  (kcell:set-attr (kview:top-cell kview) 'file file)
+  (when (and (file-directory-p file) buffer-file-name)
+    (setq file (concat (file-name-as-directory file)
+		       (file-name-nondirectory buffer-file-name))))
   (set-visited-file-name file)
   ;; Set-visited-file-name clears local-write-file-hooks that we use to save
   ;; koutlines properly, so reinitialize local variables.
@@ -356,7 +358,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
 
 (defun kfile:build-structure-v2 (kotl-structure cell-data)
   "Build cell list from the KOTL-STRUCTURE and its CELL-DATA.
-Assumes all arguments are valid.  CELL-DATA is a vector of cell fields read
+Assume all arguments are valid.  CELL-DATA is a vector of cell fields read
 from a koutline file.
 
 Return list of outline cells in depth first order.  Invisible top cell is not
@@ -379,16 +381,17 @@ included in the list."
 	     ((equal item 0) nil)
 	     (t (setq cell (kcell-data:to-kcell-v2 (aref cell-data item))
 		      cell-list (cons cell cell-list)
-		      sibling-p t)
-		)))
+		      sibling-p t))))
      kotl-structure)
     (nreverse cell-list)))
 
-(defun kfile:insert-attributes-v2 (kview kcell-list)
+(defun kfile:insert-attributes-v2 (_kview kcell-list)
   "Set cell attributes within KVIEW for each element in KCELL-LIST.
-Assumes all cell contents are already in kview and that no cells are
+Assume all cell contents are already in kview and that no cells are
 hidden."
-  (let (buffer-read-only)
+  (let (buffer-read-only
+	idstamp
+	kcell-data)
     (while
 	(progn
 	  (skip-chars-forward "\n")
@@ -396,53 +399,62 @@ hidden."
 	  ;; Here we search past the cell identifier
 	  ;; for the location at which to place cell properties.
 	  ;; Be sure not to skip past a period which may terminate the label.
-	  (if (re-search-forward "[A-Za-z0-9]\\(\\.?[A-Za-z0-9]\\)*" nil t)
-	      (progn
-		(kproperty:set 'kcell (car kcell-list))
-		(setq kcell-list (cdr kcell-list))))
+	  (when (re-search-forward "[A-Za-z0-9]\\(\\.?[A-Za-z0-9]\\)*" nil t)
+	    (setq kcell-data (car kcell-list)
+		  ;; Repair invalid idstamps on the fly.
+		  idstamp (if (vectorp kcell-data)
+			      (or (kcell-data:idstamp kcell-data) (kview:id-increment kview))
+			    (kview:id-increment kview)))
+	    (kproperty:set 'idstamp idstamp)
+	    (kproperty:set 'kcell (car kcell-list))
+	    (setq kcell-list (cdr kcell-list)))
 	  (search-forward "\n\n" nil t)))))
 
-(defun kfile:insert-attributes-v3 (kview kcell-vector)
-  "Set cell attributes within KVIEW for each element in KCELL-VECTOR.
-Assumes all cell contents are already in kview and that no cells are
+(defun kfile:insert-attributes-v3 (_kview kcell-vector)
+  "Set cell attributes within _KVIEW for each element in KCELL-VECTOR.
+Assume all cell contents are already in kview and that no cells are
 hidden."
   (let ((kcell-num 1)
-	(buffer-read-only))
+	buffer-read-only
+	idstamp
+	kcell-data)
     (while
 	(progn
 	  (skip-chars-forward "\n")
-	  ;; !!! Won't work if label-type is 'no.
+	  ;; !! Todo: Won't work if label-type is 'no.
 	  ;; Here we search past the cell identifier
 	  ;; for the location at which to place cell properties.
 	  ;; Be sure not to skip past a period which may terminate the label.
-	  (if (re-search-forward "[A-Za-z0-9]\\(\\.?[A-Za-z0-9]\\)*" nil t)
-	      (progn
-		(kproperty:set 'kcell
-			       (kcell-data:to-kcell-v3
-				(aref kcell-vector kcell-num)))
-		(setq kcell-num (1+ kcell-num))))
+	  (when (re-search-forward "[A-Za-z0-9]\\(\\.?[A-Za-z0-9]\\)*" nil t)
+	    (setq kcell-data (aref kcell-vector kcell-num)
+		  ;; Repair invalid idstamps on the fly.
+		  idstamp (if (vectorp kcell-data)
+			      (or (kcell-data:idstamp kcell-data) (kview:id-increment kview))
+			    (kview:id-increment kview)))
+	    (kproperty:set 'idstamp idstamp)
+	    (kproperty:set 'kcell (kcell-data:to-kcell-v3 kcell-data))
+	    (setq kcell-num (1+ kcell-num)))
 	  (search-forward "\n\n" nil t)))))
 
 (defun kfile:narrow-to-kcells ()
   "Narrow kotl file to kcell section only."
   (interactive)
-  (if (kview:is-p kview)
-      (let ((start-text) (end-text))
-	(save-excursion
-	  (widen)
-	  (goto-char (point-min))
-	  ;; Skip to start of kcells.
-	  (if (search-forward "\n\^_" nil t)
-	      (setq start-text (1+ (match-end 0))))
-	  ;; Skip past end of kcells.
-	  (if (and start-text (search-forward "\n\^_" nil t))
-	      (setq end-text (1+ (match-beginning 0))))
-	  (if (and start-text end-text)
-	      (progn (narrow-to-region start-text end-text)
-		     (goto-char (point-min)))
-	    (error
-	     "(kfile:narrow-to-kcells): Cannot find start or end of kcells"))
-	  ))))
+  (when (kview:is-p kview)
+    (let ((start-text) (end-text))
+      (save-excursion
+	(widen)
+	(goto-char (point-min))
+	;; Skip to start of kcells.
+	(when (search-forward "\n\^_" nil t)
+	  (setq start-text (1+ (match-end 0))))
+	;; Skip past end of kcells.
+	(when (and start-text (search-forward "\n\^_" nil t))
+	  (setq end-text (1+ (match-beginning 0))))
+	(if (and start-text end-text)
+	    (progn (narrow-to-region start-text end-text)
+		   (goto-char (point-min)))
+	  (error
+	   "(kfile:narrow-to-kcells): Cannot find start or end of kcells"))))))
 
 (defun kfile:print-to-string (object)
   "Return a string containing OBJECT, any Lisp object, in pretty-printed form.
@@ -490,8 +502,8 @@ handle, whenever this is possible."
 		(delete-region
 		 (point)
 		 (progn (skip-chars-forward " \t") (point)))
-		(if (not (eq ?' (char-after (1- (point)))))
-		    (insert ?\n)))
+		(when (not (eq ?' (char-after (1- (point)))))
+		  (insert ?\n)))
 	       ((condition-case ()
 		    (prog1 t (up-list 1))
 		  (error nil))
@@ -501,8 +513,8 @@ handle, whenever this is possible."
 		(delete-region
 		 (point)
 		 (progn (skip-chars-forward " \t") (point)))
-		(if (not (eq ?' (char-after (1- (point)))))
-		    (insert ?\n)))
+		(when (not (eq ?' (char-after (1- (point)))))
+		  (insert ?\n)))
 	       (t (goto-char (point-max)))))
 	    (goto-char (point-min))
 	    (indent-sexp)
@@ -521,8 +533,8 @@ Output stream is STREAM, or value of `standard-output' (which see)."
   (let ((filename))
     (while (not filename)
       (setq filename (read-file-name prompt nil nil existing-p))
-      (if (or (null filename) (equal filename ""))
-	  (progn (beep) (setq filename nil))))
+      (when (or (null filename) (equal filename ""))
+	(beep) (setq filename nil)))
     filename))
 
 (provide 'kfile)
