@@ -1,10 +1,11 @@
-;;; kimport.el --- Convert and insert other outline file formats into koutlines
+;;; kimport.el --- Convert and insert other outline file formats into koutlines  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    15-Nov-93 at 11:57:05
+;; Last-Mod:     11-Apr-22 at 23:51:03 by Bob Weiner
 ;;
-;; Copyright (C) 1993-2017  Free Software Foundation, Inc.
+;; Copyright (C) 1993-2021  Free Software Foundation, Inc.
 ;; See the "../HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -18,6 +19,7 @@
 
 (require 'kfile)
 (require 'hyrolo)
+(require 'kotl-mode)
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -48,19 +50,22 @@ as the initial set of children of the current cell, if any.
 
 ;;;###autoload
 (defvar kimport:suffix-alist
-  '(("\\.otl$". kimport:star-outline)
+  '(("\\.org$" . kimport:star-outline)
+    ("\\.otl$" . kimport:star-outline)
     ("\\.aug$" . kimport:aug-post-outline))
   "Alist of (buffer-name-suffix-regexp . importation-function) elements.
 This determines the type of importation done on a file when `kimport:file' is
 called.  Each importation-function must take two arguments, a buffer/file to
-import and a buffer/file into which to insert the imported elements and a
-third optional argument, CHILDREN-P, which when non-nil means insert imported
-cells as the initial set of children of the current cell, if any.
+import and a buffer/file into which to insert the imported elements.
+A third optional argument, CHILDREN-P, may be given; when non-nil, insert
+imported cells as the initial set of children of the current cell, if any.
 
-   .otl  - imported as an Emacs outline whose entries begin with asterisks;
+   .org  - import format is an Emacs outline whose entries begin with asterisks;
+   .otl  - import format is an Emacs outline whose entries begin with asterisks;
    .kot
-   .kotl - imported as a structured koutline
-   .aug  - imported as an Augment post-numbered outline.")
+   .kotl - import format is a Koutline
+   .aug  - import format is an Augment post-numbered outline
+           (see https://dougengelbart.org/content/view/148/).")
 
 (defconst kimport:star-heading "^\\(\\*+\\)"
   "Regular expression matching a star outline heading with the number of stars given by groupoing 1.")
@@ -115,6 +120,7 @@ BUFFER may be a buffer or a buffer name."
   (insert-buffer-substring buffer)
   (kotl-mode:add-indent-to-region))
 
+;;;###autoload
 (defun kimport:insert-file (import-from children-p)
   "Insert each element in IMPORT-FROM as a separate cell in the current view.
 Insert as sibling cells following the current cell unless prefix arg,
@@ -142,6 +148,7 @@ Set mark after the inserted text."
     (push-mark (+ (point) (car (cdr tem)))))
   (kotl-mode:add-indent-to-region))
 
+;;;###autoload
 (defun kimport:insert-register (register &optional arg)
   "Insert contents of register REGISTER at point in current cell.
 REGISTER is a character naming the register to insert.
@@ -162,8 +169,9 @@ Interactively, second arg is non-nil if prefix ARG is supplied."
            (princ (marker-position val) (current-buffer)))
           (t
            (error "Register `%c' does not contain text" register))))
-  ;; Do want to activate the mark here.
-  (if (not arg) (exchange-point-and-mark)))
+  (unless arg
+    ;; Do want to activate the mark here.
+    (exchange-point-and-mark)))
 
 ;;; Augment right-side numbered files, blank line between cells
 ;;;
@@ -657,7 +665,7 @@ in IMPORT-FROM, used to show a running tally of the imported entries."
   count)
 
 (defun kimport:text-paragraphs (import-from output-to klabel
-			        output-level count total)
+			        output-level _count total)
   "Insert text paragraphs from IMPORT-FROM into existing OUTPUT-TO.
 First cell is inserted with KLABEL at OUTPUT-LEVEL, as the sibling of the
 previous cell, with the COUNT of inserted paragraphs starting at 0.  TOTAL is
