@@ -1,10 +1,11 @@
-;;; set.el --- General mathematical operators for unordered sets
+;;; set.el --- General mathematical operators for unordered sets  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    26-Sep-91 at 19:24:19
+;; Last-Mod:     12-Feb-22 at 10:42:19 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; Copyright (C) 1991-2021  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -69,11 +70,11 @@ Use (setq set (set:remove elt set)) to assure set is always properly modified."
 
 (defun set:combinations (set &optional arity)
   "Return all possible combinations (subsets) of SET including the empty set and the SET itself.
-Assumes SET is a valid set.  With optional ARITY, return only subsets with
+Assume SET is a valid set.  With optional ARITY, return only subsets with
 ARITY members."
   (cond ((null arity) 
 	 (setq arity 0)
-	 (cons nil (apply 'nconc (mapcar (lambda (elt) (setq arity (1+ arity)) (set:combinations set arity))
+	 (cons nil (apply 'nconc (mapcar (lambda (_elt) (setq arity (1+ arity)) (set:combinations set arity))
 					 set))))
 	((= arity 1) set)
 	((<= arity 0) '(nil))
@@ -126,7 +127,7 @@ Uses `set:equal-op' for comparison."
 
 (defun set:get (key set)
   "Return the value associated with KEY in SET or nil.
-Assumes elements of SET are of the form (key . value)."
+Assume elements of SET are of the form (key . value)."
   (cdr (car (let ((set:equal-op (lambda (key elt) (equal key (car elt)))))
 	      (set:member key set)))))
 
@@ -164,8 +165,8 @@ Uses `set:equal-op' for comparison.  See also `set:create'."
 Return set if modified, else nil.
 Use (setq set (set:replace elt set)) to assure set is always properly modified.
 
-Uses `set:equal-op' to match against KEY.  Assumes each element in the set
-has a car and a cdr."
+Use `set:equal-op' to match against KEY.  Assume each element in the set has a
+car and a cdr."
   (let ((elt-set (set:member key set)))
     (if elt-set
 	;; replace element
